@@ -15,6 +15,7 @@ pub fn rust_assembler(f: &mut Formatter, insts: &[dsl::Inst]) {
     generate_inst_display_impl(f, insts);
     generate_inst_encode_impl(f, insts);
     generate_inst_visit_impl(f, insts);
+    generate_inst_is_available_impl(f, insts);
     generate_inst_features_impl(f, insts);
 
     // Generate per-instruction structs.
@@ -27,8 +28,8 @@ pub fn rust_assembler(f: &mut Formatter, insts: &[dsl::Inst]) {
         f.empty_line();
     }
 
-    // Generate the `Feature` enum.
-    dsl::Feature::generate_enum(f);
+    // Generate the `Feature` trait.
+    dsl::Feature::generate_macro(f);
 }
 
 /// `enum Inst { ... }`
@@ -117,10 +118,27 @@ fn generate_inst_visit_impl(f: &mut Formatter, insts: &[dsl::Inst]) {
     fmtln!(f, "}}");
 }
 
+/// `impl Inst { fn is_available... }`
+fn generate_inst_is_available_impl(f: &mut Formatter, insts: &[dsl::Inst]) {
+    f.add_block("impl<R: Registers> Inst<R>", |f| {
+        f.add_block(
+            "pub fn is_available(&self, f: &impl AvailableFeatures) -> bool",
+            |f| {
+                f.add_block("match self", |f| {
+                    for inst in insts {
+                        let variant_name = inst.name();
+                        fmtln!(f, "Self::{variant_name}(i) => i.is_available(f),");
+                    }
+                });
+            },
+        );
+    });
+}
+
 /// `impl Inst { fn features... }`
 fn generate_inst_features_impl(f: &mut Formatter, insts: &[dsl::Inst]) {
     f.add_block("impl<R: Registers> Inst<R>", |f| {
-        f.add_block("pub fn features(&self) -> Vec<Feature>", |f| {
+        f.add_block("pub fn features(&self) -> &'static Features", |f| {
             f.add_block("match self", |f| {
                 for inst in insts {
                     let variant_name = inst.name();
