@@ -14,10 +14,11 @@ impl dsl::Operand {
                     format!("Imm{bits}")
                 }
             }
-            al | ax | eax | rax | cl | dx | edx | rdx => {
+            al | ax | eax | rax | rbx | cl | rcx | dx | edx | rdx => {
                 let enc = match self.location {
                     al | ax | eax | rax => "{ gpr::enc::RAX }",
-                    cl => "{ gpr::enc::RCX }",
+                    rbx => "{ gpr::enc::RBX }",
+                    cl | rcx => "{ gpr::enc::RCX }",
                     dx | edx | rdx => "{ gpr::enc::RDX }",
                     _ => unreachable!(),
                 };
@@ -28,10 +29,11 @@ impl dsl::Operand {
             xmm1 | xmm2 | xmm3 => {
                 format!("Xmm<R::{mut_}Xmm>")
             }
-            xmm_m16 | xmm_m32 | xmm_m64 | xmm_m128 => {
+            xmm_m8 | xmm_m16 | xmm_m32 | xmm_m64 | xmm_m128 => {
                 format!("XmmMem<R::{mut_}Xmm, R::ReadGpr>")
             }
-            m8 | m16 | m32 | m64 => format!("Amode<R::ReadGpr>"),
+            m8 | m16 | m32 | m64 | m128 => format!("Amode<R::ReadGpr>"),
+            xmm0 => format!("Fixed<R::{mut_}Xmm, {{ xmm::enc::XMM0 }}>"),
         }
     }
 }
@@ -50,12 +52,20 @@ impl dsl::Location {
                     format!("self.{self}.to_string()")
                 }
             }
-            al | ax | eax | rax | cl | dx | edx | rdx | r8 | r16 | r32 | r32a | r32b | r64
-            | r64a | r64b | rm8 | rm16 | rm32 | rm64 => match self.generate_size() {
-                Some(size) => format!("self.{self}.to_string({size})"),
-                None => unreachable!(),
-            },
-            xmm_m16 | xmm_m32 | xmm_m64 | xmm1 | xmm2 | xmm3 | xmm_m128 | m8 | m16 | m32 | m64 => {
+            r8 | r16 | r32 | r32a | r32b | r64 | r64a | r64b | rm8 | rm16 | rm32 | rm64 => {
+                match self.generate_size() {
+                    Some(size) => format!("self.{self}.to_string({size})"),
+                    None => unreachable!(),
+                }
+            }
+            al | ax | eax | rax | rbx | cl | rcx | dx | edx | rdx | xmm0 => {
+                match self.generate_size() {
+                    Some(size) => format!("self.{self}.to_string(Some({size}))"),
+                    None => format!("self.{self}.to_string(None)"),
+                }
+            }
+            xmm1 | xmm2 | xmm3 | xmm_m8 | xmm_m16 | xmm_m32 | xmm_m64 | xmm_m128 | m8 | m16
+            | m32 | m64 | m128 => {
                 format!("self.{self}.to_string()")
             }
         }
@@ -70,13 +80,11 @@ impl dsl::Location {
             al | cl | r8 | rm8 => Some("Size::Byte"),
             ax | dx | r16 | rm16 => Some("Size::Word"),
             eax | edx | r32 | r32a | r32b | rm32 => Some("Size::Doubleword"),
-            rax | rdx | r64 | r64a | r64b | rm64 => Some("Size::Quadword"),
-            m8 | m16 | m32 | m64 => {
+            rax | rbx | rcx | rdx | r64 | r64a | r64b | rm64 => Some("Size::Quadword"),
+            m8 | m16 | m32 | m64 | m128 => {
                 panic!("no need to generate a size for memory-only access")
             }
-            xmm1 | xmm2 | xmm3 | xmm_m16 | xmm_m32 | xmm_m64 | xmm_m128 => {
-                panic!("no need to generate a size for XMM-sized access")
-            }
+            xmm1 | xmm2 | xmm3 | xmm_m8 | xmm_m16 | xmm_m32 | xmm_m64 | xmm_m128 | xmm0 => None,
         }
     }
 }
