@@ -52,14 +52,14 @@ pub fn list() -> Vec<Inst> {
         // manual: "when the destination operand is an XMM register, the source
         // operand is written to the low doubleword of the register, and the
         // register is zero-extended to 128 bits."
-        inst("movd", fmt("A", [w(xmm1), r(rm32)]), rex([0x66, 0x0F, 0x6E]).r(), _64b | compat | sse2),
-        inst("movq", fmt("A", [w(xmm1), r(rm64)]), rex([0x66, 0x0F, 0x6E]).r().w(), _64b | sse2),
-        inst("movd", fmt("B", [w(rm32), r(xmm2)]), rex([0x66, 0x0F, 0x7E]).r(), _64b | compat | sse2),
-        inst("movq", fmt("B", [w(rm64), r(xmm2)]), rex([0x66, 0x0F, 0x7E]).r().w(), _64b | sse2),
-        inst("vmovd", fmt("A", [w(xmm1), r(rm32)]), vex(L128)._66()._0f().w0().op(0x6E).r(), _64b | compat | avx),
-        inst("vmovq", fmt("A", [w(xmm1), r(rm64)]), vex(L128)._66()._0f().w1().op(0x6E).r(), _64b | avx),
-        inst("vmovd", fmt("B", [w(rm32), r(xmm2)]), vex(L128)._66()._0f().w0().op(0x7E).r(), _64b | compat | avx),
-        inst("vmovq", fmt("B", [w(rm64), r(xmm2)]), vex(L128)._66()._0f().w1().op(0x7E).r(), _64b | avx),
+        inst("movd", fmt("A", [w(xmm1), r(rm32)]), rex([0x66, 0x0F, 0x6E]).r(), (_64b | compat) & sse2),
+        inst("movq", fmt("A", [w(xmm1), r(rm64)]), rex([0x66, 0x0F, 0x6E]).r().w(), _64b & sse2),
+        inst("movd", fmt("B", [w(rm32), r(xmm2)]), rex([0x66, 0x0F, 0x7E]).r(), (_64b | compat) & sse2),
+        inst("movq", fmt("B", [w(rm64), r(xmm2)]), rex([0x66, 0x0F, 0x7E]).r().w(), _64b & sse2),
+        inst("vmovd", fmt("A", [w(xmm1), r(rm32)]), vex(L128)._66()._0f().w0().op(0x6E).r(), (_64b | compat) & avx),
+        inst("vmovq", fmt("A", [w(xmm1), r(rm64)]), vex(L128)._66()._0f().w1().op(0x6E).r(), _64b & avx),
+        inst("vmovd", fmt("B", [w(rm32), r(xmm2)]), vex(L128)._66()._0f().w0().op(0x7E).r(), (_64b | compat) & avx),
+        inst("vmovq", fmt("B", [w(rm64), r(xmm2)]), vex(L128)._66()._0f().w1().op(0x7E).r(), _64b & avx),
 
         // Move floating-point values to and from XMM locations. Some
         // memory-loading versions of `movs*` clear the upper bits of the XMM
@@ -119,31 +119,31 @@ pub fn list() -> Vec<Inst> {
         inst("vmovdqu", fmt("B", [w(xmm_m128), r(xmm1)]), vex(L128)._f3()._0f().op(0x7F).r(), compat | _64b | avx),
 
         // Move and extend packed integers to and from XMM locations with sign extension.
-        inst("pmovsxbw", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x20]).r(), _64b | compat | sse41).alt(avx, "vpmovsxbw_a"),
-        inst("pmovsxbd", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x21]).r(), _64b | compat | sse41).alt(avx, "vpmovsxbd_a"),
-        inst("pmovsxbq", fmt("A", [w(xmm1), r(xmm_m16)]), rex([0x66, 0x0F, 0x38, 0x22]).r(), _64b | compat | sse41).alt(avx, "vpmovsxbq_a"),
-        inst("pmovsxwd", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x23]).r(), _64b | compat | sse41).alt(avx, "vpmovsxwd_a"),
-        inst("pmovsxwq", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x24]).r(), _64b | compat | sse41).alt(avx, "vpmovsxwq_a"),
-        inst("pmovsxdq", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x25]).r(), _64b | compat | sse41).alt(avx, "vpmovsxdq_a"),
-        inst("vpmovsxbw", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x20).r(), _64b | compat | avx),
-        inst("vpmovsxbd", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x21).r(), _64b | compat | avx),
-        inst("vpmovsxbq", fmt("A", [w(xmm1), r(xmm_m16)]), vex(L128)._66()._0f38().op(0x22).r(), _64b | compat | avx),
-        inst("vpmovsxwd", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x23).r(), _64b | compat | avx),
-        inst("vpmovsxwq", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x24).r(), _64b | compat | avx),
-        inst("vpmovsxdq", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x25).r(), _64b | compat | avx),
+        inst("pmovsxbw", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x20]).r(), (_64b | compat) & sse41).alt(avx, "vpmovsxbw_a"),
+        inst("pmovsxbd", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x21]).r(), (_64b | compat) & sse41).alt(avx, "vpmovsxbd_a"),
+        inst("pmovsxbq", fmt("A", [w(xmm1), r(xmm_m16)]), rex([0x66, 0x0F, 0x38, 0x22]).r(), (_64b | compat) & sse41).alt(avx, "vpmovsxbq_a"),
+        inst("pmovsxwd", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x23]).r(), (_64b | compat) & sse41).alt(avx, "vpmovsxwd_a"),
+        inst("pmovsxwq", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x24]).r(), (_64b | compat) & sse41).alt(avx, "vpmovsxwq_a"),
+        inst("pmovsxdq", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x25]).r(), (_64b | compat) & sse41).alt(avx, "vpmovsxdq_a"),
+        inst("vpmovsxbw", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x20).r(), (_64b | compat) & avx),
+        inst("vpmovsxbd", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x21).r(), (_64b | compat) & avx),
+        inst("vpmovsxbq", fmt("A", [w(xmm1), r(xmm_m16)]), vex(L128)._66()._0f38().op(0x22).r(), (_64b | compat) & avx),
+        inst("vpmovsxwd", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x23).r(), (_64b | compat) & avx),
+        inst("vpmovsxwq", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x24).r(), (_64b | compat) & avx),
+        inst("vpmovsxdq", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x25).r(), (_64b | compat) & avx),
 
         // Move and extend packed integers to and from XMM locations with zero extension.
-        inst("pmovzxbw", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x30]).r(), _64b | compat | sse41).alt(avx, "vpmovzxbw_a"),
-        inst("pmovzxbd", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x31]).r(), _64b | compat | sse41).alt(avx, "vpmovzxbd_a"),
-        inst("pmovzxbq", fmt("A", [w(xmm1), r(xmm_m16)]), rex([0x66, 0x0F, 0x38, 0x32]).r(), _64b | compat | sse41).alt(avx, "vpmovzxbq_a"),
-        inst("pmovzxwd", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x33]).r(), _64b | compat | sse41).alt(avx, "vpmovzxwd_a"),
-        inst("pmovzxwq", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x34]).r(), _64b | compat | sse41).alt(avx, "vpmovzxwq_a"),
-        inst("pmovzxdq", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x35]).r(), _64b | compat | sse41).alt(avx, "vpmovzxdq_a"),
-        inst("vpmovzxbw", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x30).r(), _64b | compat | avx),
-        inst("vpmovzxbd", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x31).r(), _64b | compat | avx),
-        inst("vpmovzxbq", fmt("A", [w(xmm1), r(xmm_m16)]), vex(L128)._66()._0f38().op(0x32).r(), _64b | compat | avx),
-        inst("vpmovzxwd", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x33).r(), _64b | compat | avx),
-        inst("vpmovzxwq", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x34).r(), _64b | compat | avx),
-        inst("vpmovzxdq", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x35).r(), _64b | compat | avx),
+        inst("pmovzxbw", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x30]).r(), (_64b | compat) & sse41).alt(avx, "vpmovzxbw_a"),
+        inst("pmovzxbd", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x31]).r(), (_64b | compat) & sse41).alt(avx, "vpmovzxbd_a"),
+        inst("pmovzxbq", fmt("A", [w(xmm1), r(xmm_m16)]), rex([0x66, 0x0F, 0x38, 0x32]).r(), (_64b | compat) & sse41).alt(avx, "vpmovzxbq_a"),
+        inst("pmovzxwd", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x33]).r(), (_64b | compat) & sse41).alt(avx, "vpmovzxwd_a"),
+        inst("pmovzxwq", fmt("A", [w(xmm1), r(xmm_m32)]), rex([0x66, 0x0F, 0x38, 0x34]).r(), (_64b | compat) & sse41).alt(avx, "vpmovzxwq_a"),
+        inst("pmovzxdq", fmt("A", [w(xmm1), r(xmm_m64)]), rex([0x66, 0x0F, 0x38, 0x35]).r(), (_64b | compat) & sse41).alt(avx, "vpmovzxdq_a"),
+        inst("vpmovzxbw", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x30).r(), (_64b | compat) & avx),
+        inst("vpmovzxbd", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x31).r(), (_64b | compat) & avx),
+        inst("vpmovzxbq", fmt("A", [w(xmm1), r(xmm_m16)]), vex(L128)._66()._0f38().op(0x32).r(), (_64b | compat) & avx),
+        inst("vpmovzxwd", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x33).r(), (_64b | compat) & avx),
+        inst("vpmovzxwq", fmt("A", [w(xmm1), r(xmm_m32)]), vex(L128)._66()._0f38().op(0x34).r(), (_64b | compat) & avx),
+        inst("vpmovzxdq", fmt("A", [w(xmm1), r(xmm_m64)]), vex(L128)._66()._0f38().op(0x35).r(), (_64b | compat) & avx),
     ]
 }
