@@ -145,11 +145,28 @@ fn pulley_emit<P>(
 
         Inst::GetSpecial { dst, reg } => enc::xmov(sink, dst, reg),
 
-        Inst::LoadExtName { .. } => todo!(),
+        Inst::LoadExtNameNear { dst, name, offset } => {
+            patch_pc_rel_offset(sink, |sink| enc::xpcadd(sink, dst, 0));
+            let end = sink.cur_offset();
+            sink.add_reloc_at_offset(end - 4, Reloc::PulleyPcRel, &**name, *offset);
+        }
+
+        Inst::LoadExtNameFar { dst, name, offset } => {
+            let size = match P::pointer_width() {
+                PointerWidth::PointerWidth32 => {
+                    enc::xconst32(sink, dst, 0);
+                    4
+                }
+                PointerWidth::PointerWidth64 => {
+                    enc::xconst64(sink, dst, 0);
+                    8
+                }
+            };
+            let end = sink.cur_offset();
+            sink.add_reloc_at_offset(end - size, Reloc::Abs8, &**name, *offset);
+        }
 
         Inst::Call { info } => {
-            let offset = sink.cur_offset();
-
             // If arguments happen to already be in the right register for the
             // ABI then remove them from this list. Otherwise emit the
             // appropriate `Call` instruction depending on how many arguments we
@@ -159,32 +176,26 @@ fn pulley_emit<P>(
             while !args.is_empty() && args.last().copied() == XReg::new(x_reg(args.len() - 1)) {
                 args = &args[..args.len() - 1];
             }
-            match args {
+            patch_pc_rel_offset(sink, |sink| match args {
                 [] => enc::call(sink, 0),
                 [x0] => enc::call1(sink, x0, 0),
                 [x0, x1] => enc::call2(sink, x0, x1, 0),
                 [x0, x1, x2] => enc::call3(sink, x0, x1, x2, 0),
                 [x0, x1, x2, x3] => enc::call4(sink, x0, x1, x2, x3, 0),
                 _ => unreachable!(),
-            }
+            });
             let end = sink.cur_offset();
-            sink.add_reloc_at_offset(
-                end - 4,
-                // TODO: is it actually okay to reuse this reloc here?
-                Reloc::X86CallPCRel4,
-                &info.dest.name,
-                // This addend adjusts for the difference between the start of
-                // the instruction and the beginning of the immediate offset
-                // field which is always the final 4 bytes of the instruction.
-                -i64::from(end - offset - 4),
-            );
+            sink.add_reloc_at_offset(end - 4, Reloc::PulleyPcRel, &info.dest.name, 0);
             if let Some(s) = state.take_stack_map() {
                 let offset = sink.cur_offset();
                 sink.push_user_stack_map(state, offset, s);
             }
 
             if let Some(try_call) = info.try_call_info.as_ref() {
-                sink.add_try_call_site(try_call.exception_handlers(&state.frame_layout));
+                sink.add_try_call_site(
+                    Some(state.frame_layout.sp_to_fp()),
+                    try_call.exception_handlers(&state.frame_layout),
+                );
             } else {
                 sink.add_call_site();
             }
@@ -224,7 +235,10 @@ fn pulley_emit<P>(
             }
 
             if let Some(try_call) = info.try_call_info.as_ref() {
-                sink.add_try_call_site(try_call.exception_handlers(&state.frame_layout));
+                sink.add_try_call_site(
+                    Some(state.frame_layout.sp_to_fp()),
+                    try_call.exception_handlers(&state.frame_layout),
+                );
             } else {
                 sink.add_call_site();
             }
@@ -261,8 +275,8 @@ fn pulley_emit<P>(
             // Emit an unconditional jump which is quite similar to `Inst::Call`
             // except that a `jump` opcode is used instead of a `call` opcode.
             sink.put1(pulley_interpreter::Opcode::Jump as u8);
-            sink.add_reloc(Reloc::X86CallPCRel4, &info.dest, -1);
-            sink.put4(0);
+            sink.add_reloc(Reloc::PulleyPcRel, &info.dest, 0);
+            sink.put4(1);
 
             // Islands were manually handled in
             // `emit_return_call_common_sequence`.
@@ -290,7 +304,10 @@ fn pulley_emit<P>(
             }
 
             if let Some(try_call) = info.try_call_info.as_ref() {
-                sink.add_try_call_site(try_call.exception_handlers(&state.frame_layout));
+                sink.add_try_call_site(
+                    Some(state.frame_layout.sp_to_fp()),
+                    try_call.exception_handlers(&state.frame_layout),
+                );
             } else {
                 sink.add_call_site();
             }
@@ -301,9 +318,9 @@ fn pulley_emit<P>(
         }
 
         Inst::Jump { label } => {
-            sink.use_label_at_offset(*start_offset + 1, *label, LabelUse::Jump(1));
+            sink.use_label_at_offset(*start_offset + 1, *label, LabelUse::PcRel);
             sink.add_uncond_branch(*start_offset, *start_offset + 5, *label);
-            enc::jump(sink, 0x00000000);
+            patch_pc_rel_offset(sink, |sink| enc::jump(sink, 0));
         }
 
         Inst::BrIf {
@@ -315,9 +332,12 @@ fn pulley_emit<P>(
             // their trailing 4 bytes as the relative offset which is what we're
             // going to target here within the `MachBuffer`.
             let mut inverted = SmallVec::<[u8; 16]>::new();
-            cond.invert().encode(&mut inverted);
+            cond.invert().encode(&mut inverted, 0);
             let len = inverted.len() as u32;
-            debug_assert!(len > 4);
+            inverted.clear();
+            cond.invert()
+                .encode(&mut inverted, i32::try_from(len - 4).unwrap());
+            assert!(len > 4);
 
             // Use the `taken` label 4 bytes before the end of the instruction
             // we're about to emit as that's the base of `PcRelOffset`. Note
@@ -325,9 +345,9 @@ fn pulley_emit<P>(
             // instruction to the start of the relative offset, hence `len - 4`
             // as the factor to adjust by.
             let taken_end = *start_offset + len;
-            sink.use_label_at_offset(taken_end - 4, *taken, LabelUse::Jump(len - 4));
+            sink.use_label_at_offset(taken_end - 4, *taken, LabelUse::PcRel);
             sink.add_cond_branch(*start_offset, taken_end, *taken, &inverted);
-            cond.encode(sink);
+            patch_pc_rel_offset(sink, |sink| cond.encode(sink, 0));
             debug_assert_eq!(sink.cur_offset(), taken_end);
 
             // For the not-taken branch use an unconditional jump to the
@@ -335,9 +355,9 @@ fn pulley_emit<P>(
             // long where the final 4 bytes are the offset to jump by.
             let not_taken_start = taken_end + 1;
             let not_taken_end = not_taken_start + 4;
-            sink.use_label_at_offset(not_taken_start, *not_taken, LabelUse::Jump(1));
+            sink.use_label_at_offset(not_taken_start, *not_taken, LabelUse::PcRel);
             sink.add_uncond_branch(taken_end, not_taken_end, *not_taken);
-            enc::jump(sink, 0x00000000);
+            patch_pc_rel_offset(sink, |sink| enc::jump(sink, 0));
             assert_eq!(sink.cur_offset(), not_taken_end);
         }
 
@@ -546,11 +566,11 @@ fn pulley_emit<P>(
             enc::br_table32(sink, *idx, amt);
             for target in targets.iter() {
                 let offset = sink.cur_offset();
-                sink.use_label_at_offset(offset, *target, LabelUse::Jump(0));
+                sink.use_label_at_offset(offset, *target, LabelUse::PcRel);
                 sink.put4(0);
             }
             let offset = sink.cur_offset();
-            sink.use_label_at_offset(offset, *default, LabelUse::Jump(0));
+            sink.use_label_at_offset(offset, *default, LabelUse::PcRel);
             sink.put4(0);
 
             // We manually handled `emit_island` above when dealing with
@@ -647,3 +667,27 @@ fn return_call_emit_impl<T, P>(
         }
     }
 }
+
+/// Invokes `f` with `sink` and assumes that a single instruction is emitted
+/// which ends with a Pulley `PcRelOffset`.
+///
+/// The offset at that location is patched to include the size of the
+/// instruction before the relative offset since relocations will be applied to
+/// the address of the offset and added to the contents at the offset. The
+/// Pulley interpreter, however, will calculate the offset from the start of the
+/// instruction, so this extra offset is required.
+fn patch_pc_rel_offset<P>(
+    sink: &mut MachBuffer<InstAndKind<P>>,
+    f: impl FnOnce(&mut MachBuffer<InstAndKind<P>>),
+) where
+    P: PulleyTargetKind,
+{
+    let patch = sink.start_patchable();
+    let start = sink.cur_offset();
+    f(sink);
+    let end = sink.cur_offset();
+    let region = sink.end_patchable(patch).patch(sink);
+    let chunk = region.last_chunk_mut::<4>().unwrap();
+    assert_eq!(*chunk, [0, 0, 0, 0]);
+    *chunk = (end - start - 4).to_le_bytes();
+}
