@@ -1,7 +1,6 @@
 //! ISLE integration glue code for riscv64 lowering.
 
 // Pull in the ISLE generated code.
-#[allow(unused)]
 pub mod generated_code;
 use generated_code::MInst;
 
@@ -434,10 +433,6 @@ impl generated_code::Context for RV64IsleContext<'_, '_, MInst, Riscv64Backend>
         UImm5::maybe_from_u8(frm.bits()).unwrap()
     }
 
-    fn u8_as_i32(&mut self, x: u8) -> i32 {
-        x as i32
-    }
-
     fn imm12_const(&mut self, val: i32) -> Imm12 {
         if let Some(res) = Imm12::maybe_from_i64(val as i64) {
             res
@@ -580,15 +575,6 @@ impl generated_code::Context for RV64IsleContext<'_, '_, MInst, Riscv64Backend>
     fn store_op(&mut self, ty: Type) -> StoreOP {
         StoreOP::from_type(ty)
     }
-    fn load_ext_name(&mut self, name: ExternalName, offset: i64) -> Reg {
-        let tmp = self.temp_writable_reg(I64);
-        self.emit(&MInst::LoadExtName {
-            rd: tmp,
-            name: Box::new(name),
-            offset,
-        });
-        tmp.to_reg()
-    }
 
     fn gen_stack_addr(&mut self, slot: StackSlot, offset: Offset32) -> Reg {
         let result = self.temp_writable_reg(I64);
@@ -732,6 +718,10 @@ impl generated_code::Context for RV64IsleContext<'_, '_, MInst, Riscv64Backend>
             _ => unimplemented!(),
         }
     }
+
+    fn is_pic(&mut self) -> bool {
+        self.backend.flags.is_pic()
+    }
 }
 
 /// The main entry point for lowering with ISLE.
