@@ -65,7 +65,6 @@
 
 use crate::dbg::DisplayList;
 use crate::dominator_tree::DominatorTree;
-use crate::dominator_tree::DominatorTreePreorder;
 use crate::entity::SparseSet;
 use crate::flowgraph::{BlockPredecessor, ControlFlowGraph};
 use crate::ir::ExceptionTableItem;
@@ -303,7 +302,6 @@ enum BlockCallTargetType {
 struct Verifier<'a> {
     func: &'a Function,
     expected_cfg: ControlFlowGraph,
-    expected_domtree_preorder: DominatorTreePreorder,
     expected_domtree: DominatorTree,
     isa: Option<&'a dyn TargetIsa>,
 }
@@ -312,13 +310,10 @@ impl<'a> Verifier<'a> {
     pub fn new(func: &'a Function, fisa: FlagsOrIsa<'a>) -> Self {
         let expected_cfg = ControlFlowGraph::with_function(func);
         let expected_domtree = DominatorTree::with_function(func, &expected_cfg);
-        let mut expected_domtree_preorder = DominatorTreePreorder::new();
-        expected_domtree_preorder.compute(&expected_domtree);
         Self {
             func,
             expected_cfg,
             expected_domtree,
-            expected_domtree_preorder,
             isa: fisa.isa,
         }
     }
@@ -1006,11 +1001,10 @@ impl<'a> Verifier<'a> {
                 }
                 // Defining instruction dominates the instruction that uses the value.
                 if is_reachable {
-                    if !self.expected_domtree_preorder.dominates_inst(
-                        def_inst,
-                        loc_inst,
-                        &self.func.layout,
-                    ) {
+                    if !self
+                        .expected_domtree
+                        .dominates(def_inst, loc_inst, &self.func.layout)
+                    {
                         return errors.fatal((
                             loc_inst,
                             self.context(loc_inst),
@@ -1045,7 +1039,7 @@ impl<'a> Verifier<'a> {
                 }
                 let user_block = self.func.layout.inst_block(loc_inst).expect("Expected instruction to be in a block as we're traversing code already in layout");
                 // The defining block dominates the instruction using this value.
-                if is_reachable && !self.expected_domtree_preorder.dominates(block, user_block) {
+                if is_reachable && !self.expected_domtree.block_dominates(block, user_block) {
                     return errors.fatal((
                         loc_inst,
                         self.context(loc_inst),
