@@ -1,15 +1,25 @@
 //! Generate the Cranelift-specific integration of the x64 assembler.
 
 use cranelift_assembler_x64_meta::dsl::{
-    Format, Inst, Location, Mutability, Operand, OperandKind, RegClass,
+    Feature, Format, Inst, Location, Mutability, Operand, OperandKind, RegClass,
 };
 use cranelift_srcgen::{Formatter, fmtln};
 
 /// This factors out use of the assembler crate name.
 const ASM: &str = "cranelift_assembler_x64";
 
+fn include_inst(inst: &Inst) -> bool {
+    // No need to worry about this instruction shape in ISLE as it's generated
+    // in ABI code, not ISLE.
+    if inst.mnemonic.starts_with("push") {
+        return false;
+    }
+
+    true
+}
+
 /// Returns the Rust type used for the `IsleConstructorRaw` variants.
-pub fn rust_param_raw(op: &Operand) -> String {
+fn rust_param_raw(op: &Operand) -> String {
     match op.location.kind() {
         OperandKind::Imm(loc) => {
             let bits = loc.bits();
@@ -34,7 +44,7 @@ pub fn rust_param_raw(op: &Operand) -> String {
 /// Returns the conversion function, if any, when converting the ISLE type for
 /// this parameter to the assembler type for this parameter. Effectively
 /// converts `self.rust_param_raw()` to the assembler type.
-pub fn rust_convert_isle_to_assembler(op: &Operand) -> String {
+fn rust_convert_isle_to_assembler(op: &Operand) -> String {
     match op.location.kind() {
         OperandKind::Imm(loc) => {
             let bits = loc.bits();
@@ -87,7 +97,7 @@ pub fn rust_convert_isle_to_assembler(op: &Operand) -> String {
 /// # Panics
 ///
 /// This function panics if the instruction has no operands.
-pub fn generate_macro_inst_fn(f: &mut Formatter, inst: &Inst) {
+fn generate_macro_inst_fn(f: &mut Formatter, inst: &Inst) {
     use OperandKind::*;
 
     let struct_name = inst.name();
@@ -152,7 +162,7 @@ pub fn generate_macro_inst_fn(f: &mut Formatter, inst: &Inst) {
                 (ty, var)
             };
             match results.as_slice() {
-                [] => fmtln!(f, "SideEffectNoResult::Inst(inst)"),
+                [] => fmtln!(f, "AssemblerOutputs::SideEffect {{ inst }}"),
                 [op] => match op.location.kind() {
                     Imm(_) => unreachable!(),
                     Reg(r) | FixedReg(r) => {
@@ -186,8 +196,28 @@ pub fn generate_macro_inst_fn(f: &mut Formatter, inst: &Inst) {
                         fmtln!(f, "let regs = ValueRegs::two(one, two);");
                         fmtln!(f, "AssemblerOutputs::RetValueRegs {{ inst, regs }}");
                     }
+                    (Reg(reg), Mem(_)) | (Mem(_) | RegMem(_), Reg(reg) | FixedReg(reg)) => {
+                        let (ty, var) = ty_var_of_reg(reg);
+                        fmtln!(f, "let {var} = {reg}.as_ref().{};", access_reg(op2));
+                        fmtln!(f, "AssemblerOutputs::Ret{ty} {{ inst, {var} }}");
+                    }
                     _ => unimplemented!("unhandled results: {results:?}"),
                 },
+
+                [op1, op2, op3] => match (
+                    op1.location.kind(),
+                    op2.location.kind(),
+                    op3.location.kind(),
+                ) {
+                    (FixedReg(loc1), FixedReg(loc2), Mem(_)) => {
+                        fmtln!(f, "let one = {loc1}.as_ref().{}.to_reg();", access_reg(op1));
+                        fmtln!(f, "let two = {loc2}.as_ref().{}.to_reg();", access_reg(op2));
+                        fmtln!(f, "let regs = ValueRegs::two(one, two);");
+                        fmtln!(f, "AssemblerOutputs::RetValueRegs {{ inst, regs }}");
+                    }
+                    _ => unimplemented!("unhandled results: {results:?}"),
+                },
+
                 _ => panic!("instruction has more than one result"),
             }
         },
@@ -202,7 +232,9 @@ pub fn generate_rust_macro(f: &mut Formatter, insts: &[Inst]) {
         fmtln!(f, "() => {{");
         f.indent(|f| {
             for inst in insts {
-                generate_macro_inst_fn(f, inst);
+                if include_inst(inst) {
+                    generate_macro_inst_fn(f, inst);
+                }
             }
         });
         fmtln!(f, "}};");
@@ -212,7 +244,7 @@ pub fn generate_rust_macro(f: &mut Formatter, insts: &[Inst]) {
 
 /// Returns the type of this operand in ISLE as a part of the ISLE "raw"
 /// constructors.
-pub fn isle_param_raw(op: &Operand) -> String {
+fn isle_param_raw(op: &Operand) -> String {
     match op.location.kind() {
         OperandKind::Imm(loc) => {
             let bits = loc.bits();
@@ -246,7 +278,7 @@ pub fn isle_param_raw(op: &Operand) -> String {
 /// read/write operand is `GprMem` will generate two constructors though, one
 /// for memory and one for in registers.
 #[derive(Copy, Clone, Debug)]
-pub enum IsleConstructor {
+enum IsleConstructor {
     /// This constructor only produces a side effect, meaning that the
     /// instruction does not produce results in registers. This may produce
     /// a result in memory, however.
@@ -262,35 +294,59 @@ pub enum IsleConstructor {
     /// This "special" constructor captures multiple written-to registers (e.g.
     /// `mul`).
     RetValueRegs,
+
+    /// This constructor does not return any results, but produces a side effect affecting EFLAGs.
+    NoReturnSideEffect,
+
+    /// This constructor produces no results, but the flags register is written,
+    /// so a `ProducesFlags` value is returned with a side effect.
+    ProducesFlagsSideEffect,
+
+    /// This instructions reads EFLAGS, and returns a single gpr, so this
+    /// creates `ConsumesFlags.ConsumesFlagsReturnsReg`.
+    ConsumesFlagsReturnsGpr,
 }
 
 impl IsleConstructor {
     /// Returns the result type, in ISLE, that this constructor generates.
-    pub fn result_ty(&self) -> &'static str {
+    fn result_ty(&self) -> &'static str {
         match self {
-            IsleConstructor::RetMemorySideEffect => "SideEffectNoResult",
             IsleConstructor::RetGpr => "Gpr",
             IsleConstructor::RetXmm => "Xmm",
             IsleConstructor::RetValueRegs => "ValueRegs",
+            IsleConstructor::NoReturnSideEffect | IsleConstructor::RetMemorySideEffect => {
+                "SideEffectNoResult"
+            }
+            IsleConstructor::ProducesFlagsSideEffect => "ProducesFlags",
+            IsleConstructor::ConsumesFlagsReturnsGpr => "ConsumesFlags",
         }
     }
 
     /// Returns the constructor used to convert an `AssemblerOutput` into the
     /// type returned by [`Self::result_ty`].
-    pub fn conversion_constructor(&self) -> &'static str {
+    fn conversion_constructor(&self) -> &'static str {
         match self {
-            IsleConstructor::RetMemorySideEffect => "defer_side_effect",
+            IsleConstructor::NoReturnSideEffect | IsleConstructor::RetMemorySideEffect => {
+                "defer_side_effect"
+            }
             IsleConstructor::RetGpr => "emit_ret_gpr",
             IsleConstructor::RetXmm => "emit_ret_xmm",
             IsleConstructor::RetValueRegs => "emit_ret_value_regs",
+            IsleConstructor::ProducesFlagsSideEffect => "asm_produce_flags_side_effect",
+            IsleConstructor::ConsumesFlagsReturnsGpr => "asm_consumes_flags_returns_gpr",
         }
     }
 
     /// Returns the suffix used in the ISLE constructor name.
-    pub fn suffix(&self) -> &'static str {
+    fn suffix(&self) -> &'static str {
         match self {
             IsleConstructor::RetMemorySideEffect => "_mem",
-            IsleConstructor::RetGpr | IsleConstructor::RetXmm | IsleConstructor::RetValueRegs => "",
+            IsleConstructor::RetGpr
+            | IsleConstructor::RetXmm
+            | IsleConstructor::RetValueRegs
+            | IsleConstructor::NoReturnSideEffect
+            | IsleConstructor::ProducesFlagsSideEffect
+            | IsleConstructor::ConsumesFlagsReturnsGpr => "",
         }
     }
 
@@ -299,19 +355,22 @@ impl IsleConstructor {
     ///
     /// Memory-based ctors take an `Amode`, but register-based ctors don't take
     /// the result as an argument and instead manufacture it internally.
-    pub fn includes_write_only_reg_mem(&self) -> bool {
+    fn includes_write_only_reg_mem(&self) -> bool {
         match self {
             IsleConstructor::RetMemorySideEffect => true,
-            IsleConstructor::RetGpr | IsleConstructor::RetXmm | IsleConstructor::RetValueRegs => {
-                false
-            }
+            IsleConstructor::RetGpr
+            | IsleConstructor::RetXmm
+            | IsleConstructor::RetValueRegs
+            | IsleConstructor::NoReturnSideEffect
+            | IsleConstructor::ProducesFlagsSideEffect
+            | IsleConstructor::ConsumesFlagsReturnsGpr => false,
         }
     }
 }
 
 /// Returns the parameter type used for the `IsleConstructor` variant
 /// provided.
-pub fn isle_param_for_ctor(op: &Operand, ctor: IsleConstructor) -> String {
+fn isle_param_for_ctor(op: &Operand, ctor: IsleConstructor) -> String {
     match op.location.kind() {
         // Writable `RegMem` operands are special here: in one constructor
         // it's operating on memory so the argument is `Amode` and in the
@@ -319,9 +378,11 @@ pub fn isle_param_for_ctor(op: &Operand, ctor: IsleConstructor) -> String {
         // a `Gpr`.
         OperandKind::RegMem(_) if op.mutability.is_write() => match ctor {
             IsleConstructor::RetMemorySideEffect => "SyntheticAmode".to_string(),
-            IsleConstructor::RetGpr => "Gpr".to_string(),
+            IsleConstructor::NoReturnSideEffect => "".to_string(),
+            IsleConstructor::RetGpr | IsleConstructor::ConsumesFlagsReturnsGpr => "Gpr".to_string(),
             IsleConstructor::RetXmm => "Xmm".to_string(),
             IsleConstructor::RetValueRegs => "ValueRegs".to_string(),
+            IsleConstructor::ProducesFlagsSideEffect => todo!(),
         },
 
         // everything else is the same as the "raw" variant
@@ -334,7 +395,7 @@ pub fn isle_param_for_ctor(op: &Operand, ctor: IsleConstructor) -> String {
 ///
 /// Note that one instruction might need multiple constructors, such as one
 /// for operating on memory and one for operating on registers.
-pub fn isle_constructors(format: &Format) -> Vec<IsleConstructor> {
+fn isle_constructors(format: &Format) -> Vec<IsleConstructor> {
     use Mutability::*;
     use OperandKind::*;
 
@@ -344,9 +405,13 @@ pub fn isle_constructors(format: &Format) -> Vec<IsleConstructor> {
         .filter(|o| o.mutability.is_write())
         .collect::<Vec<_>>();
     match &write_operands[..] {
-        [] => unimplemented!(
-            "if you truly need this (and not a `SideEffect*`), add a `NoReturn` variant to `AssemblerOutputs`"
-        ),
+        [] => {
+            if format.eflags.is_write() {
+                vec![IsleConstructor::ProducesFlagsSideEffect]
+            } else {
+                vec![IsleConstructor::NoReturnSideEffect]
+            }
+        }
         [one] => match one.mutability {
             Read => unreachable!(),
             ReadWrite | Write => match one.location.kind() {
@@ -354,33 +419,78 @@ pub fn isle_constructors(format: &Format) -> Vec<IsleConstructor> {
                 // One read/write register output? Output the instruction
                 // and that register.
                 Reg(r) | FixedReg(r) => match r.reg_class().unwrap() {
-                    RegClass::Xmm => vec![IsleConstructor::RetXmm],
-                    RegClass::Gpr => vec![IsleConstructor::RetGpr],
+                    RegClass::Xmm => {
+                        assert!(!format.eflags.is_read());
+                        vec![IsleConstructor::RetXmm]
+                    }
+                    RegClass::Gpr => {
+                        if format.eflags.is_read() {
+                            vec![IsleConstructor::ConsumesFlagsReturnsGpr]
+                        } else {
+                            vec![IsleConstructor::RetGpr]
+                        }
+                    }
                 },
                 // One read/write memory operand? Output a side effect.
-                Mem(_) => vec![IsleConstructor::RetMemorySideEffect],
+                Mem(_) => {
+                    assert!(!format.eflags.is_read());
+                    vec![IsleConstructor::RetMemorySideEffect]
+                }
                 // One read/write reg-mem output? We need constructors for
                 // both variants.
                 RegMem(rm) => match rm.reg_class().unwrap() {
-                    RegClass::Xmm => vec![
-                        IsleConstructor::RetXmm,
-                        IsleConstructor::RetMemorySideEffect,
-                    ],
-                    RegClass::Gpr => vec![
-                        IsleConstructor::RetGpr,
-                        IsleConstructor::RetMemorySideEffect,
-                    ],
+                    RegClass::Xmm => {
+                        assert!(!format.eflags.is_read());
+                        vec![
+                            IsleConstructor::RetXmm,
+                            IsleConstructor::RetMemorySideEffect,
+                        ]
+                    }
+                    RegClass::Gpr => {
+                        if format.eflags.is_read() {
+                            // FIXME: should expand this to include "consumes
+                            // flags plus side effect" to model the
+                            // memory-writing variant too. For example this
+                            // means there's no memory-writing variant of
+                            // `setcc` instructions generated.
+                            vec![IsleConstructor::ConsumesFlagsReturnsGpr]
+                        } else {
+                            vec![
+                                IsleConstructor::RetGpr,
+                                IsleConstructor::RetMemorySideEffect,
+                            ]
+                        }
+                    }
                 },
             },
         },
         [one, two] => {
-            // For now, we assume that if there are two results, they are coming
-            // from a register-writing instruction like `mul`. This can be
-            // expanded as needed.
-            assert!(matches!(one.location.kind(), FixedReg(_) | Reg(_)));
-            assert!(matches!(two.location.kind(), FixedReg(_) | Reg(_)));
-            vec![IsleConstructor::RetValueRegs]
+            assert!(!format.eflags.is_read());
+            match (one.location.kind(), two.location.kind()) {
+                (FixedReg(_) | Reg(_), FixedReg(_) | Reg(_)) => {
+                    vec![IsleConstructor::RetValueRegs]
+                }
+                (Reg(r), Mem(_)) | (Mem(_) | RegMem(_), Reg(r) | FixedReg(r)) => {
+                    assert!(matches!(r.reg_class().unwrap(), RegClass::Gpr));
+                    vec![IsleConstructor::RetGpr]
+                }
+                other => panic!("unsupported number of write operands {other:?}"),
+            }
         }
+        [one, two, three] => {
+            assert!(!format.eflags.is_read());
+            match (
+                one.location.kind(),
+                two.location.kind(),
+                three.location.kind(),
+            ) {
+                (FixedReg(_), FixedReg(_), Mem(_)) => {
+                    vec![IsleConstructor::RetValueRegs]
+                }
+                other => panic!("unsupported number of write operands {other:?}"),
+            }
+        }
+
         other => panic!("unsupported number of write operands {other:?}"),
     }
 }
@@ -413,7 +523,7 @@ pub fn isle_constructors(format: &Format) -> Vec<IsleConstructor> {
 /// # Panics
 ///
 /// This function panics if the instruction has no operands.
-pub fn generate_isle_inst_decls(f: &mut Formatter, inst: &Inst) {
+fn generate_isle_inst_decls(f: &mut Formatter, inst: &Inst) {
     let (trap_type, trap_name) = if inst.has_trap {
         (Some("TrapCode".to_string()), Some("trap".to_string()))
     } else {
@@ -490,10 +600,16 @@ pub fn generate_isle_inst_decls(f: &mut Formatter, inst: &Inst) {
             .map(|o| {
                 assert!(matches!(o.location.kind(), OperandKind::RegMem(_)));
                 match ctor {
-                    IsleConstructor::RetMemorySideEffect => unreachable!(),
-                    IsleConstructor::RetGpr => "(temp_writable_gpr)",
+                    IsleConstructor::RetMemorySideEffect | IsleConstructor::NoReturnSideEffect => {
+                        unreachable!()
+                    }
+                    IsleConstructor::RetGpr | IsleConstructor::ConsumesFlagsReturnsGpr => {
+                        "(temp_writable_gpr)"
+                    }
                     IsleConstructor::RetXmm => "(temp_writable_xmm)",
-                    IsleConstructor::RetValueRegs => todo!(),
+                    IsleConstructor::RetValueRegs | IsleConstructor::ProducesFlagsSideEffect => {
+                        todo!()
+                    }
                 }
             })
             .collect::<Vec<_>>()
@@ -504,6 +620,37 @@ pub fn generate_isle_inst_decls(f: &mut Formatter, inst: &Inst) {
             f,
             "(rule ({rule_name} {param_names}) ({convert} ({raw_name} {implicit_params} {param_names})))"
         );
+
+        if let Some(alternate) = &inst.alternate {
+            // We currently plan to use alternate instructions for SSE/AVX
+            // pairs, so we expect the one of the registers to be an XMM
+            // register. In the future we could relax this, but would need to
+            // handle more cases below.
+            assert!(
+                inst.format
+                    .operands
+                    .iter()
+                    .any(|o| matches!(o.location.reg_class(), Some(RegClass::Xmm)))
+            );
+            let param_tys = if alternate.feature == Feature::avx {
+                param_tys.replace("Aligned", "")
+            } else {
+                param_tys
+            };
+            let alt_feature = alternate.feature.to_string();
+            let alt_name = &alternate.name;
+            let rule_name_or_feat = format!("{rule_name}_or_{alt_feature}");
+            fmtln!(f, "(decl {rule_name_or_feat} ({param_tys}) {result_ty})");
+            fmtln!(f, "(rule 1 ({rule_name_or_feat} {param_names})");
+            f.indent(|f| {
+                fmtln!(f, "(if-let true (use_{alt_feature}))");
+                fmtln!(f, "(x64_{alt_name}{suffix} {param_names}))");
+            });
+            fmtln!(
+                f,
+                "(rule 0 ({rule_name_or_feat} {param_names}) ({rule_name} {param_names}))"
+            );
+        }
     }
 }
 
@@ -568,8 +715,10 @@ pub fn generate_isle(f: &mut Formatter, insts: &[Inst]) {
     f.empty_line();
 
     for inst in insts {
-        generate_isle_inst_decls(f, inst);
-        f.empty_line();
+        if include_inst(inst) {
+            generate_isle_inst_decls(f, inst);
+            f.empty_line();
+        }
     }
 }
 
