@@ -160,9 +160,14 @@
 //!
 //! Maybe you want to know if a file contains other, malicious content?
 //! In this case you should use an anti-virus software, e.g. [ClamAV](https://www.clamav.net/), [Virus Total](https://www.virustotal.com/).
+//!
+//! Maybe you need to support platforms that are tricky with the `libmagic` C library, e.g. wasm32?
+//! In this case you could use pure Rust alternatives, e.g. the [`file_type` crate](https://crates.io/crates/file_type).
 
 #![deny(unsafe_code)]
 
+use core::ffi::c_int;
+
 mod ffi;
 
 /// Returns the version of the `libmagic` C library as reported by itself.
@@ -170,14 +175,14 @@ mod ffi;
 /// # Examples
 /// A version of "5.41" is returned as `541`.
 #[doc(alias = "magic_version")]
-pub fn libmagic_version() -> libc::c_int {
+pub fn libmagic_version() -> c_int {
     crate::ffi::version()
 }
 
 /// Functionality for [`Cookie`]
 pub mod cookie {
     use std::convert::TryFrom;
-    use std::ffi::CString;
+    use std::ffi::{c_int, CString};
     use std::path::Path;
 
     use magic_sys as libmagic;
@@ -229,7 +234,7 @@ pub mod cookie {
         /// assert_eq!(default_flags, magic::cookie::Flags::empty());
         /// ```
         #[derive(std::default::Default, PartialEq, Eq, PartialOrd, Ord, Hash, Debug, Clone, Copy)]
-        pub struct Flags: libc::c_int {
+        pub struct Flags: c_int {
             // MAGIC_NONE is 0/default, see https://docs.rs/bitflags/latest/bitflags/#zero-bit-flags
 
             // Define unnamed flag for all other bits https://docs.rs/bitflags/latest/bitflags/#externally-defined-flags
@@ -483,10 +488,17 @@ pub mod cookie {
     /// Invalid [`DatabasePaths`]
     ///
     /// This is returned from [`DatabasePaths::new()`](DatabasePaths::new)
-    #[derive(thiserror::Error, Debug)]
-    #[error("invalid database files path")]
+    #[derive(Debug)]
     pub struct InvalidDatabasePathError {}
 
+    impl std::error::Error for InvalidDatabasePathError {}
+
+    impl std::fmt::Display for InvalidDatabasePathError {
+        fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+            write!(f, "invalid database files path")
+        }
+    }
+
     /// Magic database file paths
     ///
     /// `libmagic` requires database file paths for certain operations on a [`Cookie`] that must:
@@ -667,19 +679,35 @@ pub mod cookie {
     ///
     /// Most functions on a [`Cookie`] can return an error from `libmagic`,
     /// which unfortunately is not very structured.
-    #[derive(thiserror::Error, Debug)]
-    #[error("magic cookie error in `libmagic` function {}", .function)]
+    #[derive(Debug)]
     pub struct Error {
         function: &'static str,
         //#[backtrace]
         source: crate::ffi::CookieError,
     }
 
-    #[doc(hidden)]
+    impl std::error::Error for Error {
+        fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+            Some(&self.source)
+        }
+    }
+
+    impl std::fmt::Display for Error {
+        fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+            let Self { function, .. } = self;
+            write!(f, "magic cookie error in `libmagic` function {0}", function,)
+        }
+    }
+
+    /// Typestate marker of opened [`Cookie`]
+    ///
+    /// In this state, a cookie has only been [opened](Cookie::open) but databases are not [loaded](Cookie::load) yet.
     #[derive(Debug)]
     pub enum Open {}
 
-    #[doc(hidden)]
+    /// Typestate marker of loaded [`Cookie`]
+    ///
+    /// In this state, a cookie has both been [opened](Cookie::open) and databases are [loaded](Cookie::load).
     #[derive(Debug)]
     pub enum Load {}
 
@@ -690,7 +718,9 @@ pub mod cookie {
         impl Sealed for super::Load {}
     }
 
-    #[doc(hidden)]
+    /// Typestate marker for [`Cookie`]
+    ///
+    /// A cookie can either be in the [`Open`] or [`Load`] state
     pub trait State: private::Sealed {}
 
     impl State for Open {}
@@ -762,8 +792,7 @@ pub mod cookie {
     /// # Ok(())
     /// # }
     /// ```
-    #[derive(thiserror::Error, Debug)]
-    #[error("magic cookie error in `libmagic` function {}", .function)]
+    #[derive(Debug)]
     pub struct LoadError<S: State> {
         function: &'static str,
         //#[backtrace]
@@ -771,6 +800,22 @@ pub mod cookie {
         cookie: Cookie<S>,
     }
 
+    impl<S: State> std::error::Error for LoadError<S>
+    where
+        Self: std::fmt::Debug,
+    {
+        fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+            Some(&self.source)
+        }
+    }
+
+    impl<S: State> std::fmt::Display for LoadError<S> {
+        fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+            let Self { function, .. } = self;
+            write!(f, "magic cookie error in `libmagic` function {0}", function,)
+        }
+    }
+
     impl<S: State> LoadError<S> {
         /// Returns the cookie in its original state
         pub fn cookie(self) -> Cookie<S> {
@@ -786,7 +831,7 @@ pub mod cookie {
         }
     }
 
-    /// Operations that are valid in the `Open` state
+    /// Operations that are valid in the [`Open`] state
     ///
     /// A new cookie created with [`Cookie::open`](Cookie::open) does not have any databases [loaded](Cookie::load).
     impl Cookie<Open> {
@@ -837,7 +882,7 @@ pub mod cookie {
         }
     }
 
-    /// Operations that are valid in the `Load` state
+    /// Operations that are valid in the [`Load`] state
     ///
     /// An opened cookie with [loaded](Cookie::load) databases can inspect [files](Cookie::file) and [buffers](Cookie::buffer).
     impl Cookie<Load> {
@@ -861,9 +906,7 @@ pub mod cookie {
         ///
         /// If there was an `libmagic` internal error, a [`cookie::Error`](Error) will be returned.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_file")]
         pub fn file<P: AsRef<Path>>(&self, filename: P) -> Result<String, Error> {
             let c_string = CString::new(filename.as_ref().to_string_lossy().into_owned()).unwrap();
@@ -897,9 +940,7 @@ pub mod cookie {
         ///
         /// If there was an `libmagic` internal error, a [`cookie::Error`](Error) will be returned.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_buffer")]
         pub fn buffer(&self, buffer: &[u8]) -> Result<String, Error> {
             match crate::ffi::buffer(&self.cookie, buffer) {
@@ -954,9 +995,7 @@ pub mod cookie {
         /// If there was an `libmagic` internal error, a [`cookie::LoadError`](LoadError) will be returned,
         /// which contains the cookie in its original state.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error or returning undefined data.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_load")]
         #[doc(alias = "--magic-file")]
         pub fn load(self, filenames: &DatabasePaths) -> Result<Cookie<Load>, LoadError<S>> {
@@ -993,9 +1032,7 @@ pub mod cookie {
         /// If there was an `libmagic` internal error, a [`cookie::LoadError`](LoadError) will be returned,
         /// which contains the cookie in its original state.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error or returning undefined data.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_load_buffers")]
         pub fn load_buffers(self, buffers: &[&[u8]]) -> Result<Cookie<Load>, LoadError<S>> {
             match crate::ffi::load_buffers(&self.cookie, buffers) {
@@ -1064,9 +1101,7 @@ pub mod cookie {
         ///
         /// If there was an `libmagic` internal error, a [`cookie::Error`](Error) will be returned.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error or returning undefined data.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_compile")]
         #[doc(alias = "--compile")]
         pub fn compile(&self, filenames: &DatabasePaths) -> Result<(), Error> {
@@ -1085,9 +1120,7 @@ pub mod cookie {
         ///
         /// If there was an `libmagic` internal error, a [`cookie::Error`](Error) will be returned.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error or returning undefined data.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_check")]
         pub fn check(&self, filenames: &DatabasePaths) -> Result<(), Error> {
             match crate::ffi::check(&self.cookie, filenames.filenames.as_deref()) {
@@ -1110,9 +1143,7 @@ pub mod cookie {
         ///
         /// If there was an `libmagic` internal error, a [`cookie::Error`](Error) will be returned.
         ///
-        /// # Panics
-        ///
-        /// Panics if `libmagic` violates its API contract, e.g. by not setting the last error or returning undefined data.
+        /// If `libmagic` violates its API contract, e.g. by not setting the last error, a [`cookie::Error`](Error) will be returned.
         #[doc(alias = "magic_list")]
         #[doc(alias = "--checking-printout")]
         pub fn list(&self, filenames: &DatabasePaths) -> Result<(), Error> {
@@ -1129,13 +1160,7 @@ pub mod cookie {
     /// Error within [`Cookie::open()`](Cookie::open)
     ///
     /// Note that a similar [`cookie::SetFlagsError`](SetFlagsError) can also occur
-    #[derive(thiserror::Error, Debug)]
-    #[error("could not open magic cookie: {}",
-        match .kind {
-            OpenErrorKind::UnsupportedFlags => format!("unsupported flags {}", .flags),
-            OpenErrorKind::Errno => "other error".to_string(),
-        }
-    )]
+    #[derive(Debug)]
     pub struct OpenError {
         flags: Flags,
         kind: OpenErrorKind,
@@ -1143,6 +1168,26 @@ pub mod cookie {
         source: crate::ffi::OpenError,
     }
 
+    impl std::error::Error for OpenError {
+        fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+            Some(&self.source)
+        }
+    }
+
+    impl std::fmt::Display for OpenError {
+        fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+            let Self { flags, kind, .. } = self;
+            write!(
+                f,
+                "could not open magic cookie: {0}",
+                match kind {
+                    OpenErrorKind::UnsupportedFlags => format!("unsupported flags {0}", flags),
+                    OpenErrorKind::Errno => "other error".to_string(),
+                }
+            )
+        }
+    }
+
     /// Kind of [`OpenError`]
     #[derive(Debug)]
     enum OpenErrorKind {
@@ -1155,13 +1200,25 @@ pub mod cookie {
     /// Error within [`Cookie::set_flags()`](Cookie::set_flags)
     ///
     /// Note that a similar [`cookie::OpenError`](OpenError) can also occur
-    #[derive(thiserror::Error, Debug)]
-    #[error("could not set magic cookie flags {}", .flags)]
+    #[derive(Debug)]
     pub struct SetFlagsError {
         flags: Flags,
         //#[backtrace]
         source: crate::ffi::SetFlagsError,
     }
+
+    impl std::error::Error for SetFlagsError {
+        fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
+            Some(&self.source)
+        }
+    }
+
+    impl std::fmt::Display for SetFlagsError {
+        fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
+            let Self { flags, .. } = self;
+            write!(f, "could not set magic cookie flags {0}", flags)
+        }
+    }
 } // mod cookie
 
 pub use crate::cookie::Cookie;
@@ -1169,9 +1226,15 @@ pub use crate::cookie::Cookie;
 #[cfg(test)]
 mod tests {
     use super::cookie::Flags;
+    use super::cookie::{Error, InvalidDatabasePathError, LoadError, OpenError, SetFlagsError};
+    use super::cookie::{Load, Open};
     use super::Cookie;
     use std::convert::TryInto;
 
+    fn assert_impl_debug<T: std::fmt::Debug>() {}
+    fn assert_impl_display<T: std::fmt::Display>() {}
+    fn assert_impl_error<T: std::error::Error>() {}
+
     // Using relative paths to test files should be fine, since cargo doc
     // https://doc.rust-lang.org/cargo/reference/build-scripts.html#inputs-to-the-build-script
     // states that cwd == CARGO_MANIFEST_DIR
@@ -1245,10 +1308,14 @@ mod tests {
         assert!(cookie.load(databases).is_ok());
     }
 
-    // TODO:
-    //static_assertions::assert_impl_all!(Cookie<S>: std::fmt::Debug);
+    #[test]
+    fn cookie_impl_debug() {
+        assert_impl_debug::<Cookie<Open>>();
+        assert_impl_debug::<Cookie<Load>>();
+    }
 
     #[test]
+    #[ignore] // FIXME: the binary .mgc file depends on libmagic (internal format) version
     fn load_buffers_file() {
         let cookie = Cookie::open(Flags::ERROR).unwrap();
         // file --compile --magic-file data/tests/db-images-png
@@ -1269,6 +1336,44 @@ mod tests {
 
         assert!(version > 500);
     }
+
+    #[test]
+    fn error_impls() {
+        assert_impl_debug::<Error>();
+        assert_impl_display::<Error>();
+        assert_impl_error::<Error>();
+    }
+
+    #[test]
+    fn invaliddatabasepatherror_impls() {
+        assert_impl_debug::<InvalidDatabasePathError>();
+        assert_impl_display::<InvalidDatabasePathError>();
+        assert_impl_error::<InvalidDatabasePathError>();
+    }
+
+    #[test]
+    fn loaderror_impls() {
+        assert_impl_debug::<LoadError<Open>>();
+        assert_impl_debug::<LoadError<Load>>();
+        assert_impl_display::<LoadError<Open>>();
+        assert_impl_display::<LoadError<Load>>();
+        assert_impl_error::<LoadError<Open>>();
+        assert_impl_error::<LoadError<Load>>();
+    }
+
+    #[test]
+    fn openerror_impls() {
+        assert_impl_debug::<OpenError>();
+        assert_impl_display::<OpenError>();
+        assert_impl_error::<OpenError>();
+    }
+
+    #[test]
+    fn setflagserror_impls() {
+        assert_impl_debug::<SetFlagsError>();
+        assert_impl_display::<SetFlagsError>();
+        assert_impl_error::<SetFlagsError>();
+    }
 }
 
 #[cfg(doctest)]
