@@ -389,19 +389,20 @@ unsafe fn _chr_qpl_c16_uu_x1(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm_loadu_si128(buf_ptr as *const __m128i);
-    let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
-    let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
-    let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
-    let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
-    let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
-    let mask_0_d = _mm_movemask_epi8(mm_0_eq_d) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    unsafe {
+        let mm_0 = _mm_loadu_si128(buf_ptr as *const __m128i);
+        let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
+        let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
+        let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
+        let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
+        let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
+        let mask_0_d = _mm_movemask_epi8(mm_0_eq_d) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    }
 }
 
 #[inline(always)]
@@ -410,19 +411,20 @@ unsafe fn _chr_qpl_c16_aa_x1(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm_load_si128(buf_ptr as *const __m128i);
-    let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
-    let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
-    let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
-    let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
-    let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
-    let mask_0_d = _mm_movemask_epi8(mm_0_eq_d) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    unsafe {
+        let mm_0 = _mm_load_si128(buf_ptr as *const __m128i);
+        let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
+        let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
+        let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
+        let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
+        let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
+        let mask_0_d = _mm_movemask_epi8(mm_0_eq_d) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    }
 }
 
 #[inline(always)]
@@ -431,11 +433,11 @@ unsafe fn _chr_qpl_c16_aa_x2(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_qpl_c16_aa_x1(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _chr_qpl_c16_aa_x1(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_qpl_c16_aa_x1(buf_ptr.add(16), mm_c16, st_ptr);
+    let r = unsafe { _chr_qpl_c16_aa_x1(buf_ptr.add(16), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -448,11 +450,11 @@ unsafe fn _chr_qpl_c16_aa_x4(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_qpl_c16_aa_x2(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _chr_qpl_c16_aa_x2(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_qpl_c16_aa_x2(buf_ptr.add(16 * 2), mm_c16, st_ptr);
+    let r = unsafe { _chr_qpl_c16_aa_x2(buf_ptr.add(16 * 2), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -465,11 +467,11 @@ unsafe fn _chr_qpl_c16_aa_x8(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_qpl_c16_aa_x4(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _chr_qpl_c16_aa_x4(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_qpl_c16_aa_x4(buf_ptr.add(16 * 4), mm_c16, st_ptr);
+    let r = unsafe { _chr_qpl_c16_aa_x4(buf_ptr.add(16 * 4), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -482,19 +484,20 @@ unsafe fn _chr_qpl_c32_uu_x1(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm256_loadu_si256(buf_ptr as *const __m256i);
-    let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
-    let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
-    let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
-    let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
-    let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
-    let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    unsafe {
+        let mm_0 = _mm256_loadu_si256(buf_ptr as *const __m256i);
+        let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
+        let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
+        let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
+        let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
+        let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
+        let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    }
 }
 
 #[inline(always)]
@@ -503,19 +506,20 @@ unsafe fn _chr_qpl_c32_aa_x1(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm256_load_si256(buf_ptr as *const __m256i);
-    let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
-    let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
-    let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
-    let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
-    let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
-    let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    unsafe {
+        let mm_0 = _mm256_load_si256(buf_ptr as *const __m256i);
+        let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
+        let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
+        let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
+        let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
+        let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
+        let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_qpl(base, mask_0_a, mask_0_b, mask_0_c, mask_0_d)
+    }
 }
 
 #[inline(always)]
@@ -524,11 +528,11 @@ unsafe fn _chr_qpl_c32_aa_x2(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_qpl_c32_aa_x1(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _chr_qpl_c32_aa_x1(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_qpl_c32_aa_x1(buf_ptr.add(32), mm_c32, st_ptr);
+    let r = unsafe { _chr_qpl_c32_aa_x1(buf_ptr.add(32), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -541,11 +545,11 @@ unsafe fn _chr_qpl_c32_aa_x4(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_qpl_c32_aa_x2(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _chr_qpl_c32_aa_x2(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_qpl_c32_aa_x2(buf_ptr.add(32 * 2), mm_c32, st_ptr);
+    let r = unsafe { _chr_qpl_c32_aa_x2(buf_ptr.add(32 * 2), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -558,11 +562,11 @@ unsafe fn _chr_qpl_c32_aa_x8(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_qpl_c32_aa_x4(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _chr_qpl_c32_aa_x4(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_qpl_c32_aa_x4(buf_ptr.add(32 * 4), mm_c32, st_ptr);
+    let r = unsafe { _chr_qpl_c32_aa_x4(buf_ptr.add(32 * 4), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
