@@ -355,17 +355,18 @@ unsafe fn _chr_tpl_c16_uu_x1(
     mm_c16: MMB16Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm_loadu_si128(buf_ptr as *const __m128i);
-    let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
-    let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
-    let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
-    let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    unsafe {
+        let mm_0 = _mm_loadu_si128(buf_ptr as *const __m128i);
+        let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
+        let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
+        let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
+        let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    }
 }
 
 #[inline(always)]
@@ -374,17 +375,18 @@ unsafe fn _chr_tpl_c16_aa_x1(
     mm_c16: MMB16Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm_load_si128(buf_ptr as *const __m128i);
-    let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
-    let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
-    let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
-    let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    unsafe {
+        let mm_0 = _mm_load_si128(buf_ptr as *const __m128i);
+        let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
+        let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
+        let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
+        let mask_0_a = _mm_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm_movemask_epi8(mm_0_eq_c) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    }
 }
 
 #[inline(always)]
@@ -393,11 +395,11 @@ unsafe fn _chr_tpl_c16_aa_x2(
     mm_c16: MMB16Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_tpl_c16_aa_x1(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _chr_tpl_c16_aa_x1(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_tpl_c16_aa_x1(buf_ptr.add(16), mm_c16, st_ptr);
+    let r = unsafe { _chr_tpl_c16_aa_x1(buf_ptr.add(16), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -410,11 +412,11 @@ unsafe fn _chr_tpl_c16_aa_x4(
     mm_c16: MMB16Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_tpl_c16_aa_x2(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _chr_tpl_c16_aa_x2(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_tpl_c16_aa_x2(buf_ptr.add(16 * 2), mm_c16, st_ptr);
+    let r = unsafe { _chr_tpl_c16_aa_x2(buf_ptr.add(16 * 2), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -427,11 +429,11 @@ unsafe fn _chr_tpl_c16_aa_x8(
     mm_c16: MMB16Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_tpl_c16_aa_x4(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _chr_tpl_c16_aa_x4(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_tpl_c16_aa_x4(buf_ptr.add(16 * 4), mm_c16, st_ptr);
+    let r = unsafe { _chr_tpl_c16_aa_x4(buf_ptr.add(16 * 4), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -444,17 +446,18 @@ unsafe fn _chr_tpl_c32_uu_x1(
     mm_c32: MMB32Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm256_loadu_si256(buf_ptr as *const __m256i);
-    let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
-    let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
-    let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
-    let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    unsafe {
+        let mm_0 = _mm256_loadu_si256(buf_ptr as *const __m256i);
+        let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
+        let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
+        let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
+        let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    }
 }
 
 #[inline(always)]
@@ -463,17 +466,18 @@ unsafe fn _chr_tpl_c32_aa_x1(
     mm_c32: MMB32Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm256_load_si256(buf_ptr as *const __m256i);
-    let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
-    let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
-    let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
-    let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
-    let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
-    let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
-    let base = buf_ptr.usz_offset_from(st_ptr);
-    //
-    _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    unsafe {
+        let mm_0 = _mm256_load_si256(buf_ptr as *const __m256i);
+        let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
+        let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
+        let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
+        let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a) as u32;
+        let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b) as u32;
+        let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c) as u32;
+        let base = buf_ptr.usz_offset_from(st_ptr);
+        //
+        _return_chr_tpl(base, mask_0_a, mask_0_b, mask_0_c)
+    }
 }
 
 #[inline(always)]
@@ -482,11 +486,11 @@ unsafe fn _chr_tpl_c32_aa_x2(
     mm_c32: MMB32Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_tpl_c32_aa_x1(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _chr_tpl_c32_aa_x1(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_tpl_c32_aa_x1(buf_ptr.add(32), mm_c32, st_ptr);
+    let r = unsafe { _chr_tpl_c32_aa_x1(buf_ptr.add(32), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -499,11 +503,11 @@ unsafe fn _chr_tpl_c32_aa_x4(
     mm_c32: MMB32Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_tpl_c32_aa_x2(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _chr_tpl_c32_aa_x2(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_tpl_c32_aa_x2(buf_ptr.add(32 * 2), mm_c32, st_ptr);
+    let r = unsafe { _chr_tpl_c32_aa_x2(buf_ptr.add(32 * 2), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -516,11 +520,11 @@ unsafe fn _chr_tpl_c32_aa_x8(
     mm_c32: MMB32Tpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _chr_tpl_c32_aa_x4(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _chr_tpl_c32_aa_x4(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _chr_tpl_c32_aa_x4(buf_ptr.add(32 * 4), mm_c32, st_ptr);
+    let r = unsafe { _chr_tpl_c32_aa_x4(buf_ptr.add(32 * 4), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
