@@ -336,25 +336,26 @@ unsafe fn _rchr_qpl_c16_uu_x1(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm_loadu_si128(buf_ptr as *const __m128i);
-    let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
-    let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
-    let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
-    let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
-    let mask_0_a = _mm_movemask_epi8(mm_0_eq_a);
-    let mask_0_b = _mm_movemask_epi8(mm_0_eq_b);
-    let mask_0_c = _mm_movemask_epi8(mm_0_eq_c);
-    let mask_0_d = _mm_movemask_epi8(mm_0_eq_d);
-    let base = buf_ptr.usz_offset_from(st_ptr) + 16 - 1;
-    //
-    _return_rchr_qpl(
-        base,
-        mask_0_a as u16,
-        mask_0_b as u16,
-        mask_0_c as u16,
-        mask_0_d as u16,
-    )
+    unsafe {
+        let mm_0 = _mm_loadu_si128(buf_ptr as *const __m128i);
+        let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
+        let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
+        let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
+        let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
+        let mask_0_a = _mm_movemask_epi8(mm_0_eq_a);
+        let mask_0_b = _mm_movemask_epi8(mm_0_eq_b);
+        let mask_0_c = _mm_movemask_epi8(mm_0_eq_c);
+        let mask_0_d = _mm_movemask_epi8(mm_0_eq_d);
+        let base = buf_ptr.usz_offset_from(st_ptr) + 16 - 1;
+        //
+        _return_rchr_qpl(
+            base,
+            mask_0_a as u16,
+            mask_0_b as u16,
+            mask_0_c as u16,
+            mask_0_d as u16,
+        )
+    }
 }
 
 #[inline(always)]
@@ -363,25 +364,26 @@ unsafe fn _rchr_qpl_c16_aa_x1(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm_load_si128(buf_ptr as *const __m128i);
-    let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
-    let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
-    let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
-    let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
-    let mask_0_a = _mm_movemask_epi8(mm_0_eq_a);
-    let mask_0_b = _mm_movemask_epi8(mm_0_eq_b);
-    let mask_0_c = _mm_movemask_epi8(mm_0_eq_c);
-    let mask_0_d = _mm_movemask_epi8(mm_0_eq_d);
-    let base = buf_ptr.usz_offset_from(st_ptr) + 16 - 1;
-    //
-    _return_rchr_qpl(
-        base,
-        mask_0_a as u16,
-        mask_0_b as u16,
-        mask_0_c as u16,
-        mask_0_d as u16,
-    )
+    unsafe {
+        let mm_0 = _mm_load_si128(buf_ptr as *const __m128i);
+        let mm_0_eq_a = _mm_cmpeq_epi8(mm_0, mm_c16.v1);
+        let mm_0_eq_b = _mm_cmpeq_epi8(mm_0, mm_c16.v2);
+        let mm_0_eq_c = _mm_cmpeq_epi8(mm_0, mm_c16.v3);
+        let mm_0_eq_d = _mm_cmpeq_epi8(mm_0, mm_c16.v4);
+        let mask_0_a = _mm_movemask_epi8(mm_0_eq_a);
+        let mask_0_b = _mm_movemask_epi8(mm_0_eq_b);
+        let mask_0_c = _mm_movemask_epi8(mm_0_eq_c);
+        let mask_0_d = _mm_movemask_epi8(mm_0_eq_d);
+        let base = buf_ptr.usz_offset_from(st_ptr) + 16 - 1;
+        //
+        _return_rchr_qpl(
+            base,
+            mask_0_a as u16,
+            mask_0_b as u16,
+            mask_0_c as u16,
+            mask_0_d as u16,
+        )
+    }
 }
 
 #[inline(always)]
@@ -390,11 +392,11 @@ unsafe fn _rchr_qpl_c16_aa_x2(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _rchr_qpl_c16_aa_x1(buf_ptr.add(16), mm_c16, st_ptr);
+    let r = unsafe { _rchr_qpl_c16_aa_x1(buf_ptr.add(16), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _rchr_qpl_c16_aa_x1(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _rchr_qpl_c16_aa_x1(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -407,11 +409,11 @@ unsafe fn _rchr_qpl_c16_aa_x4(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _rchr_qpl_c16_aa_x2(buf_ptr.add(16 * 2), mm_c16, st_ptr);
+    let r = unsafe { _rchr_qpl_c16_aa_x2(buf_ptr.add(16 * 2), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _rchr_qpl_c16_aa_x2(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _rchr_qpl_c16_aa_x2(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -424,11 +426,11 @@ unsafe fn _rchr_qpl_c16_aa_x8(
     mm_c16: MMB16Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _rchr_qpl_c16_aa_x4(buf_ptr.add(16 * 4), mm_c16, st_ptr);
+    let r = unsafe { _rchr_qpl_c16_aa_x4(buf_ptr.add(16 * 4), mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _rchr_qpl_c16_aa_x4(buf_ptr, mm_c16, st_ptr);
+    let r = unsafe { _rchr_qpl_c16_aa_x4(buf_ptr, mm_c16, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -441,25 +443,26 @@ unsafe fn _rchr_qpl_c32_uu_x1(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm256_loadu_si256(buf_ptr as *const __m256i);
-    let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
-    let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
-    let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
-    let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
-    let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a);
-    let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b);
-    let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c);
-    let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d);
-    let base = buf_ptr.usz_offset_from(st_ptr) + 32 - 1;
-    //
-    _return_rchr_qpl(
-        base,
-        mask_0_a as u32,
-        mask_0_b as u32,
-        mask_0_c as u32,
-        mask_0_d as u32,
-    )
+    unsafe {
+        let mm_0 = _mm256_loadu_si256(buf_ptr as *const __m256i);
+        let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
+        let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
+        let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
+        let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
+        let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a);
+        let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b);
+        let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c);
+        let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d);
+        let base = buf_ptr.usz_offset_from(st_ptr) + 32 - 1;
+        //
+        _return_rchr_qpl(
+            base,
+            mask_0_a as u32,
+            mask_0_b as u32,
+            mask_0_c as u32,
+            mask_0_d as u32,
+        )
+    }
 }
 
 #[inline(always)]
@@ -468,25 +471,26 @@ unsafe fn _rchr_qpl_c32_aa_x1(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    //
-    let mm_0 = _mm256_load_si256(buf_ptr as *const __m256i);
-    let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
-    let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
-    let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
-    let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
-    let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a);
-    let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b);
-    let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c);
-    let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d);
-    let base = buf_ptr.usz_offset_from(st_ptr) + 32 - 1;
-    //
-    _return_rchr_qpl(
-        base,
-        mask_0_a as u32,
-        mask_0_b as u32,
-        mask_0_c as u32,
-        mask_0_d as u32,
-    )
+    unsafe {
+        let mm_0 = _mm256_load_si256(buf_ptr as *const __m256i);
+        let mm_0_eq_a = _mm256_cmpeq_epi8(mm_0, mm_c32.v1);
+        let mm_0_eq_b = _mm256_cmpeq_epi8(mm_0, mm_c32.v2);
+        let mm_0_eq_c = _mm256_cmpeq_epi8(mm_0, mm_c32.v3);
+        let mm_0_eq_d = _mm256_cmpeq_epi8(mm_0, mm_c32.v4);
+        let mask_0_a = _mm256_movemask_epi8(mm_0_eq_a);
+        let mask_0_b = _mm256_movemask_epi8(mm_0_eq_b);
+        let mask_0_c = _mm256_movemask_epi8(mm_0_eq_c);
+        let mask_0_d = _mm256_movemask_epi8(mm_0_eq_d);
+        let base = buf_ptr.usz_offset_from(st_ptr) + 32 - 1;
+        //
+        _return_rchr_qpl(
+            base,
+            mask_0_a as u32,
+            mask_0_b as u32,
+            mask_0_c as u32,
+            mask_0_d as u32,
+        )
+    }
 }
 
 #[inline(always)]
@@ -495,11 +499,11 @@ unsafe fn _rchr_qpl_c32_aa_x2(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _rchr_qpl_c32_aa_x1(buf_ptr.add(32), mm_c32, st_ptr);
+    let r = unsafe { _rchr_qpl_c32_aa_x1(buf_ptr.add(32), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _rchr_qpl_c32_aa_x1(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _rchr_qpl_c32_aa_x1(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -512,11 +516,11 @@ unsafe fn _rchr_qpl_c32_aa_x4(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _rchr_qpl_c32_aa_x2(buf_ptr.add(32 * 2), mm_c32, st_ptr);
+    let r = unsafe { _rchr_qpl_c32_aa_x2(buf_ptr.add(32 * 2), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _rchr_qpl_c32_aa_x2(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _rchr_qpl_c32_aa_x2(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
@@ -529,11 +533,11 @@ unsafe fn _rchr_qpl_c32_aa_x8(
     mm_c32: MMB32Qpl,
     st_ptr: *const u8,
 ) -> Option<usize> {
-    let r = _rchr_qpl_c32_aa_x4(buf_ptr.add(32 * 4), mm_c32, st_ptr);
+    let r = unsafe { _rchr_qpl_c32_aa_x4(buf_ptr.add(32 * 4), mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
-    let r = _rchr_qpl_c32_aa_x4(buf_ptr, mm_c32, st_ptr);
+    let r = unsafe { _rchr_qpl_c32_aa_x4(buf_ptr, mm_c32, st_ptr) };
     if r.is_some() {
         return r;
     }
