@@ -117,35 +117,47 @@ pub fn _std_memrnechr_qpl(buf: &[u8], by1: u8, by2: u8, by3: u8, by4: u8) -> Opt
         .rposition(|&x| x != by1 && x != by2 && x != by3 && x != by4)
 }
 
-pub fn _std_memchr_iter(buf: &[u8], by1: u8) -> StdMemchrSglIter {
+pub fn _std_memchr_iter(buf: &[u8], by1: u8) -> StdMemchrSglIter<'_> {
     StdMemchrSglIter::new(buf, by1)
 }
 
-pub fn _std_memrchr_iter(buf: &[u8], by1: u8) -> StdMemrchrSglIter {
+pub fn _std_memrchr_iter(buf: &[u8], by1: u8) -> StdMemrchrSglIter<'_> {
     StdMemrchrSglIter::new(buf, by1)
 }
 
-pub fn _std_memchr_dbl_iter(buf: &[u8], by1: u8, by2: u8) -> StdMemchrDblIter {
+pub fn _std_memchr_dbl_iter(buf: &[u8], by1: u8, by2: u8) -> StdMemchrDblIter<'_> {
     StdMemchrDblIter::new(buf, by1, by2)
 }
 
-pub fn _std_memrchr_dbl_iter(buf: &[u8], by1: u8, by2: u8) -> StdMemrchrDblIter {
+pub fn _std_memrchr_dbl_iter(buf: &[u8], by1: u8, by2: u8) -> StdMemrchrDblIter<'_> {
     StdMemrchrDblIter::new(buf, by1, by2)
 }
 
-pub fn _std_memchr_tpl_iter(buf: &[u8], by1: u8, by2: u8, by3: u8) -> StdMemchrTplIter {
+pub fn _std_memchr_tpl_iter(buf: &[u8], by1: u8, by2: u8, by3: u8) -> StdMemchrTplIter<'_> {
     StdMemchrTplIter::new(buf, by1, by2, by3)
 }
 
-pub fn _std_memrchr_tpl_iter(buf: &[u8], by1: u8, by2: u8, by3: u8) -> StdMemrchrTplIter {
+pub fn _std_memrchr_tpl_iter(buf: &[u8], by1: u8, by2: u8, by3: u8) -> StdMemrchrTplIter<'_> {
     StdMemrchrTplIter::new(buf, by1, by2, by3)
 }
 
-pub fn _std_memchr_qpl_iter(buf: &[u8], by1: u8, by2: u8, by3: u8, by4: u8) -> StdMemchrQplIter {
+pub fn _std_memchr_qpl_iter(
+    buf: &[u8],
+    by1: u8,
+    by2: u8,
+    by3: u8,
+    by4: u8,
+) -> StdMemchrQplIter<'_> {
     StdMemchrQplIter::new(buf, by1, by2, by3, by4)
 }
 
-pub fn _std_memrchr_qpl_iter(buf: &[u8], by1: u8, by2: u8, by3: u8, by4: u8) -> StdMemrchrQplIter {
+pub fn _std_memrchr_qpl_iter(
+    buf: &[u8],
+    by1: u8,
+    by2: u8,
+    by3: u8,
+    by4: u8,
+) -> StdMemrchrQplIter<'_> {
     StdMemrchrQplIter::new(buf, by1, by2, by3, by4)
 }
 
@@ -162,10 +174,10 @@ pub struct StdMemchrSglIter<'a> {
     needle1: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemchrSglIter<'a> {
+impl StdMemchrSglIter<'_> {
     #[allow(dead_code)]
     #[inline]
-    pub fn new(haystack: &[u8], needle1: u8) -> StdMemchrSglIter {
+    pub fn new(haystack: &[u8], needle1: u8) -> StdMemchrSglIter<'_> {
         StdMemchrSglIter {
             needle1,
             haystack,
@@ -173,7 +185,7 @@ impl<'a> StdMemchrSglIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemchrSglIter<'a> {
+impl Iterator for StdMemchrSglIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -197,7 +209,7 @@ impl<'a> Iterator for StdMemchrSglIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemchrSglIter<'a> {
+impl DoubleEndedIterator for StdMemchrSglIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position == 0 {
@@ -221,10 +233,10 @@ pub struct StdMemrchrSglIter<'a> {
     needle1: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemrchrSglIter<'a> {
+impl StdMemrchrSglIter<'_> {
     #[allow(dead_code)]
     #[inline]
-    pub fn new(haystack: &[u8], needle1: u8) -> StdMemrchrSglIter {
+    pub fn new(haystack: &[u8], needle1: u8) -> StdMemrchrSglIter<'_> {
         StdMemrchrSglIter {
             needle1,
             haystack,
@@ -232,7 +244,7 @@ impl<'a> StdMemrchrSglIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemrchrSglIter<'a> {
+impl Iterator for StdMemrchrSglIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -255,7 +267,7 @@ impl<'a> Iterator for StdMemrchrSglIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemrchrSglIter<'a> {
+impl DoubleEndedIterator for StdMemrchrSglIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position > self.haystack.len() {
@@ -281,10 +293,10 @@ pub struct StdMemchrDblIter<'a> {
     needle2: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemchrDblIter<'a> {
+impl StdMemchrDblIter<'_> {
     #[allow(dead_code)]
     #[inline]
-    pub fn new(haystack: &[u8], needle1: u8, needle2: u8) -> StdMemchrDblIter {
+    pub fn new(haystack: &[u8], needle1: u8, needle2: u8) -> StdMemchrDblIter<'_> {
         StdMemchrDblIter {
             needle1,
             needle2,
@@ -293,7 +305,7 @@ impl<'a> StdMemchrDblIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemchrDblIter<'a> {
+impl Iterator for StdMemchrDblIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -317,7 +329,7 @@ impl<'a> Iterator for StdMemchrDblIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemchrDblIter<'a> {
+impl DoubleEndedIterator for StdMemchrDblIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position == 0 {
@@ -346,10 +358,10 @@ pub struct StdMemrchrDblIter<'a> {
     needle2: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemrchrDblIter<'a> {
+impl StdMemrchrDblIter<'_> {
     #[allow(dead_code)]
     #[inline]
-    pub fn new(haystack: &[u8], needle1: u8, needle2: u8) -> StdMemrchrDblIter {
+    pub fn new(haystack: &[u8], needle1: u8, needle2: u8) -> StdMemrchrDblIter<'_> {
         StdMemrchrDblIter {
             needle1,
             needle2,
@@ -358,7 +370,7 @@ impl<'a> StdMemrchrDblIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemrchrDblIter<'a> {
+impl Iterator for StdMemrchrDblIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -385,7 +397,7 @@ impl<'a> Iterator for StdMemrchrDblIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemrchrDblIter<'a> {
+impl DoubleEndedIterator for StdMemrchrDblIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position > self.haystack.len() {
@@ -412,10 +424,10 @@ pub struct StdMemchrTplIter<'a> {
     needle3: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemchrTplIter<'a> {
+impl StdMemchrTplIter<'_> {
     #[allow(dead_code)]
     #[inline]
-    pub fn new(haystack: &[u8], needle1: u8, needle2: u8, needle3: u8) -> StdMemchrTplIter {
+    pub fn new(haystack: &[u8], needle1: u8, needle2: u8, needle3: u8) -> StdMemchrTplIter<'_> {
         StdMemchrTplIter {
             needle1,
             needle2,
@@ -425,7 +437,7 @@ impl<'a> StdMemchrTplIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemchrTplIter<'a> {
+impl Iterator for StdMemchrTplIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -454,7 +466,7 @@ impl<'a> Iterator for StdMemchrTplIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemchrTplIter<'a> {
+impl DoubleEndedIterator for StdMemchrTplIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position == 0 {
@@ -485,10 +497,10 @@ pub struct StdMemrchrTplIter<'a> {
     needle3: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemrchrTplIter<'a> {
+impl StdMemrchrTplIter<'_> {
     #[allow(dead_code)]
     #[inline]
-    pub fn new(haystack: &[u8], needle1: u8, needle2: u8, needle3: u8) -> StdMemrchrTplIter {
+    pub fn new(haystack: &[u8], needle1: u8, needle2: u8, needle3: u8) -> StdMemrchrTplIter<'_> {
         StdMemrchrTplIter {
             needle1,
             needle2,
@@ -498,7 +510,7 @@ impl<'a> StdMemrchrTplIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemrchrTplIter<'a> {
+impl Iterator for StdMemrchrTplIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -526,7 +538,7 @@ impl<'a> Iterator for StdMemrchrTplIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemrchrTplIter<'a> {
+impl DoubleEndedIterator for StdMemrchrTplIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position > self.haystack.len() {
@@ -559,7 +571,7 @@ pub struct StdMemchrQplIter<'a> {
     needle4: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemchrQplIter<'a> {
+impl StdMemchrQplIter<'_> {
     #[allow(dead_code)]
     #[inline]
     pub fn new(
@@ -568,7 +580,7 @@ impl<'a> StdMemchrQplIter<'a> {
         needle2: u8,
         needle3: u8,
         needle4: u8,
-    ) -> StdMemchrQplIter {
+    ) -> StdMemchrQplIter<'_> {
         StdMemchrQplIter {
             needle1,
             needle2,
@@ -579,7 +591,7 @@ impl<'a> StdMemchrQplIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemchrQplIter<'a> {
+impl Iterator for StdMemchrQplIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -609,7 +621,7 @@ impl<'a> Iterator for StdMemchrQplIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemchrQplIter<'a> {
+impl DoubleEndedIterator for StdMemchrQplIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position == 0 {
@@ -642,7 +654,7 @@ pub struct StdMemrchrQplIter<'a> {
     needle4: u8,
     position: usize, // 0: idx is -1, 1: idx is 0, 2: idx is 1
 }
-impl<'a> StdMemrchrQplIter<'a> {
+impl StdMemrchrQplIter<'_> {
     #[allow(dead_code)]
     #[inline]
     pub fn new(
@@ -651,7 +663,7 @@ impl<'a> StdMemrchrQplIter<'a> {
         needle2: u8,
         needle3: u8,
         needle4: u8,
-    ) -> StdMemrchrQplIter {
+    ) -> StdMemrchrQplIter<'_> {
         StdMemrchrQplIter {
             needle1,
             needle2,
@@ -662,7 +674,7 @@ impl<'a> StdMemrchrQplIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemrchrQplIter<'a> {
+impl Iterator for StdMemrchrQplIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -691,7 +703,7 @@ impl<'a> Iterator for StdMemrchrQplIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemrchrQplIter<'a> {
+impl DoubleEndedIterator for StdMemrchrQplIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position > self.haystack.len() {
@@ -733,7 +745,7 @@ impl<'a> StdMemmemIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemmemIter<'a> {
+impl Iterator for StdMemmemIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -757,7 +769,7 @@ impl<'a> Iterator for StdMemmemIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemmemIter<'a> {
+impl DoubleEndedIterator for StdMemmemIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position == 0 {
@@ -792,7 +804,7 @@ impl<'a> StdMemrmemIter<'a> {
         }
     }
 }
-impl<'a> Iterator for StdMemrmemIter<'a> {
+impl Iterator for StdMemrmemIter<'_> {
     type Item = usize;
     #[inline]
     fn next(&mut self) -> Option<usize> {
@@ -815,7 +827,7 @@ impl<'a> Iterator for StdMemrmemIter<'a> {
         (0, Some(self.haystack.len()))
     }
 }
-impl<'a> DoubleEndedIterator for StdMemrmemIter<'a> {
+impl DoubleEndedIterator for StdMemrmemIter<'_> {
     #[inline]
     fn next_back(&mut self) -> Option<Self::Item> {
         if self.position > self.haystack.len() {
