@@ -1,4 +1,4 @@
-use crate::{Operand, OperandConstraint, OperandKind};
+use crate::{Operand, OperandConstraint, OperandKind, OperandPos};
 
 pub struct Operands<'a>(pub &'a [Operand]);
 
@@ -33,6 +33,18 @@ impl<'a> Operands<'a> {
     pub fn fixed(&self) -> impl Iterator<Item = (usize, Operand)> + 'a {
         self.matches(|op| matches!(op.constraint(), OperandConstraint::FixedReg(_)))
     }
+
+    pub fn any_reg(&self) -> impl Iterator<Item = (usize, Operand)> + 'a {
+        self.matches(|op| matches!(op.constraint(), OperandConstraint::Reg))
+    }
+
+    pub fn late(&self) -> impl Iterator<Item = (usize, Operand)> + 'a {
+        self.matches(|op| op.pos() == OperandPos::Late)
+    }
+
+    pub fn early(&self) -> impl Iterator<Item = (usize, Operand)> + 'a {
+        self.matches(|op| op.pos() == OperandPos::Early)
+    }
 }
 
 impl<'a> core::ops::Index<usize> for Operands<'a> {
