@@ -1,9 +1,8 @@
 use super::CORE_TYPE_SORT;
 use crate::{
     encode_section, Alias, ComponentExportKind, ComponentOuterAliasKind, ComponentSection,
-    ComponentSectionId, ComponentTypeRef, CoreTypeEncoder, Encode, EntityType, ValType,
+    ComponentSectionId, ComponentTypeRef, Encode, EntityType, ValType,
 };
-use alloc::vec::Vec;
 
 /// Represents the type of a core module.
 #[derive(Debug, Clone, Default)]
@@ -21,7 +20,7 @@ impl ModuleType {
 
     /// Defines an import in this module type.
     pub fn import(&mut self, module: &str, name: &str, ty: EntityType) -> &mut Self {
-        self.bytes.push(0x00);
+        crate::component::imports::push_extern_name_byte(&mut self.bytes, name);
         module.encode(&mut self.bytes);
         name.encode(&mut self.bytes);
         ty.encode(&mut self.bytes);
@@ -37,10 +36,7 @@ impl ModuleType {
         self.bytes.push(0x01);
         self.num_added += 1;
         self.types_added += 1;
-        CoreTypeEncoder {
-            push_prefix_if_component_core_type: false,
-            bytes: &mut self.bytes,
-        }
+        CoreTypeEncoder(&mut self.bytes)
     }
 
     /// Defines an outer core type alias in this module type.
@@ -80,22 +76,31 @@ impl Encode for ModuleType {
 
 /// Used to encode core types.
 #[derive(Debug)]
-pub struct ComponentCoreTypeEncoder<'a>(pub(crate) &'a mut Vec<u8>);
+pub struct CoreTypeEncoder<'a>(pub(crate) &'a mut Vec<u8>);
+
+impl<'a> CoreTypeEncoder<'a> {
+    /// Define a function type.
+    pub fn function<P, R>(self, params: P, results: R)
+    where
+        P: IntoIterator<Item = ValType>,
+        P::IntoIter: ExactSizeIterator,
+        R: IntoIterator<Item = ValType>,
+        R::IntoIter: ExactSizeIterator,
+    {
+        let params = params.into_iter();
+        let results = results.into_iter();
+
+        self.0.push(0x60);
+        params.len().encode(self.0);
+        params.for_each(|p| p.encode(self.0));
+        results.len().encode(self.0);
+        results.for_each(|p| p.encode(self.0));
+    }
 
-impl<'a> ComponentCoreTypeEncoder<'a> {
     /// Define a module type.
     pub fn module(self, ty: &ModuleType) {
         ty.encode(self.0);
     }
-
-    /// Define any core type other than a module type.
-    #[must_use = "the encoder must be used to encode the type"]
-    pub fn core(self) -> CoreTypeEncoder<'a> {
-        CoreTypeEncoder {
-            bytes: self.0,
-            push_prefix_if_component_core_type: true,
-        }
-    }
 }
 
 /// An encoder for the core type section of WebAssembly components.
@@ -107,7 +112,7 @@ impl<'a> ComponentCoreTypeEncoder<'a> {
 ///
 /// let mut types = CoreTypeSection::new();
 ///
-/// types.ty().module(&ModuleType::new());
+/// types.module(&ModuleType::new());
 ///
 /// let mut component = Component::new();
 /// component.section(&types);
@@ -140,9 +145,29 @@ impl CoreTypeSection {
     ///
     /// The returned encoder must be finished before adding another type.
     #[must_use = "the encoder must be used to encode the type"]
-    pub fn ty(&mut self) -> ComponentCoreTypeEncoder<'_> {
+    pub fn ty(&mut self) -> CoreTypeEncoder<'_> {
         self.num_added += 1;
-        ComponentCoreTypeEncoder(&mut self.bytes)
+        CoreTypeEncoder(&mut self.bytes)
+    }
+
+    /// Define a function type in this type section.
+    pub fn function<P, R>(&mut self, params: P, results: R) -> &mut Self
+    where
+        P: IntoIterator<Item = ValType>,
+        P::IntoIter: ExactSizeIterator,
+        R: IntoIterator<Item = ValType>,
+        R::IntoIter: ExactSizeIterator,
+    {
+        self.ty().function(params, results);
+        self
+    }
+
+    /// Define a module type in this type section.
+    ///
+    /// Currently this is only used for core type sections in components.
+    pub fn module(&mut self, ty: &ModuleType) -> &mut Self {
+        self.ty().module(ty);
+        self
     }
 }
 
@@ -178,11 +203,11 @@ impl ComponentType {
     ///
     /// The returned encoder must be used before adding another definition.
     #[must_use = "the encoder must be used to encode the type"]
-    pub fn core_type(&mut self) -> ComponentCoreTypeEncoder {
+    pub fn core_type(&mut self) -> CoreTypeEncoder {
         self.bytes.push(0x00);
         self.num_added += 1;
         self.core_types_added += 1;
-        ComponentCoreTypeEncoder(&mut self.bytes)
+        CoreTypeEncoder(&mut self.bytes)
     }
 
     /// Define a type in this component type.
@@ -227,7 +252,8 @@ impl ComponentType {
     /// Defines an import in this component type.
     pub fn import(&mut self, name: &str, ty: ComponentTypeRef) -> &mut Self {
         self.bytes.push(0x03);
-        crate::encode_component_import_name(&mut self.bytes, name);
+        crate::component::imports::push_extern_name_byte(&mut self.bytes, name);
+        name.encode(&mut self.bytes);
         ty.encode(&mut self.bytes);
         self.num_added += 1;
         match ty {
@@ -241,7 +267,8 @@ impl ComponentType {
     /// Defines an export in this component type.
     pub fn export(&mut self, name: &str, ty: ComponentTypeRef) -> &mut Self {
         self.bytes.push(0x04);
-        crate::encode_component_export_name(&mut self.bytes, name);
+        crate::component::imports::push_extern_name_byte(&mut self.bytes, name);
+        name.encode(&mut self.bytes);
         ty.encode(&mut self.bytes);
         self.num_added += 1;
         match ty {
@@ -291,7 +318,7 @@ impl InstanceType {
     ///
     /// The returned encoder must be used before adding another definition.
     #[must_use = "the encoder must be used to encode the type"]
-    pub fn core_type(&mut self) -> ComponentCoreTypeEncoder {
+    pub fn core_type(&mut self) -> CoreTypeEncoder {
         self.0.core_type()
     }
 
@@ -352,43 +379,28 @@ impl Encode for InstanceType {
 
 /// Used to encode component function types.
 #[derive(Debug)]
-pub struct ComponentFuncTypeEncoder<'a> {
-    params_encoded: bool,
-    results_encoded: bool,
-    sink: &'a mut Vec<u8>,
-}
+pub struct ComponentFuncTypeEncoder<'a>(&'a mut Vec<u8>);
 
 impl<'a> ComponentFuncTypeEncoder<'a> {
     fn new(sink: &'a mut Vec<u8>) -> Self {
         sink.push(0x40);
-        Self {
-            params_encoded: false,
-            results_encoded: false,
-            sink,
-        }
+        Self(sink)
     }
 
     /// Defines named parameters.
     ///
     /// Parameters must be defined before defining results.
-    ///
-    /// # Panics
-    ///
-    /// This method will panic if the function is called twice since parameters
-    /// can only be encoded once.
     pub fn params<'b, P, T>(&mut self, params: P) -> &mut Self
     where
         P: IntoIterator<Item = (&'b str, T)>,
         P::IntoIter: ExactSizeIterator,
         T: Into<ComponentValType>,
     {
-        assert!(!self.params_encoded);
-        self.params_encoded = true;
         let params = params.into_iter();
-        params.len().encode(self.sink);
+        params.len().encode(self.0);
         for (name, ty) in params {
-            name.encode(self.sink);
-            ty.into().encode(self.sink);
+            name.encode(self.0);
+            ty.into().encode(self.0);
         }
         self
     }
@@ -396,30 +408,29 @@ impl<'a> ComponentFuncTypeEncoder<'a> {
     /// Defines a single unnamed result.
     ///
     /// This method cannot be used with `results`.
-    ///
-    /// # Panics
-    ///
-    /// This method will panic if the function is called twice, called before
-    /// the `params` method, or called in addition to the `results` method.
-    pub fn result(&mut self, ty: Option<ComponentValType>) -> &mut Self {
-        assert!(self.params_encoded);
-        assert!(!self.results_encoded);
-        self.results_encoded = true;
-        encode_resultlist(self.sink, ty);
+    pub fn result(&mut self, ty: impl Into<ComponentValType>) -> &mut Self {
+        self.0.push(0x00);
+        ty.into().encode(self.0);
         self
     }
-}
 
-pub(crate) fn encode_resultlist(sink: &mut Vec<u8>, ty: Option<ComponentValType>) {
-    match ty {
-        Some(ty) => {
-            sink.push(0x00);
-            ty.encode(sink);
-        }
-        None => {
-            sink.push(0x01);
-            sink.push(0x00);
+    /// Defines named results.
+    ///
+    /// This method cannot be used with `result`.
+    pub fn results<'b, R, T>(&mut self, results: R) -> &mut Self
+    where
+        R: IntoIterator<Item = (&'b str, T)>,
+        R::IntoIter: ExactSizeIterator,
+        T: Into<ComponentValType>,
+    {
+        self.0.push(0x01);
+        let results = results.into_iter();
+        results.len().encode(self.0);
+        for (name, ty) in results {
+            name.encode(self.0);
+            ty.into().encode(self.0);
         }
+        self
     }
 }
 
@@ -494,8 +505,6 @@ pub enum PrimitiveValType {
     Char,
     /// The type is a string.
     String,
-    /// Type for `error-context` added with async support in the component model.
-    ErrorContext,
 }
 
 impl Encode for PrimitiveValType {
@@ -514,7 +523,6 @@ impl Encode for PrimitiveValType {
             Self::F64 => 0x75,
             Self::Char => 0x74,
             Self::String => 0x73,
-            Self::ErrorContext => 0x64,
         });
     }
 }
@@ -593,13 +601,6 @@ impl ComponentDefinedTypeEncoder<'_> {
         ty.into().encode(self.0);
     }
 
-    /// Define a fixed size list type.
-    pub fn fixed_size_list(self, ty: impl Into<ComponentValType>, elements: u32) {
-        self.0.push(0x67);
-        ty.into().encode(self.0);
-        elements.encode(self.0);
-    }
-
     /// Define a tuple type.
     pub fn tuple<I, T>(self, types: I)
     where
@@ -667,18 +668,6 @@ impl ComponentDefinedTypeEncoder<'_> {
         self.0.push(0x68);
         idx.encode(self.0);
     }
-
-    /// Define a `future` type with the specified payload.
-    pub fn future(self, payload: Option<ComponentValType>) {
-        self.0.push(0x65);
-        payload.encode(self.0);
-    }
-
-    /// Define a `stream` type with the specified payload.
-    pub fn stream(self, payload: Option<ComponentValType>) {
-        self.0.push(0x66);
-        payload.encode(self.0);
-    }
 }
 
 /// An encoder for the type section of WebAssembly components.
@@ -699,7 +688,7 @@ impl ComponentDefinedTypeEncoder<'_> {
 ///       ("b", PrimitiveValType::String)
 ///     ]
 ///   )
-///   .result(Some(PrimitiveValType::String.into()));
+///   .result(PrimitiveValType::String);
 ///
 /// let mut component = Component::new();
 /// component.section(&types);
