@@ -9,6 +9,7 @@ use object::SectionFlags;
 use object::endian::Endianness;
 use object::read::{Object, ObjectSection, elf::ElfFile64};
 use wasmtime_environ::{Trap, lookup_trap_code, obj};
+use wasmtime_unwinder::ExceptionTable;
 
 /// Management of executable memory within a `MmapVec`
 ///
@@ -34,6 +35,7 @@ pub struct CodeMemory {
     wasm_data: Range<usize>,
     address_map_data: Range<usize>,
     stack_map_data: Range<usize>,
+    exception_data: Range<usize>,
     func_name_data: Range<usize>,
     info_data: Range<usize>,
     wasm_dwarf: Range<usize>,
@@ -119,6 +121,7 @@ impl CodeMemory {
         #[cfg(feature = "debug-builtins")]
         let mut has_native_debug_info = false;
         let mut trap_data = 0..0;
+        let mut exception_data = 0..0;
         let mut wasm_data = 0..0;
         let mut address_map_data = 0..0;
         let mut stack_map_data = 0..0;
@@ -168,6 +171,7 @@ impl CodeMemory {
                 obj::ELF_WASMTIME_ADDRMAP => address_map_data = range,
                 obj::ELF_WASMTIME_STACK_MAP => stack_map_data = range,
                 obj::ELF_WASMTIME_TRAPS => trap_data = range,
+                obj::ELF_WASMTIME_EXCEPTIONS => exception_data = range,
                 obj::ELF_NAME_DATA => func_name_data = range,
                 obj::ELF_WASMTIME_INFO => info_data = range,
                 obj::ELF_WASMTIME_DWARF => wasm_dwarf = range,
@@ -182,6 +186,17 @@ impl CodeMemory {
         #[cfg(not(has_host_compiler_backend))]
         let _ = &mut unwind;
 
+        // Ensure that the exception table is well-formed. This parser
+        // construction is cheap: it reads the header and validates
+        // ranges but nothing else. We do this only in debug-assertion
+        // builds because we otherwise require for safety that the
+        // compiled artifact is as-produced-by this version of
+        // Wasmtime, and we should always produce a correct exception
+        // table (i.e., we are not expecting untrusted data here).
+        if cfg!(debug_assertions) {
+            let _ = ExceptionTable::parse(&mmap[exception_data.clone()])?;
+        }
+
         Ok(Self {
             mmap,
             #[cfg(has_host_compiler_backend)]
@@ -200,6 +215,7 @@ impl CodeMemory {
             trap_data,
             address_map_data,
             stack_map_data,
+            exception_data,
             func_name_data,
             wasm_dwarf,
             info_data,
@@ -255,6 +271,12 @@ impl CodeMemory {
         &self.mmap[self.stack_map_data.clone()]
     }
 
+    /// Returns the encoded exception-tables section to pass to
+    /// `wasmtime_unwinder::ExceptionTable::parse`.
+    pub fn exception_tables(&self) -> &[u8] {
+        &self.mmap[self.exception_data.clone()]
+    }
+
     /// Returns the contents of the `ELF_WASMTIME_INFO` section, or an empty
     /// slice if it wasn't found.
     #[inline]
@@ -319,27 +341,10 @@ impl CodeMemory {
                     if !self.mmap.supports_virtual_memory() {
                         bail!("this target requires virtual memory to be enabled");
                     }
-
                     #[cfg(has_virtual_memory)]
-                    {
-                        let text = self.text();
-
-                        use wasmtime_jit_icache_coherence as icache_coherence;
-
-                        // Clear the newly allocated code from cache if the processor requires it
-                        //
-                        // Do this before marking the memory as R+X, technically we should be able to do it after
-                        // but there are some CPU's that have had errata about doing this with read only memory.
-                        icache_coherence::clear_cache(text.as_ptr().cast(), text.len())
-                            .expect("Failed cache clear");
-
-                        self.mmap
-                            .make_executable(self.text.clone(), self.enable_branch_protection)
-                            .context("unable to make memory executable")?;
-
-                        // Flush any in-flight instructions from the pipeline
-                        icache_coherence::pipeline_flush_mt().expect("Failed pipeline flush");
-                    }
+                    self.mmap
+                        .make_executable(self.text.clone(), self.enable_branch_protection)
+                        .context("unable to make memory executable")?;
                 }
             }
 
@@ -387,12 +392,14 @@ impl CodeMemory {
         {
             let text = self.text();
             let unwind_info = &self.mmap[self.unwind.clone()];
-            let registration = crate::runtime::vm::UnwindRegistration::new(
-                text.as_ptr(),
-                unwind_info.as_ptr(),
-                unwind_info.len(),
-            )
-            .context("failed to create unwind info registration")?;
+            let registration = unsafe {
+                crate::runtime::vm::UnwindRegistration::new(
+                    text.as_ptr(),
+                    unwind_info.as_ptr(),
+                    unwind_info.len(),
+                )
+                .context("failed to create unwind info registration")?
+            };
             self.unwind_registration = Some(registration);
             return Ok(());
         }
