@@ -4,16 +4,21 @@ use crate::component::storage::storage_as_slice;
 use crate::component::types::Type;
 use crate::component::values::Val;
 use crate::prelude::*;
-use crate::runtime::vm::component::{ComponentInstance, ResourceTables};
-use crate::runtime::vm::{Export, ExportFunction};
+use crate::runtime::vm::component::{ComponentInstance, InstanceFlags, ResourceTables};
+use crate::runtime::vm::{Export, VMFuncRef};
 use crate::store::StoreOpaque;
 use crate::{AsContext, AsContextMut, StoreContextMut, ValRaw};
+use anyhow::Context as _;
 use core::mem::{self, MaybeUninit};
 use core::ptr::NonNull;
 use wasmtime_environ::component::{
-    ExportIndex, InterfaceType, MAX_FLAT_PARAMS, MAX_FLAT_RESULTS, TypeFuncIndex, TypeTuple,
+    CanonicalOptions, ExportIndex, InterfaceType, MAX_FLAT_PARAMS, MAX_FLAT_RESULTS, TypeFuncIndex,
+    TypeTuple,
 };
 
+#[cfg(feature = "component-model-async")]
+use crate::component::concurrent::{self, AsAccessor, PreparedCall};
+
 mod host;
 mod options;
 mod typed;
@@ -21,12 +26,6 @@ pub use self::host::*;
 pub use self::options::*;
 pub use self::typed::*;
 
-#[repr(C)]
-union ParamsAndResults<Params: Copy, Return: Copy> {
-    params: Params,
-    ret: Return,
-}
-
 /// A WebAssembly component function which can be called.
 ///
 /// This type is the dual of [`wasmtime::Func`](crate::Func) for component
@@ -167,7 +166,7 @@ impl Func {
         Params: ComponentNamedList + Lower,
         Return: ComponentNamedList + Lift,
     {
-        let cx = InstanceType::new(instance.unwrap_or_else(|| &store[self.instance.id()]));
+        let cx = InstanceType::new(instance.unwrap_or_else(|| self.instance.id().get(store)));
         let ty = &cx.types[self.ty(store)];
 
         Params::typecheck(&InterfaceType::Tuple(ty.params), &cx)
@@ -181,7 +180,7 @@ impl Func {
     /// Get the parameter names and types for this function.
     pub fn params(&self, store: impl AsContext) -> Box<[(String, Type)]> {
         let store = store.as_context();
-        let instance = &store[self.instance.id()];
+        let instance = self.instance.id().get(store.0);
         let types = instance.component().types();
         let func_ty = &types[self.ty(store.0)];
         types[func_ty.params]
@@ -195,7 +194,7 @@ impl Func {
     /// Get the result types for this function.
     pub fn results(&self, store: impl AsContext) -> Box<[Type]> {
         let store = store.as_context();
-        let instance = &store[self.instance.id()];
+        let instance = self.instance.id().get(store.0);
         let types = instance.component().types();
         let ty = self.ty(store.0);
         types[types[ty].results]
@@ -206,7 +205,7 @@ impl Func {
     }
 
     fn ty(&self, store: &StoreOpaque) -> TypeFuncIndex {
-        let instance = &store[self.instance.id()];
+        let instance = self.instance.id().get(store);
         let (ty, _, _) = instance.component().export_lifted_function(self.index);
         ty
     }
@@ -278,13 +277,149 @@ impl Func {
         results: &mut [Val],
     ) -> Result<()> {
         let mut store = store.as_context_mut();
-        assert!(
-            store.0.async_support(),
-            "cannot use `call_async` without enabling async support in the config"
-        );
-        store
-            .on_fiber(|store| self.call_impl(store, params, results))
-            .await?
+
+        #[cfg(feature = "component-model-async")]
+        {
+            self.instance
+                .run_concurrent(&mut store, async |store| {
+                    self.call_concurrent_dynamic(store, params, results, false)
+                        .await
+                        .map(drop)
+                })
+                .await?
+        }
+        #[cfg(not(feature = "component-model-async"))]
+        {
+            assert!(
+                store.0.async_support(),
+                "cannot use `call_async` without enabling async support in the config"
+            );
+            store
+                .on_fiber(|store| self.call_impl(store, params, results))
+                .await?
+        }
+    }
+
+    fn check_params_results<T>(
+        &self,
+        store: StoreContextMut<T>,
+        params: &[Val],
+        results: &mut [Val],
+    ) -> Result<()> {
+        let param_tys = self.params(&store);
+        if param_tys.len() != params.len() {
+            bail!(
+                "expected {} argument(s), got {}",
+                param_tys.len(),
+                params.len(),
+            );
+        }
+
+        let result_tys = self.results(&store);
+
+        if result_tys.len() != results.len() {
+            bail!(
+                "expected {} result(s), got {}",
+                result_tys.len(),
+                results.len(),
+            );
+        }
+
+        Ok(())
+    }
+
+    /// Start a concurrent call to this function.
+    ///
+    /// Unlike [`Self::call`] and [`Self::call_async`] (both of which require
+    /// exclusive access to the store until the completion of the call), calls
+    /// made using this method may run concurrently with other calls to the same
+    /// instance.  In addition, the runtime will call the `post-return` function
+    /// (if any) automatically when the guest task completes -- no need to
+    /// explicitly call `Func::post_return` afterward.
+    ///
+    /// This returns a [`TaskExit`] representing the completion of the guest
+    /// task and any transitive subtasks it might create.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the store that the [`Accessor`] is derived from does not own
+    /// this function.
+    #[cfg(feature = "component-model-async")]
+    pub async fn call_concurrent(
+        self,
+        accessor: impl AsAccessor<Data: Send>,
+        params: &[Val],
+        results: &mut [Val],
+    ) -> Result<TaskExit> {
+        self.call_concurrent_dynamic(accessor, params, results, true)
+            .await
+    }
+
+    /// Internal helper function for `call_async` and `call_concurrent`.
+    #[cfg(feature = "component-model-async")]
+    async fn call_concurrent_dynamic(
+        self,
+        accessor: impl AsAccessor<Data: Send>,
+        params: &[Val],
+        results: &mut [Val],
+        call_post_return_automatically: bool,
+    ) -> Result<TaskExit> {
+        let result = accessor.as_accessor().with(|mut store| {
+            assert!(
+                store.as_context_mut().0.async_support(),
+                "cannot use `call_concurrent` when async support is not enabled on the config"
+            );
+            self.check_params_results(store.as_context_mut(), params, results)?;
+            let prepared = self.prepare_call_dynamic(
+                store.as_context_mut(),
+                params.to_vec(),
+                call_post_return_automatically,
+            )?;
+            concurrent::queue_call(store.as_context_mut(), prepared)
+        })?;
+
+        let (run_results, rx) = result.await?;
+        assert_eq!(run_results.len(), results.len());
+        for (result, slot) in run_results.into_iter().zip(results) {
+            *slot = result;
+        }
+        Ok(TaskExit(rx))
+    }
+
+    /// Calls `concurrent::prepare_call` with monomorphized functions for
+    /// lowering the parameters and lifting the result.
+    #[cfg(feature = "component-model-async")]
+    fn prepare_call_dynamic<'a, T: Send + 'static>(
+        self,
+        mut store: StoreContextMut<'a, T>,
+        params: Vec<Val>,
+        call_post_return_automatically: bool,
+    ) -> Result<PreparedCall<Vec<Val>>> {
+        let store = store.as_context_mut();
+
+        concurrent::prepare_call(
+            store,
+            self,
+            MAX_FLAT_PARAMS,
+            true,
+            call_post_return_automatically,
+            move |func, store, params_out| {
+                func.with_lower_context(store, call_post_return_automatically, |cx, ty| {
+                    Self::lower_args(cx, &params, ty, params_out)
+                })
+            },
+            move |func, store, results| {
+                let max_flat = if func.abi_async(store) {
+                    MAX_FLAT_PARAMS
+                } else {
+                    MAX_FLAT_RESULTS
+                };
+                let results = func.with_lift_context(store, |cx, ty| {
+                    Self::lift_results(cx, ty, results, max_flat)?.collect::<Result<Vec<_>>>()
+                })?;
+                Ok(Box::new(results))
+            },
+        )
     }
 
     fn call_impl(
@@ -293,63 +428,97 @@ impl Func {
         params: &[Val],
         results: &mut [Val],
     ) -> Result<()> {
-        let store = &mut store.as_context_mut();
+        let mut store = store.as_context_mut();
 
-        let param_tys = self.params(&store);
-        let result_tys = self.results(&store);
+        self.check_params_results(store.as_context_mut(), params, results)?;
 
-        if param_tys.len() != params.len() {
-            bail!(
-                "expected {} argument(s), got {}",
-                param_tys.len(),
-                params.len()
-            );
-        }
-        if result_tys.len() != results.len() {
-            bail!(
-                "expected {} results(s), got {}",
-                result_tys.len(),
-                results.len()
+        if self.abi_async(store.0) {
+            unreachable!(
+                "async-lifted exports should have failed validation \
+                 when `component-model-async` feature disabled"
             );
         }
 
-        self.call_raw(
-            store,
-            params,
-            |cx, params, params_ty, dst: &mut MaybeUninit<[ValRaw; MAX_FLAT_PARAMS]>| {
-                let params_ty = match params_ty {
-                    InterfaceType::Tuple(i) => &cx.types[i],
-                    _ => unreachable!(),
-                };
-                if params_ty.abi.flat_count(MAX_FLAT_PARAMS).is_some() {
-                    let dst = &mut unsafe {
-                        mem::transmute::<_, &mut [MaybeUninit<ValRaw>; MAX_FLAT_PARAMS]>(dst)
-                    }
-                    .iter_mut();
-
-                    params
-                        .iter()
-                        .zip(params_ty.types.iter())
-                        .try_for_each(|(param, ty)| param.lower(cx, *ty, dst))
-                } else {
-                    self.store_args(cx, &params_ty, params, dst)
-                }
-            },
-            |cx, results_ty, src: &[ValRaw; MAX_FLAT_RESULTS]| {
-                let results_ty = match results_ty {
-                    InterfaceType::Tuple(i) => &cx.types[i],
-                    _ => unreachable!(),
-                };
-                if results_ty.abi.flat_count(MAX_FLAT_RESULTS).is_some() {
-                    let mut flat = src.iter();
-                    for (ty, slot) in results_ty.types.iter().zip(results) {
-                        *slot = Val::lift(cx, *ty, &mut flat)?;
+        // SAFETY: the chosen representations of type parameters to `call_raw`
+        // here should be generally safe to work with:
+        //
+        // * parameters use `MaybeUninit<[MaybeUninit<ValRaw>; MAX_FLAT_PARAMS]>`
+        //   which represents the maximal possible number of parameters that can
+        //   be passed to lifted component functions. This is modeled with
+        //   `MaybeUninit` to represent how it all starts as uninitialized and
+        //   thus can't be safely read during lowering.
+        //
+        // * results are modeled as `[ValRaw; MAX_FLAT_RESULTS]` which
+        //   represents the maximal size of values that can be returned. Note
+        //   that if the function doesn't actually have a return value then the
+        //   `ValRaw` inside the array will have undefined contents. That is
+        //   safe in Rust, however, due to `ValRaw` being a `union`. The
+        //   contents should dynamically not be read due to the type of the
+        //   function used here matching the actual lift.
+        unsafe {
+            self.call_raw(
+                store,
+                |cx, ty, dst: &mut MaybeUninit<[MaybeUninit<ValRaw>; MAX_FLAT_PARAMS]>| {
+                    // SAFETY: it's safe to assume that
+                    // `MaybeUninit<array-of-maybe-uninit>` is initialized because
+                    // each individual element is still considered uninitialized.
+                    let dst: &mut [MaybeUninit<ValRaw>] = dst.assume_init_mut();
+                    Self::lower_args(cx, params, ty, dst)
+                },
+                |cx, results_ty, src: &[ValRaw; MAX_FLAT_RESULTS]| {
+                    let max_flat = MAX_FLAT_RESULTS;
+                    for (result, slot) in
+                        Self::lift_results(cx, results_ty, src, max_flat)?.zip(results)
+                    {
+                        *slot = result?;
                     }
                     Ok(())
-                } else {
-                    Self::load_results(cx, results_ty, results, &mut src.iter())
-                }
-            },
+                },
+            )
+        }
+    }
+
+    pub(crate) fn lifted_core_func(&self, store: &mut StoreOpaque) -> NonNull<VMFuncRef> {
+        let def = {
+            let instance = self.instance.id().get(store);
+            let (_ty, def, _options) = instance.component().export_lifted_function(self.index);
+            def.clone()
+        };
+        match self.instance.lookup_vmdef(store, &def) {
+            Export::Function(f) => f.vm_func_ref(store),
+            _ => unreachable!(),
+        }
+    }
+
+    pub(crate) fn post_return_core_func(&self, store: &StoreOpaque) -> Option<NonNull<VMFuncRef>> {
+        let instance = self.instance.id().get(store);
+        let component = instance.component();
+        let (_ty, _def, options) = component.export_lifted_function(self.index);
+        let post_return = component.env_component().options[options].post_return;
+        post_return.map(|i| instance.runtime_post_return(i))
+    }
+
+    pub(crate) fn abi_async(&self, store: &StoreOpaque) -> bool {
+        let instance = self.instance.id().get(store);
+        let component = instance.component();
+        let (_ty, _def, options) = component.export_lifted_function(self.index);
+        component.env_component().options[options].async_
+    }
+
+    pub(crate) fn abi_info<'a>(
+        &self,
+        store: &'a StoreOpaque,
+    ) -> (Options, InstanceFlags, TypeFuncIndex, &'a CanonicalOptions) {
+        let vminstance = self.instance.id().get(store);
+        let component = vminstance.component();
+        let (ty, _def, options_index) = component.export_lifted_function(self.index);
+        let raw_options = &component.env_component().options[options_index];
+        let options = Options::new_index(store, self.instance, options_index);
+        (
+            options,
+            vminstance.instance_flags(raw_options.instance),
+            ty,
+            raw_options,
         )
     }
 
@@ -361,13 +530,18 @@ impl Func {
     /// are what will be allocated on the stack for this function call. They
     /// should be appropriately sized for the lowering/lifting operation
     /// happening.
-    fn call_raw<T, Params: ?Sized, Return, LowerParams, LowerReturn>(
+    ///
+    /// # Safety
+    ///
+    /// The safety of this function relies on the correct definitions of the
+    /// `LowerParams` and `LowerReturn` type. They must match the type of `self`
+    /// for the params/results that are going to be produced. Additionally
+    /// these types must be representable with a sequence of `ValRaw` values.
+    unsafe fn call_raw<T, Return, LowerParams, LowerReturn>(
         &self,
-        store: &mut StoreContextMut<'_, T>,
-        params: &Params,
+        mut store: StoreContextMut<'_, T>,
         lower: impl FnOnce(
             &mut LowerContext<'_, T>,
-            &Params,
             InterfaceType,
             &mut MaybeUninit<LowerParams>,
         ) -> Result<()>,
@@ -377,21 +551,15 @@ impl Func {
         LowerParams: Copy,
         LowerReturn: Copy,
     {
-        let vminstance = self.instance.instance(store.0);
-        let (ty, def, options) = vminstance.component().export_lifted_function(self.index);
-        let export = match vminstance.lookup_def(store.0, def) {
-            Export::Function(f) => f,
-            _ => unreachable!(),
-        };
-        let component_instance = options.instance;
-        let memory = options
-            .memory
-            .map(|i| NonNull::new(vminstance.runtime_memory(i)).unwrap());
-        let realloc = options.realloc.map(|i| vminstance.runtime_realloc(i));
-        let options =
-            unsafe { Options::new(store.0.id(), memory, realloc, options.string_encoding) };
+        let export = self.lifted_core_func(store.0);
 
-        let space = &mut MaybeUninit::<ParamsAndResults<LowerParams, LowerReturn>>::uninit();
+        #[repr(C)]
+        union Union<Params: Copy, Return: Copy> {
+            params: Params,
+            ret: Return,
+        }
+
+        let space = &mut MaybeUninit::<Union<LowerParams, LowerReturn>>::uninit();
 
         // Double-check the size/alignment of `space`, just in case.
         //
@@ -408,87 +576,62 @@ impl Func {
         assert!(mem::align_of_val(map_maybe_uninit!(space.params)) == val_align);
         assert!(mem::align_of_val(map_maybe_uninit!(space.ret)) == val_align);
 
-        let types = vminstance.component().types().clone();
-        let mut flags = vminstance.instance_flags(component_instance);
-
-        unsafe {
-            // Test the "may enter" flag which is a "lock" on this instance.
-            // This is immediately set to `false` afterwards and note that
-            // there's no on-cleanup setting this flag back to true. That's an
-            // intentional design aspect where if anything goes wrong internally
-            // from this point on the instance is considered "poisoned" and can
-            // never be entered again. The only time this flag is set to `true`
-            // again is after post-return logic has completed successfully.
-            if !flags.may_enter() {
-                bail!(crate::Trap::CannotEnterComponent);
-            }
-            flags.set_may_enter(false);
-
-            debug_assert!(flags.may_leave());
-            flags.set_may_leave(false);
-            let instance_ptr = self.instance.instance_ptr(store.0).as_ptr();
-            let mut cx = LowerContext::new(store.as_context_mut(), &options, &types, instance_ptr);
+        self.with_lower_context(store.as_context_mut(), false, |cx, ty| {
             cx.enter_call();
-            let result = lower(
-                &mut cx,
-                params,
-                InterfaceType::Tuple(types[ty].params),
-                map_maybe_uninit!(space.params),
-            );
-            flags.set_may_leave(true);
-            result?;
+            lower(cx, ty, map_maybe_uninit!(space.params))
+        })?;
 
-            // This is unsafe as we are providing the guarantee that all the
-            // inputs are valid. The various pointers passed in for the function
-            // are all valid since they're coming from our store, and the
-            // `params_and_results` should have the correct layout for the core
-            // wasm function we're calling. Note that this latter point relies
-            // on the correctness of this module and `ComponentType`
-            // implementations, hence `ComponentType` being an `unsafe` trait.
+        // SAFETY: We are providing the guarantee that all the inputs are valid.
+        // The various pointers passed in for the function are all valid since
+        // they're coming from our store, and the `params_and_results` should
+        // have the correct layout for the core wasm function we're calling.
+        // Note that this latter point relies on the correctness of this module
+        // and `ComponentType` implementations, hence `ComponentType` being an
+        // `unsafe` trait.
+        unsafe {
             crate::Func::call_unchecked_raw(
-                store,
-                export.func_ref,
+                &mut store,
+                export,
                 NonNull::new(core::ptr::slice_from_raw_parts_mut(
                     space.as_mut_ptr().cast(),
                     mem::size_of_val(space) / mem::size_of::<ValRaw>(),
                 ))
                 .unwrap(),
             )?;
-
-            // Note that `.assume_init_ref()` here is unsafe but we're relying
-            // on the correctness of the structure of `LowerReturn` and the
-            // type-checking performed to acquire the `TypedFunc` to make this
-            // safe. It should be the case that `LowerReturn` is the exact
-            // representation of the return value when interpreted as
-            // `[ValRaw]`, and additionally they should have the correct types
-            // for the function we just called (which filled in the return
-            // values).
-            let ret = map_maybe_uninit!(space.ret).assume_init_ref();
-
-            // Lift the result into the host while managing post-return state
-            // here as well.
-            //
-            // After a successful lift the return value of the function, which
-            // is currently required to be 0 or 1 values according to the
-            // canonical ABI, is saved within the `Store`'s `FuncData`. This'll
-            // later get used in post-return.
-            flags.set_needs_post_return(true);
-            let val = lift(
-                &mut LiftContext::new(store.0, &options, &types, instance_ptr),
-                InterfaceType::Tuple(types[ty].results),
-                ret,
-            )?;
-            let ret_slice = storage_as_slice(ret);
-            (*instance_ptr).post_return_arg_set(
-                self.index,
-                match ret_slice.len() {
-                    0 => ValRaw::i32(0),
-                    1 => ret_slice[0],
-                    _ => unreachable!(),
-                },
-            );
-            return Ok(val);
         }
+
+        // SAFETY: We're relying on the correctness of the structure of
+        // `LowerReturn` and the type-checking performed to acquire the
+        // `TypedFunc` to make this safe. It should be the case that
+        // `LowerReturn` is the exact representation of the return value when
+        // interpreted as `[ValRaw]`, and additionally they should have the
+        // correct types for the function we just called (which filled in the
+        // return values).
+        let ret: &LowerReturn = unsafe { map_maybe_uninit!(space.ret).assume_init_ref() };
+
+        // Lift the result into the host while managing post-return state
+        // here as well.
+        //
+        // After a successful lift the return value of the function, which
+        // is currently required to be 0 or 1 values according to the
+        // canonical ABI, is saved within the `Store`'s `FuncData`. This'll
+        // later get used in post-return.
+        // flags.set_needs_post_return(true);
+        let val = self.with_lift_context(store.0, |cx, ty| lift(cx, ty, ret))?;
+
+        // SAFETY: it's a contract of this function that `LowerReturn` is an
+        // appropriate representation of the result of this function.
+        let ret_slice = unsafe { storage_as_slice(ret) };
+
+        self.instance.id().get_mut(store.0).post_return_arg_set(
+            self.index,
+            match ret_slice.len() {
+                0 => ValRaw::i32(0),
+                1 => ret_slice[0],
+                _ => unreachable!(),
+            },
+        );
+        return Ok(val);
     }
 
     /// Invokes the `post-return` canonical ABI option, if specified, after a
@@ -539,7 +682,7 @@ impl Func {
         let mut store = store.as_context_mut();
         assert!(
             store.0.async_support(),
-            "cannot use `call_async` without enabling async support in the config"
+            "cannot use `post_return_async` without enabling async support in the config"
         );
         // Future optimization opportunity: conditionally use a fiber here since
         // some func's post_return will not need the async context (i.e. end up
@@ -549,19 +692,18 @@ impl Func {
 
     fn post_return_impl(&self, mut store: impl AsContextMut) -> Result<()> {
         let mut store = store.as_context_mut();
+
         let index = self.index;
-        let vminstance = &store.0[self.instance.id()];
-        let (_ty, _def, options) = vminstance.component().export_lifted_function(index);
-        let post_return = options.post_return.map(|i| {
-            let func_ref = vminstance.runtime_post_return(i);
-            ExportFunction { func_ref }
-        });
-        let instance = self.instance.instance_ptr(store.0).as_ptr();
+        let vminstance = self.instance.id().get(store.0);
+        let component = vminstance.component();
+        let (_ty, _def, options) = component.export_lifted_function(index);
+        let post_return = self.post_return_core_func(store.0);
+        let mut flags =
+            vminstance.instance_flags(component.env_component().options[options].instance);
+        let mut instance = self.instance.id().get_mut(store.0);
+        let post_return_arg = instance.as_mut().post_return_arg_take(index);
 
         unsafe {
-            let post_return_arg = (*instance).post_return_arg_take(index);
-            let mut flags = (*instance).instance_flags(options.instance);
-
             // First assert that the instance is in a "needs post return" state.
             // This will ensure that the previous action on the instance was a
             // function call above. This flag is only set after a component
@@ -601,7 +743,7 @@ impl Func {
             if let Some(func) = post_return {
                 crate::Func::call_unchecked_raw(
                     &mut store,
-                    func.func_ref,
+                    func,
                     NonNull::new(core::ptr::slice_from_raw_parts(&post_return_arg, 1).cast_mut())
                         .unwrap(),
                 )?;
@@ -612,23 +754,46 @@ impl Func {
             // of the component.
             flags.set_may_enter(true);
 
-            let (calls, host_table, _) = store.0.component_resource_state();
+            let (calls, host_table, _, instance) = store
+                .0
+                .component_resource_state_with_instance(self.instance);
             ResourceTables {
-                calls,
                 host_table: Some(host_table),
-                guest: Some((*instance).guest_tables()),
+                calls,
+                guest: Some(instance.guest_tables()),
             }
             .exit_call()?;
         }
         Ok(())
     }
 
+    fn lower_args<T>(
+        cx: &mut LowerContext<'_, T>,
+        params: &[Val],
+        params_ty: InterfaceType,
+        dst: &mut [MaybeUninit<ValRaw>],
+    ) -> Result<()> {
+        let params_ty = match params_ty {
+            InterfaceType::Tuple(i) => &cx.types[i],
+            _ => unreachable!(),
+        };
+        if params_ty.abi.flat_count(MAX_FLAT_PARAMS).is_some() {
+            let dst = &mut dst.iter_mut();
+
+            params
+                .iter()
+                .zip(params_ty.types.iter())
+                .try_for_each(|(param, ty)| param.lower(cx, *ty, dst))
+        } else {
+            Self::store_args(cx, &params_ty, params, dst)
+        }
+    }
+
     fn store_args<T>(
-        &self,
         cx: &mut LowerContext<'_, T>,
         params_ty: &TypeTuple,
         args: &[Val],
-        dst: &mut MaybeUninit<[ValRaw; MAX_FLAT_PARAMS]>,
+        dst: &mut [MaybeUninit<ValRaw>],
     ) -> Result<()> {
         let size = usize::try_from(params_ty.abi.size32).unwrap();
         let ptr = cx.realloc(0, 0, params_ty.abi.align32, size)?;
@@ -638,17 +803,40 @@ impl Func {
             arg.store(cx, *ty, abi.next_field32_size(&mut offset))?;
         }
 
-        map_maybe_uninit!(dst[0]).write(ValRaw::i64(ptr as i64));
+        dst[0].write(ValRaw::i64(ptr as i64));
 
         Ok(())
     }
 
-    fn load_results(
-        cx: &mut LiftContext<'_>,
-        results_ty: &TypeTuple,
-        results: &mut [Val],
+    fn lift_results<'a, 'b>(
+        cx: &'a mut LiftContext<'b>,
+        results_ty: InterfaceType,
+        src: &'a [ValRaw],
+        max_flat: usize,
+    ) -> Result<Box<dyn Iterator<Item = Result<Val>> + 'a>> {
+        let results_ty = match results_ty {
+            InterfaceType::Tuple(i) => &cx.types[i],
+            _ => unreachable!(),
+        };
+        if results_ty.abi.flat_count(max_flat).is_some() {
+            let mut flat = src.iter();
+            Ok(Box::new(
+                results_ty
+                    .types
+                    .iter()
+                    .map(move |ty| Val::lift(cx, *ty, &mut flat)),
+            ))
+        } else {
+            let iter = Self::load_results(cx, results_ty, &mut src.iter())?;
+            Ok(Box::new(iter))
+        }
+    }
+
+    fn load_results<'a, 'b>(
+        cx: &'a mut LiftContext<'b>,
+        results_ty: &'a TypeTuple,
         src: &mut core::slice::Iter<'_, ValRaw>,
-    ) -> Result<()> {
+    ) -> Result<impl Iterator<Item = Result<Val>> + use<'a, 'b>> {
         // FIXME(#4311): needs to read an i64 for memory64
         let ptr = usize::try_from(src.next().unwrap().get_u32())?;
         if ptr % usize::try_from(results_ty.abi.align32)? != 0 {
@@ -662,11 +850,123 @@ impl Func {
             .ok_or_else(|| anyhow::anyhow!("pointer out of bounds of memory"))?;
 
         let mut offset = 0;
-        for (ty, slot) in results_ty.types.iter().zip(results) {
+        Ok(results_ty.types.iter().map(move |ty| {
             let abi = cx.types.canonical_abi(ty);
             let offset = abi.next_field32_size(&mut offset);
-            *slot = Val::load(cx, *ty, &bytes[offset..][..abi.size32 as usize])?;
+            Val::load(cx, *ty, &bytes[offset..][..abi.size32 as usize])
+        }))
+    }
+
+    #[cfg(feature = "component-model-async")]
+    pub(crate) fn instance(self) -> Instance {
+        self.instance
+    }
+
+    #[cfg(feature = "component-model-async")]
+    pub(crate) fn index(self) -> ExportIndex {
+        self.index
+    }
+
+    /// Creates a `LowerContext` using the configuration values of this lifted
+    /// function.
+    ///
+    /// The `lower` closure provided should perform the actual lowering and
+    /// return the result of the lowering operation which is then returned from
+    /// this function as well.
+    fn with_lower_context<T>(
+        self,
+        mut store: StoreContextMut<T>,
+        may_enter: bool,
+        lower: impl FnOnce(&mut LowerContext<T>, InterfaceType) -> Result<()>,
+    ) -> Result<()> {
+        let types = self.instance.id().get(store.0).component().types().clone();
+        let (options, mut flags, ty, _) = self.abi_info(store.0);
+
+        // Test the "may enter" flag which is a "lock" on this instance.
+        // This is immediately set to `false` afterwards and note that
+        // there's no on-cleanup setting this flag back to true. That's an
+        // intentional design aspect where if anything goes wrong internally
+        // from this point on the instance is considered "poisoned" and can
+        // never be entered again. The only time this flag is set to `true`
+        // again is after post-return logic has completed successfully.
+        unsafe {
+            if !flags.may_enter() {
+                bail!(crate::Trap::CannotEnterComponent);
+            }
+            flags.set_may_enter(false);
         }
+
+        // Perform the actual lowering, where while this is running the
+        // component is forbidden from calling imports.
+        unsafe {
+            debug_assert!(flags.may_leave());
+            flags.set_may_leave(false);
+        }
+        let mut cx = LowerContext::new(store.as_context_mut(), &options, &types, self.instance);
+        let result = lower(&mut cx, InterfaceType::Tuple(types[ty].params));
+        unsafe { flags.set_may_leave(true) };
+        result?;
+
+        // If this is an async function and `may_enter == true` then we're
+        // allowed to reenter the component at this point, and otherwise flag a
+        // post-return call being required as we're about to enter wasm and
+        // afterwards need a post-return.
+        unsafe {
+            if may_enter && options.async_() {
+                flags.set_may_enter(true);
+            } else {
+                flags.set_needs_post_return(true);
+            }
+        }
+
         Ok(())
     }
+
+    /// Creates a `LiftContext` using the configuration values with this lifted
+    /// function.
+    ///
+    /// The closure `lift` provided should actually perform the lift itself and
+    /// the result of that closure is returned from this function call as well.
+    fn with_lift_context<R>(
+        self,
+        store: &mut StoreOpaque,
+        lift: impl FnOnce(&mut LiftContext, InterfaceType) -> Result<R>,
+    ) -> Result<R> {
+        let (options, _flags, ty, _) = self.abi_info(store);
+        let mut cx = LiftContext::new(store, &options, self.instance);
+        let ty = InterfaceType::Tuple(cx.types[ty].results);
+        lift(&mut cx, ty)
+    }
+}
+
+/// Represents the completion of a task created using
+/// `[Typed]Func::call_concurrent`.
+///
+/// In general, a guest task may continue running after returning a value.
+/// Moreover, any given guest task may create its own subtasks before or after
+/// returning and may exit before some or all of those subtasks have finished
+/// running.  In that case, the still-running subtasks will be "reparented" to
+/// the nearest surviving caller, which may be the original host call.  The
+/// future returned by `TaskExit::block` will resolve once all transitive
+/// subtasks created directly or indirectly by the original call to
+/// `Instance::call_concurrent` have exited.
+#[cfg(feature = "component-model-async")]
+pub struct TaskExit(futures::channel::oneshot::Receiver<()>);
+
+#[cfg(feature = "component-model-async")]
+impl TaskExit {
+    /// Returns a future which will resolve once all transitive subtasks created
+    /// directly or indirectly by the original call to
+    /// `Instance::call_concurrent` have exited.
+    pub async fn block(self, accessor: impl AsAccessor<Data: Send>) {
+        // The current implementation makes no use of `accessor`, but future
+        // implementations might (e.g. by using a more efficient mechanism than
+        // a oneshot channel).
+        _ = accessor;
+
+        // We don't care whether the sender sent us a value or was dropped
+        // first; either one counts as a notification, so we ignore the result
+        // once the future resolves:
+        _ = self.0.await;
+    }
 }
