@@ -1,16 +1,14 @@
 use crate::prelude::*;
-use crate::runtime::vm::{
-    self, ExportGlobalKind, VMGlobalDefinition, VMGlobalKind, VMOpaqueContext,
-};
+use crate::runtime::vm::{self, VMGlobalDefinition, VMGlobalKind, VMOpaqueContext};
 use crate::{
-    AnyRef, AsContext, AsContextMut, ExternRef, Func, GlobalType, HeapType, Mutability, Ref,
-    RootedGcRefImpl, Val, ValType,
-    store::{AutoAssertNoGc, InstanceId, StoreId, StoreOpaque},
+    AnyRef, AsContext, AsContextMut, ExnRef, ExternRef, Func, GlobalType, HeapType, Mutability,
+    Ref, RootedGcRefImpl, Val, ValType,
+    store::{AutoAssertNoGc, InstanceId, StoreId, StoreInstanceId, StoreOpaque},
     trampoline::generate_global_export,
 };
 use core::ptr;
 use core::ptr::NonNull;
-use wasmtime_environ::{DefinedGlobalIndex, TypeTrace};
+use wasmtime_environ::DefinedGlobalIndex;
 
 /// A WebAssembly `global` value which can be read and written to.
 ///
@@ -99,10 +97,7 @@ impl Global {
         val.ensure_matches_ty(store, ty.content()).context(
             "type mismatch: initial value provided does not match the type of this global",
         )?;
-        unsafe {
-            let wasmtime_export = generate_global_export(store, ty, val)?;
-            Ok(Global::from_wasmtime_global(wasmtime_export, store))
-        }
+        generate_global_export(store, ty, val)
     }
 
     pub(crate) fn new_host(store: &StoreOpaque, index: DefinedGlobalIndex) -> Global {
@@ -144,10 +139,13 @@ impl Global {
     ///
     /// Panics if `store` does not own this global.
     pub fn get(&self, mut store: impl AsContextMut) -> Val {
+        let mut store = AutoAssertNoGc::new(store.as_context_mut().0);
+        self._get(&mut store)
+    }
+
+    pub(crate) fn _get(&self, store: &mut AutoAssertNoGc<'_>) -> Val {
         unsafe {
-            let store = store.as_context_mut();
-            let definition = self.definition(store.0).as_ref();
-            let mut store = AutoAssertNoGc::new(store.0);
+            let definition = self.definition(store).as_ref();
             match self._ty(&store).content() {
                 ValType::I32 => Val::from(*definition.as_i32()),
                 ValType::I64 => Val::from(*definition.as_i64()),
@@ -157,18 +155,31 @@ impl Global {
                 ValType::Ref(ref_ty) => {
                     let reference: Ref = match ref_ty.heap_type() {
                         HeapType::Func | HeapType::ConcreteFunc(_) => {
-                            Func::_from_raw(&mut store, definition.as_func_ref().cast()).into()
+                            Func::_from_raw(store, definition.as_func_ref().cast()).into()
                         }
 
                         HeapType::NoFunc => Ref::Func(None),
 
                         HeapType::Extern => Ref::Extern(definition.as_gc_ref().map(|r| {
-                            let r = store.unwrap_gc_store_mut().clone_gc_ref(r);
-                            ExternRef::from_cloned_gc_ref(&mut store, r)
+                            let r = store.clone_gc_ref(r);
+                            ExternRef::from_cloned_gc_ref(store, r)
                         })),
 
+                        HeapType::NoCont | HeapType::ConcreteCont(_) | HeapType::Cont => {
+                            // TODO(#10248) Required to support stack switching in the embedder API.
+                            unimplemented!()
+                        }
+
                         HeapType::NoExtern => Ref::Extern(None),
 
+                        HeapType::Exn | HeapType::ConcreteExn(_) => definition
+                            .as_gc_ref()
+                            .map(|r| {
+                                let r = store.clone_gc_ref(r);
+                                ExnRef::from_cloned_gc_ref(store, r)
+                            })
+                            .into(),
+
                         HeapType::Any
                         | HeapType::Eq
                         | HeapType::I31
@@ -178,11 +189,13 @@ impl Global {
                         | HeapType::ConcreteArray(_) => definition
                             .as_gc_ref()
                             .map(|r| {
-                                let r = store.unwrap_gc_store_mut().clone_gc_ref(r);
-                                AnyRef::from_cloned_gc_ref(&mut store, r)
+                                let r = store.clone_gc_ref(r);
+                                AnyRef::from_cloned_gc_ref(store, r)
                             })
                             .into(),
 
+                        HeapType::NoExn => Ref::Exn(None),
+
                         HeapType::None => Ref::Any(None),
                     };
                     debug_assert!(
@@ -207,21 +220,38 @@ impl Global {
     ///
     /// Panics if `store` does not own this global.
     pub fn set(&self, mut store: impl AsContextMut, val: Val) -> Result<()> {
-        let mut store = AutoAssertNoGc::new(store.as_context_mut().0);
+        self._set(store.as_context_mut().0, val)
+    }
+
+    pub(crate) fn _set(&self, store: &mut StoreOpaque, val: Val) -> Result<()> {
         let global_ty = self._ty(&store);
         if global_ty.mutability() != Mutability::Var {
             bail!("immutable global cannot be set");
         }
         val.ensure_matches_ty(&store, global_ty.content())
             .context("type mismatch: attempt to set global to value of wrong type")?;
+
+        // SAFETY: mutability and a type-check above makes this safe to perform.
+        unsafe { self.set_unchecked(store, &val) }
+    }
+
+    /// Sets this global to `val`.
+    ///
+    /// # Safety
+    ///
+    /// This function requires that `val` is of the correct type for this
+    /// global. Furthermore this requires that the global is mutable or this is
+    /// the first time the global is initialized.
+    pub(crate) unsafe fn set_unchecked(&self, store: &mut StoreOpaque, val: &Val) -> Result<()> {
+        let mut store = AutoAssertNoGc::new(store);
         unsafe {
             let definition = self.definition(&store).as_mut();
             match val {
-                Val::I32(i) => *definition.as_i32_mut() = i,
-                Val::I64(i) => *definition.as_i64_mut() = i,
-                Val::F32(f) => *definition.as_u32_mut() = f,
-                Val::F64(f) => *definition.as_u64_mut() = f,
-                Val::V128(i) => definition.set_u128(i.into()),
+                Val::I32(i) => *definition.as_i32_mut() = *i,
+                Val::I64(i) => *definition.as_i64_mut() = *i,
+                Val::F32(f) => *definition.as_u32_mut() = *f,
+                Val::F64(f) => *definition.as_u64_mut() = *f,
+                Val::V128(i) => definition.set_u128((*i).into()),
                 Val::FuncRef(f) => {
                     *definition.as_func_ref_mut() =
                         f.map_or(ptr::null_mut(), |f| f.vm_func_ref(&store).as_ptr().cast());
@@ -229,20 +259,36 @@ impl Global {
                 Val::ExternRef(e) => {
                     let new = match e {
                         None => None,
-                        #[cfg_attr(not(feature = "gc"), allow(unreachable_patterns))]
                         Some(e) => Some(e.try_gc_ref(&store)?.unchecked_copy()),
                     };
                     let new = new.as_ref();
-                    definition.write_gc_ref(store.unwrap_gc_store_mut(), new);
+                    definition.write_gc_ref(&mut store, new);
                 }
                 Val::AnyRef(a) => {
                     let new = match a {
                         None => None,
-                        #[cfg_attr(not(feature = "gc"), allow(unreachable_patterns))]
                         Some(a) => Some(a.try_gc_ref(&store)?.unchecked_copy()),
                     };
                     let new = new.as_ref();
-                    definition.write_gc_ref(store.unwrap_gc_store_mut(), new);
+                    definition.write_gc_ref(&mut store, new);
+                }
+                Val::ExnRef(e) => {
+                    let new = match e {
+                        None => None,
+                        Some(e) => Some(e.try_gc_ref(&store)?.unchecked_copy()),
+                    };
+                    let new = new.as_ref();
+                    definition.write_gc_ref(&mut store, new);
+                }
+                Val::ContRef(None) => {
+                    // Allow null continuation references for globals - these are just placeholders
+                    definition.write_gc_ref(&mut store, None);
+                }
+                Val::ContRef(Some(_)) => {
+                    // TODO(#10248): Implement non-null global continuation reference handling
+                    return Err(anyhow::anyhow!(
+                        "setting non-null continuation references in globals not yet supported"
+                    ));
                 }
             }
         }
@@ -264,32 +310,31 @@ impl Global {
         }
     }
 
-    pub(crate) unsafe fn from_wasmtime_global(
-        wasmtime_export: vm::ExportGlobal,
-        store: &StoreOpaque,
-    ) -> Global {
-        debug_assert!(
-            wasmtime_export
-                .global
-                .wasm_ty
-                .is_canonicalized_for_runtime_usage()
-        );
-        let (instance, kind) = match wasmtime_export.kind {
-            ExportGlobalKind::Host(index) => (0, VMGlobalKind::Host(index)),
-            ExportGlobalKind::Instance(vmctx, index) => (
-                vm::Instance::from_vmctx(vmctx, |i| i.id().as_u32()),
-                VMGlobalKind::Instance(index),
-            ),
-            #[cfg(feature = "component-model")]
-            ExportGlobalKind::ComponentFlags(vmctx, index) => (
-                vm::component::ComponentInstance::from_vmctx(vmctx, |i| i.id().as_u32()),
-                VMGlobalKind::ComponentFlags(index),
-            ),
-        };
+    pub(crate) fn from_host(store: StoreId, index: DefinedGlobalIndex) -> Global {
         Global {
-            store: store.id(),
-            instance,
-            kind,
+            store,
+            instance: 0,
+            kind: VMGlobalKind::Host(index),
+        }
+    }
+
+    pub(crate) fn from_core(instance: StoreInstanceId, index: DefinedGlobalIndex) -> Global {
+        Global {
+            store: instance.store_id(),
+            instance: instance.instance().as_u32(),
+            kind: VMGlobalKind::Instance(index),
+        }
+    }
+
+    #[cfg(feature = "component-model")]
+    pub(crate) fn from_component_flags(
+        instance: crate::component::store::StoreComponentInstanceId,
+        index: wasmtime_environ::component::RuntimeComponentInstanceIndex,
+    ) -> Global {
+        Global {
+            store: instance.store_id(),
+            instance: instance.instance().as_u32(),
+            kind: VMGlobalKind::ComponentFlags(index),
         }
     }
 
@@ -298,7 +343,7 @@ impl Global {
         match self.kind {
             VMGlobalKind::Instance(index) => {
                 let instance = InstanceId::from_u32(self.instance);
-                let module = store.instance(instance).module();
+                let module = store.instance(instance).env_module();
                 let index = module.global_index(index);
                 &module.globals[index]
             }
@@ -356,7 +401,7 @@ impl Global {
         match self.kind {
             VMGlobalKind::Instance(index) => {
                 let instance = InstanceId::from_u32(self.instance);
-                store.instance(instance).instance().global_ptr(index)
+                store.instance(instance).global_ptr(index)
             }
             VMGlobalKind::Host(index) => unsafe {
                 NonNull::from(&mut store.host_globals()[index].get().as_mut().global)
