@@ -99,11 +99,11 @@ mod trampolines {
                     {
                         $(shims!(@validate_param $pname $param);)*
 
-                        let ret = crate::runtime::vm::traphandlers::catch_unwind_and_record_trap(|| unsafe {
-                            ComponentInstance::from_vmctx(vmctx, |store, instance| {
+                        let ret = unsafe {
+                            ComponentInstance::enter_host_from_wasm(vmctx, |store, instance| {
                                 shims!(@invoke $name(store, instance,) $($pname)*)
                             })
-                        });
+                        };
                         shims!(@convert_ret ret $($pname: $param)*)
                     }
                     $(
@@ -664,9 +664,28 @@ fn backpressure_set(
     caller_instance: u32,
     enabled: u32,
 ) -> Result<()> {
-    instance.concurrent_state_mut(store).backpressure_set(
+    instance.concurrent_state_mut(store).backpressure_modify(
         RuntimeComponentInstanceIndex::from_u32(caller_instance),
-        enabled,
+        |_| Some(if enabled != 0 { 1 } else { 0 }),
+    )
+}
+
+#[cfg(feature = "component-model-async")]
+fn backpressure_modify(
+    store: &mut dyn VMStore,
+    instance: Instance,
+    caller_instance: u32,
+    increment: u8,
+) -> Result<()> {
+    instance.concurrent_state_mut(store).backpressure_modify(
+        RuntimeComponentInstanceIndex::from_u32(caller_instance),
+        |old| {
+            if increment != 0 {
+                old.checked_add(1)
+            } else {
+                old.checked_sub(1)
+            }
+        },
     )
 }
 
@@ -702,7 +721,8 @@ fn waitable_set_new(
     caller_instance: u32,
 ) -> Result<u32> {
     instance
-        .concurrent_state_mut(store)
+        .id()
+        .get_mut(store)
         .waitable_set_new(RuntimeComponentInstanceIndex::from_u32(caller_instance))
 }
 
@@ -735,7 +755,7 @@ fn waitable_set_drop(
     caller_instance: u32,
     set: u32,
 ) -> Result<()> {
-    instance.concurrent_state_mut(store).waitable_set_drop(
+    instance.id().get_mut(store).waitable_set_drop(
         RuntimeComponentInstanceIndex::from_u32(caller_instance),
         set,
     )
@@ -749,7 +769,7 @@ fn waitable_join(
     waitable: u32,
     set: u32,
 ) -> Result<()> {
-    instance.concurrent_state_mut(store).waitable_join(
+    instance.id().get_mut(store).waitable_join(
         RuntimeComponentInstanceIndex::from_u32(caller_instance),
         waitable,
         set,
@@ -757,8 +777,8 @@ fn waitable_join(
 }
 
 #[cfg(feature = "component-model-async")]
-fn yield_(store: &mut dyn VMStore, instance: Instance, async_: u8) -> Result<bool> {
-    instance.yield_(store, async_ != 0)
+fn thread_yield(store: &mut dyn VMStore, instance: Instance, cancellable: u8) -> Result<bool> {
+    instance.thread_yield(store, cancellable != 0)
 }
 
 #[cfg(feature = "component-model-async")]
@@ -768,7 +788,7 @@ fn subtask_drop(
     caller_instance: u32,
     task_id: u32,
 ) -> Result<()> {
-    instance.concurrent_state_mut(store).subtask_drop(
+    instance.id().get_mut(store).subtask_drop(
         RuntimeComponentInstanceIndex::from_u32(caller_instance),
         task_id,
     )
@@ -876,7 +896,7 @@ fn future_transfer(
     src_table: u32,
     dst_table: u32,
 ) -> Result<u32> {
-    instance.concurrent_state_mut(store).future_transfer(
+    instance.id().get_mut(store).future_transfer(
         src_idx,
         TypeFutureTableIndex::from_u32(src_table),
         TypeFutureTableIndex::from_u32(dst_table),
@@ -891,7 +911,7 @@ fn stream_transfer(
     src_table: u32,
     dst_table: u32,
 ) -> Result<u32> {
-    instance.concurrent_state_mut(store).stream_transfer(
+    instance.id().get_mut(store).stream_transfer(
         src_idx,
         TypeStreamTableIndex::from_u32(src_table),
         TypeStreamTableIndex::from_u32(dst_table),
@@ -909,7 +929,8 @@ fn error_context_transfer(
     let src_table = TypeComponentLocalErrorContextTableIndex::from_u32(src_table);
     let dst_table = TypeComponentLocalErrorContextTableIndex::from_u32(dst_table);
     instance
-        .concurrent_state_mut(store)
+        .id()
+        .get_mut(store)
         .error_context_transfer(src_idx, src_table, dst_table)
 }
 
@@ -927,7 +948,8 @@ unsafe impl HostResultHasUnwindSentinel for ResourcePair {
 #[cfg(feature = "component-model-async")]
 fn future_new(store: &mut dyn VMStore, instance: Instance, ty: u32) -> Result<ResourcePair> {
     instance
-        .concurrent_state_mut(store)
+        .id()
+        .get_mut(store)
         .future_new(TypeFutureTableIndex::from_u32(ty))
 }
 
@@ -975,7 +997,8 @@ fn future_cancel_write(
     async_: u8,
     writer: u32,
 ) -> Result<u32> {
-    instance.concurrent_state_mut(store).future_cancel_write(
+    instance.future_cancel_write(
+        store,
         TypeFutureTableIndex::from_u32(ty),
         async_ != 0,
         writer,
@@ -990,7 +1013,8 @@ fn future_cancel_read(
     async_: u8,
     reader: u32,
 ) -> Result<u32> {
-    instance.concurrent_state_mut(store).future_cancel_read(
+    instance.future_cancel_read(
+        store,
         TypeFutureTableIndex::from_u32(ty),
         async_ != 0,
         reader,
@@ -1024,7 +1048,8 @@ fn future_drop_readable(
 #[cfg(feature = "component-model-async")]
 fn stream_new(store: &mut dyn VMStore, instance: Instance, ty: u32) -> Result<ResourcePair> {
     instance
-        .concurrent_state_mut(store)
+        .id()
+        .get_mut(store)
         .stream_new(TypeStreamTableIndex::from_u32(ty))
 }
 
@@ -1076,7 +1101,8 @@ fn stream_cancel_write(
     async_: u8,
     writer: u32,
 ) -> Result<u32> {
-    instance.concurrent_state_mut(store).stream_cancel_write(
+    instance.stream_cancel_write(
+        store,
         TypeStreamTableIndex::from_u32(ty),
         async_ != 0,
         writer,
@@ -1091,7 +1117,8 @@ fn stream_cancel_read(
     async_: u8,
     reader: u32,
 ) -> Result<u32> {
-    instance.concurrent_state_mut(store).stream_cancel_read(
+    instance.stream_cancel_read(
+        store,
         TypeStreamTableIndex::from_u32(ty),
         async_ != 0,
         reader,
@@ -1213,7 +1240,7 @@ fn error_context_drop(
     ty: u32,
     err_ctx_handle: u32,
 ) -> Result<()> {
-    instance.concurrent_state_mut(store).error_context_drop(
+    instance.id().get_mut(store).error_context_drop(
         TypeComponentLocalErrorContextTableIndex::from_u32(ty),
         err_ctx_handle,
     )
