@@ -235,8 +235,9 @@ impl VMArrayRef {
                     Some(e) => Some(e.try_gc_ref(store)?.unchecked_copy()),
                     None => None,
                 };
-                store.gc_store_mut()?.write_gc_ref(&mut gc_ref, e.as_ref());
-                let data = store.gc_store_mut()?.gc_object_data(self.as_gc_ref());
+                let store = store.require_gc_store_mut()?;
+                store.write_gc_ref(&mut gc_ref, e.as_ref());
+                let data = store.gc_object_data(self.as_gc_ref());
                 data.write_u32(offset, gc_ref.map_or(0, |r| r.as_raw_u32()));
             }
             Val::AnyRef(a) => {
@@ -246,8 +247,9 @@ impl VMArrayRef {
                     Some(a) => Some(a.try_gc_ref(store)?.unchecked_copy()),
                     None => None,
                 };
-                store.gc_store_mut()?.write_gc_ref(&mut gc_ref, a.as_ref());
-                let data = store.gc_store_mut()?.gc_object_data(self.as_gc_ref());
+                let store = store.require_gc_store_mut()?;
+                store.write_gc_ref(&mut gc_ref, a.as_ref());
+                let data = store.gc_object_data(self.as_gc_ref());
                 data.write_u32(offset, gc_ref.map_or(0, |r| r.as_raw_u32()));
             }
             Val::ExnRef(e) => {
@@ -257,8 +259,9 @@ impl VMArrayRef {
                     Some(e) => Some(e.try_gc_ref(store)?.unchecked_copy()),
                     None => None,
                 };
-                store.gc_store_mut()?.write_gc_ref(&mut gc_ref, e.as_ref());
-                let data = store.gc_store_mut()?.gc_object_data(self.as_gc_ref());
+                let store = store.require_gc_store_mut()?;
+                store.write_gc_ref(&mut gc_ref, e.as_ref());
+                let data = store.gc_object_data(self.as_gc_ref());
                 data.write_u32(offset, gc_ref.map_or(0, |r| r.as_raw_u32()));
             }
 
@@ -267,12 +270,18 @@ impl VMArrayRef {
                     Some(f) => Some(SendSyncPtr::new(f.vm_func_ref(store))),
                     None => None,
                 };
-                let id = unsafe { store.gc_store_mut()?.func_ref_table.intern(func_ref) };
+                let store = store.require_gc_store_mut()?;
+                let id = unsafe { store.func_ref_table.intern(func_ref) };
                 store
-                    .gc_store_mut()?
                     .gc_object_data(self.as_gc_ref())
                     .write_u32(offset, id.into_raw());
             }
+            Val::ContRef(_) => {
+                // TODO(#10248): Implement array continuation reference element handling
+                return Err(anyhow::anyhow!(
+                    "setting continuation references in array elements not yet supported"
+                ));
+            }
         }
         Ok(())
     }
@@ -311,33 +320,27 @@ impl VMArrayRef {
     ) -> Result<()> {
         debug_assert!(val._matches_ty(&store, &ty.unpack())?);
         let offset = layout.elem_offset(index);
+        let gcstore = store.require_gc_store_mut()?;
         match val {
-            Val::I32(i) if ty.is_i8() => store
-                .gc_store_mut()?
+            Val::I32(i) if ty.is_i8() => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_i8(offset, truncate_i32_to_i8(i)),
-            Val::I32(i) if ty.is_i16() => store
-                .gc_store_mut()?
+            Val::I32(i) if ty.is_i16() => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_i16(offset, truncate_i32_to_i16(i)),
-            Val::I32(i) => store
-                .gc_store_mut()?
+            Val::I32(i) => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_i32(offset, i),
-            Val::I64(i) => store
-                .gc_store_mut()?
+            Val::I64(i) => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_i64(offset, i),
-            Val::F32(f) => store
-                .gc_store_mut()?
+            Val::F32(f) => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_u32(offset, f),
-            Val::F64(f) => store
-                .gc_store_mut()?
+            Val::F64(f) => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_u64(offset, f),
-            Val::V128(v) => store
-                .gc_store_mut()?
+            Val::V128(v) => gcstore
                 .gc_object_data(self.as_gc_ref())
                 .write_v128(offset, v),
 
@@ -350,7 +353,7 @@ impl VMArrayRef {
                     Some(x) => x.try_clone_gc_ref(store)?.as_raw_u32(),
                 };
                 store
-                    .gc_store_mut()?
+                    .require_gc_store_mut()?
                     .gc_object_data(self.as_gc_ref())
                     .write_u32(offset, x);
             }
@@ -360,7 +363,7 @@ impl VMArrayRef {
                     Some(x) => x.try_clone_gc_ref(store)?.as_raw_u32(),
                 };
                 store
-                    .gc_store_mut()?
+                    .require_gc_store_mut()?
                     .gc_object_data(self.as_gc_ref())
                     .write_u32(offset, x);
             }
@@ -370,7 +373,7 @@ impl VMArrayRef {
                     Some(x) => x.try_clone_gc_ref(store)?.as_raw_u32(),
                 };
                 store
-                    .gc_store_mut()?
+                    .require_gc_store_mut()?
                     .gc_object_data(self.as_gc_ref())
                     .write_u32(offset, x);
             }
@@ -380,12 +383,18 @@ impl VMArrayRef {
                     Some(f) => Some(SendSyncPtr::new(f.vm_func_ref(store))),
                     None => None,
                 };
-                let id = unsafe { store.gc_store_mut()?.func_ref_table.intern(func_ref) };
-                store
-                    .gc_store_mut()?
+                let gcstore = store.require_gc_store_mut()?;
+                let id = unsafe { gcstore.func_ref_table.intern(func_ref) };
+                gcstore
                     .gc_object_data(self.as_gc_ref())
                     .write_u32(offset, id.into_raw());
             }
+            Val::ContRef(_) => {
+                // TODO(#10248): Implement array continuation reference init handling
+                return Err(anyhow::anyhow!(
+                    "initializing continuation references in array elements not yet supported"
+                ));
+            }
         }
         Ok(())
     }
