@@ -76,28 +76,31 @@ impl TrapHandler {
         // panic in `lazy_per_thread_init` if the child attempts to invoke
         // WebAssembly.
         unsafe extern "C" fn child() {
-            CHILD_OF_FORKED_PROCESS = true;
+            unsafe {
+                CHILD_OF_FORKED_PROCESS = true;
+            }
         }
-        let rc = libc::pthread_atfork(None, None, Some(child));
-        assert_eq!(rc, 0, "failed to configure `pthread_atfork` handler");
-
-        // Allocate our WASMTIME_PORT and make sure that it can be sent to so we
-        // can receive exceptions.
-        let me = mach_task_self();
-        let kret = mach_port_allocate(me, MACH_PORT_RIGHT_RECEIVE, &raw mut WASMTIME_PORT);
-        assert_eq!(kret, KERN_SUCCESS, "failed to allocate port");
-        let kret =
-            mach_port_insert_right(me, WASMTIME_PORT, WASMTIME_PORT, MACH_MSG_TYPE_MAKE_SEND);
-        assert_eq!(kret, KERN_SUCCESS, "failed to insert right");
-
-        // Spin up our handler thread which will solely exist to service exceptions
-        // generated by other threads. Note that this is a background thread that
-        // we're not very interested in so it's detached here.
-        let thread = thread::spawn(|| handler_thread());
-
-        // Setup a SIGBUS handler which is used for printing that the stack was
-        // overflowed when a host overflows its fiber stack.
+        let thread;
         unsafe {
+            let rc = libc::pthread_atfork(None, None, Some(child));
+            assert_eq!(rc, 0, "failed to configure `pthread_atfork` handler");
+
+            // Allocate our WASMTIME_PORT and make sure that it can be sent to so we
+            // can receive exceptions.
+            let me = mach_task_self();
+            let kret = mach_port_allocate(me, MACH_PORT_RIGHT_RECEIVE, &raw mut WASMTIME_PORT);
+            assert_eq!(kret, KERN_SUCCESS, "failed to allocate port");
+            let kret =
+                mach_port_insert_right(me, WASMTIME_PORT, WASMTIME_PORT, MACH_MSG_TYPE_MAKE_SEND);
+            assert_eq!(kret, KERN_SUCCESS, "failed to insert right");
+
+            // Spin up our handler thread which will solely exist to service exceptions
+            // generated by other threads. Note that this is a background thread that
+            // we're not very interested in so it's detached here.
+            thread = thread::spawn(|| handler_thread());
+
+            // Setup a SIGBUS handler which is used for printing that the stack was
+            // overflowed when a host overflows its fiber stack.
             let mut handler: libc::sigaction = mem::zeroed();
             handler.sa_flags = libc::SA_SIGINFO | libc::SA_ONSTACK;
             handler.sa_sigaction = sigbus_handler as usize;
@@ -139,20 +142,23 @@ unsafe extern "C" fn sigbus_handler(
             Some(info) => info,
             None => return,
         };
-        let faulting_addr = (*siginfo).si_addr() as usize;
-        let start = info.async_guard_range.start;
-        let end = info.async_guard_range.end;
-        if start as usize <= faulting_addr && faulting_addr < end as usize {
-            super::signals::abort_stack_overflow();
+        unsafe {
+            let faulting_addr = (*siginfo).si_addr() as usize;
+            let range = &info.vm_store_context.as_ref().async_guard_range;
+            if range.start.addr() <= faulting_addr && faulting_addr < range.end.addr() {
+                super::signals::abort_stack_overflow();
+            }
         }
     });
 
-    super::signals::delegate_signal_to_previous_handler(
-        &raw mut PREV_SIGBUS,
-        signum,
-        siginfo,
-        context,
-    )
+    unsafe {
+        super::signals::delegate_signal_to_previous_handler(
+            &raw mut PREV_SIGBUS,
+            signum,
+            siginfo,
+            context,
+        )
+    }
 }
 
 // Note that this is copied from Gecko at
@@ -204,27 +210,33 @@ unsafe fn handler_thread() {
         // Block this thread reading a message from our port. This will block
         // until some thread throws an exception. Note that messages are all
         // expected to be exceptions here.
-        let mut request: ExceptionRequest = mem::zeroed();
-        let kret = mach_msg(
-            &mut request.body.Head,
-            MACH_RCV_MSG | MACH_RCV_INTERRUPT,
-            0,
-            mem::size_of_val(&request) as u32,
-            WASMTIME_PORT,
-            MACH_MSG_TIMEOUT_NONE,
-            MACH_PORT_NULL,
-        );
+        let mut request: ExceptionRequest = unsafe { mem::zeroed() };
+        let kret = unsafe {
+            mach_msg(
+                &mut request.body.Head,
+                MACH_RCV_MSG | MACH_RCV_INTERRUPT,
+                0,
+                mem::size_of_val(&request) as u32,
+                WASMTIME_PORT,
+                MACH_MSG_TIMEOUT_NONE,
+                MACH_PORT_NULL,
+            )
+        };
         if kret == MACH_RCV_PORT_CHANGED {
             // this port has been closed;
             break;
         }
         if kret != KERN_SUCCESS {
             eprintln!("mach_msg failed with {kret} ({kret:x})");
-            libc::abort();
+            unsafe {
+                libc::abort();
+            }
         }
         if request.body.Head.msgh_id != EXCEPTION_MSG_ID {
             eprintln!("unexpected msg header id {}", request.body.Head.msgh_id);
-            libc::abort();
+            unsafe {
+                libc::abort();
+            }
         }
 
         // Attempt to handle the exception below which will process the state
@@ -233,7 +245,7 @@ unsafe fn handler_thread() {
         // We unconditionally need to send a message back on our port after
         // this exception is received, and our reply code here dictates whether
         // the thread crashes or whether we continue execution of the thread.
-        let reply_code = if handle_exception(&mut request) {
+        let reply_code = if unsafe { handle_exception(&mut request) } {
             KERN_SUCCESS
         } else {
             KERN_FAILURE
@@ -242,24 +254,26 @@ unsafe fn handler_thread() {
         // This magic incantation to send a reply back to the kernel was
         // derived from the exc_server generated by
         // 'mig -v /usr/include/mach/mach_exc.defs'.
-        let mut reply: __Reply__exception_raise_t = mem::zeroed();
+        let mut reply: __Reply__exception_raise_t = unsafe { mem::zeroed() };
         reply.Head.msgh_bits =
             MACH_MSGH_BITS(request.body.Head.msgh_bits & MACH_MSGH_BITS_REMOTE_MASK, 0);
         reply.Head.msgh_size = mem::size_of_val(&reply) as u32;
         reply.Head.msgh_remote_port = request.body.Head.msgh_remote_port;
         reply.Head.msgh_local_port = MACH_PORT_NULL;
         reply.Head.msgh_id = request.body.Head.msgh_id + 100;
-        reply.NDR = NDR_record;
+        reply.NDR = unsafe { NDR_record };
         reply.RetCode = reply_code;
-        mach_msg(
-            &mut reply.Head,
-            MACH_SEND_MSG,
-            mem::size_of_val(&reply) as u32,
-            0,
-            MACH_PORT_NULL,
-            MACH_MSG_TIMEOUT_NONE,
-            MACH_PORT_NULL,
-        );
+        unsafe {
+            mach_msg(
+                &mut reply.Head,
+                MACH_SEND_MSG,
+                mem::size_of_val(&reply) as u32,
+                0,
+                MACH_PORT_NULL,
+                MACH_MSG_TIMEOUT_NONE,
+                MACH_PORT_NULL,
+            );
+        }
     }
 }
 
@@ -334,7 +348,9 @@ unsafe fn handle_exception(request: &mut ExceptionRequest) -> bool {
                 // not the end of the world anyway.
                 if state.__rsp % 16 == 0 {
                     state.__rsp -= 8;
-                    *(state.__rsp as *mut u64) = state.__rip;
+                    unsafe {
+                        *(state.__rsp as *mut u64) = state.__rip;
+                    }
                 }
                 state.__rip = unwind as u64;
                 state.__rdi = pc as u64;
@@ -370,7 +386,7 @@ unsafe fn handle_exception(request: &mut ExceptionRequest) -> bool {
                 state.__x[4] = trap as u64;
                 state.__pc = unwind as u64;
             };
-            let mut thread_state = mem::zeroed::<ThreadState>();
+            let mut thread_state = unsafe { mem::zeroed::<ThreadState>() };
         } else {
             compile_error!("unsupported target architecture");
         }
@@ -379,12 +395,14 @@ unsafe fn handle_exception(request: &mut ExceptionRequest) -> bool {
     // First up read our origin thread's state into the area defined above.
     let origin_thread = request.body.thread.name;
     let mut thread_state_count = ThreadState::count();
-    let kret = thread_get_state(
-        origin_thread,
-        thread_state_flavor,
-        &mut thread_state as *mut ThreadState as *mut u32,
-        &mut thread_state_count,
-    );
+    let kret = unsafe {
+        thread_get_state(
+            origin_thread,
+            thread_state_flavor,
+            &mut thread_state as *mut ThreadState as *mut u32,
+            &mut thread_state_count,
+        )
+    };
     if kret != KERN_SUCCESS {
         return false;
     }
@@ -421,12 +439,14 @@ unsafe fn handle_exception(request: &mut ExceptionRequest) -> bool {
         Some(addr) => (1, addr),
     };
     resume(&mut thread_state, pc as usize, fp, fault1, fault2, trap);
-    let kret = thread_set_state(
-        origin_thread,
-        thread_state_flavor,
-        &mut thread_state as *mut ThreadState as *mut u32,
-        thread_state_count,
-    );
+    let kret = unsafe {
+        thread_set_state(
+            origin_thread,
+            thread_state_flavor,
+            &mut thread_state as *mut ThreadState as *mut u32,
+            thread_state_count,
+        )
+    };
     kret == KERN_SUCCESS
 }
 
@@ -450,7 +470,9 @@ unsafe extern "C" fn unwind(pc: usize, fp: usize, fault1: usize, fault2: usize,
         state.take_jmp_buf()
     });
     debug_assert!(!jmp_buf.is_null());
-    wasmtime_longjmp(jmp_buf);
+    unsafe {
+        wasmtime_longjmp(jmp_buf);
+    }
 }
 
 /// Exceptions on macOS can be delivered to either thread-level or task-level
