@@ -15,15 +15,19 @@ mod signals;
 #[cfg(all(has_native_signals))]
 pub use self::signals::*;
 
+#[cfg(feature = "gc")]
+use crate::ThrownException;
 use crate::runtime::module::lookup_code;
 use crate::runtime::store::{ExecutorRef, StoreOpaque};
 use crate::runtime::vm::sys::traphandlers;
-use crate::runtime::vm::{InterpreterRef, VMContext, VMStoreContext, f32x4, f64x2, i8x16};
+use crate::runtime::vm::{InterpreterRef, VMContext, VMStore, VMStoreContext, f32x4, f64x2, i8x16};
 use crate::{EntryStoreContext, prelude::*};
 use crate::{StoreContextMut, WasmBacktrace};
 use core::cell::Cell;
 use core::num::NonZeroU32;
 use core::ptr::{self, NonNull};
+#[cfg(feature = "gc")]
+use wasmtime_unwinder::Handler;
 
 pub use self::backtrace::Backtrace;
 #[cfg(feature = "gc")]
@@ -62,36 +66,45 @@ fn lazy_per_thread_init() {
     traphandlers::lazy_per_thread_init();
 }
 
-/// Raises a preexisting trap and unwinds.
+/// Raises a preexisting trap or exception and unwinds.
 ///
-/// This function will execute the `longjmp` to make its way back to the
-/// original `setjmp` performed when wasm was entered. This is currently
-/// only called from the `raise` builtin of Wasmtime. This builtin is only used
-/// when the host returns back to wasm and indicates that a trap should be
-/// raised. In this situation the host has already stored trap information
-/// within the `CallThreadState` and this is the low-level operation to actually
-/// perform an unwind.
+/// If the preexisting state has registered a trap, this function will
+/// execute the `longjmp` to make its way back to the original
+/// `setjmp` performed when Wasm was entered. If the state has
+/// registered an exception, this function will perform the unwind
+/// action registered: either resetting PC, FP, and SP to the handler
+/// in the middle of the Wasm activation on the stack, or `longjmp`
+/// back to the entry from the host, if the exception is uncaught.
 ///
-/// This function won't be use with Pulley, for example, as the interpreter
-/// halts differently than native code. Additionally one day this will ideally
-/// be implemented by Cranelift itself without need of a libcall when Cranelift
-/// implements the exception handling proposal for example.
+/// This is currently only called from the `raise` builtin of
+/// Wasmtime. This builtin is only used when the host returns back to
+/// wasm and indicates that a trap or exception should be raised. In
+/// this situation the host has already stored trap or exception
+/// information within the `CallThreadState` and this is the low-level
+/// operation to actually perform an unwind.
+///
+/// Note that this function is used both for Pulley and for native execution.
+/// For Pulley this function will return and the interpreter will be
+/// responsible for handling the control-flow transfer. For native this
+/// function will not return as the control flow transfer will be handled
+/// internally.
 ///
 /// # Safety
 ///
 /// Only safe to call when wasm code is on the stack, aka `catch_traps` must
 /// have been previously called. Additionally no Rust destructors can be on the
 /// stack. They will be skipped and not executed.
-#[cfg(has_host_compiler_backend)]
-pub(super) unsafe fn raise_preexisting_trap() -> ! {
-    tls::with(|info| unsafe { info.unwrap().unwind() })
+pub(super) unsafe fn raise_preexisting_trap(store: &mut dyn VMStore) {
+    tls::with(|info| unsafe { info.unwrap().unwind(store) })
 }
 
-/// Invokes the closure `f` and returns a `bool` if it succeeded.
+/// Invokes the closure `f` and handles any error/panic/trap that happens
+/// within.
 ///
-/// This will invoke the closure `f` which returns a value that implements
-/// `HostResult`. This trait abstracts over how host values are translated to
-/// ABI values when going back into wasm. Some examples are:
+/// This will invoke the closure `f` with the provided `store` and the closure
+/// will return a value that implements `HostResult`. This trait abstracts over
+/// how host values are translated to ABI values when going back into wasm.
+/// Some examples are:
 ///
 /// * `T` - bare return types (not results) are simply returned as-is. No
 ///   `catch_unwind` happens as if a trap can't happen then the host shouldn't
@@ -109,7 +122,10 @@ pub(super) unsafe fn raise_preexisting_trap() -> ! {
 /// This function acts as a bridge between the two to appropriately handle
 /// encoding host values to Cranelift-understood ABIs via the `HostResult`
 /// trait.
-pub fn catch_unwind_and_record_trap<R>(f: impl FnOnce() -> R) -> R::Abi
+pub fn catch_unwind_and_record_trap<R>(
+    store: &mut dyn VMStore,
+    f: impl FnOnce(&mut dyn VMStore) -> R,
+) -> R::Abi
 where
     R: HostResult,
 {
@@ -117,13 +133,26 @@ where
     // return value is always provided and if unwind information is provided
     // (e.g. `ret` is a "false"-y value) then it's recorded in TLS for the
     // unwind operation that's about to happen from Cranelift-generated code.
-    let (ret, unwind) = R::maybe_catch_unwind(f);
+    let (ret, unwind) = R::maybe_catch_unwind(store, |store| f(store));
     if let Some(unwind) = unwind {
-        tls::with(|info| info.unwrap().record_unwind(unwind));
+        tls::with(|info| info.unwrap().record_unwind(store, unwind));
     }
     ret
 }
 
+/// Hook used by Pulley to configure the `jmp_buf` field in `CallThreadState`
+/// once it starts executing.
+pub(super) fn set_jmp_buf(jmp_buf: *const u8) {
+    tls::with(|info| {
+        let info = info.unwrap();
+        assert_eq!(
+            info.jmp_buf.get(),
+            CallThreadState::JMP_BUF_INTERPRETER_SENTINEL
+        );
+        info.jmp_buf.set(jmp_buf);
+    });
+}
+
 /// A trait used in conjunction with `catch_unwind_and_record_trap` to convert a
 /// Rust-based type to a specific ABI while handling traps/unwinds.
 ///
@@ -151,7 +180,10 @@ pub trait HostResult {
     /// back to wasm (which should be soon after calling this through
     /// `catch_unwind_and_record_trap`) then wasm will very quickly turn around
     /// and initiate an unwind (currently through `raise_preexisting_trap`).
-    fn maybe_catch_unwind(f: impl FnOnce() -> Self) -> (Self::Abi, Option<UnwindReason>);
+    fn maybe_catch_unwind(
+        store: &mut dyn VMStore,
+        f: impl FnOnce(&mut dyn VMStore) -> Self,
+    ) -> (Self::Abi, Option<UnwindReason>);
 }
 
 // Base case implementations that do not catch unwinds. These are for libcalls
@@ -164,8 +196,11 @@ macro_rules! host_result_no_catch {
         $(
             impl HostResult for $t {
                 type Abi = $t;
-                fn maybe_catch_unwind(f: impl FnOnce() -> $t) -> ($t, Option<UnwindReason>) {
-                    (f(), None)
+                fn maybe_catch_unwind(
+                    store: &mut dyn VMStore,
+                    f: impl FnOnce(&mut dyn VMStore) -> $t,
+                ) -> ($t, Option<UnwindReason>) {
+                    (f(store), None)
                 }
             }
         )*
@@ -187,8 +222,11 @@ host_result_no_catch! {
 
 impl HostResult for NonNull<u8> {
     type Abi = *mut u8;
-    fn maybe_catch_unwind(f: impl FnOnce() -> Self) -> (*mut u8, Option<UnwindReason>) {
-        (f().as_ptr(), None)
+    fn maybe_catch_unwind(
+        store: &mut dyn VMStore,
+        f: impl FnOnce(&mut dyn VMStore) -> Self,
+    ) -> (*mut u8, Option<UnwindReason>) {
+        (f(store).as_ptr(), None)
     }
 }
 
@@ -214,11 +252,14 @@ where
 {
     type Abi = T::Abi;
 
-    fn maybe_catch_unwind(f: impl FnOnce() -> Result<T, E>) -> (T::Abi, Option<UnwindReason>) {
+    fn maybe_catch_unwind(
+        store: &mut dyn VMStore,
+        f: impl FnOnce(&mut dyn VMStore) -> Result<T, E>,
+    ) -> (T::Abi, Option<UnwindReason>) {
         // First prepare the closure `f` as something that'll be invoked to
         // generate the return value of this function. This is the
         // conditionally, below, passed to `catch_unwind`.
-        let f = move || match f() {
+        let f = move || match f(store) {
             Ok(ret) => (ret.into_abi(), None),
             Err(reason) => (T::SENTINEL, Some(UnwindReason::Trap(reason.into()))),
         };
@@ -326,7 +367,8 @@ pub struct Trap {
     pub coredumpstack: Option<CoreDumpStack>,
 }
 
-/// Enumeration of different methods of raising a trap.
+/// Enumeration of different methods of raising a trap (or a sentinel
+/// for an exception).
 #[derive(Debug)]
 pub enum TrapReason {
     /// A user-raised trap through `raise_user_trap`.
@@ -357,11 +399,27 @@ pub enum TrapReason {
 
     /// A trap raised from a wasm libcall
     Wasm(wasmtime_environ::Trap),
+
+    /// An exception.
+    ///
+    /// Note that internally, exceptions are rooted on the Store, while
+    /// when crossing the public API, exceptions are held in a
+    /// `wasmtime::Exception` which contains a boxed root and implements
+    /// `Error`. This choice is intentional, to keep the internal
+    /// implementation lightweight and ensure the types represent only
+    /// allowable states.
+    #[cfg(feature = "gc")]
+    Exception,
 }
 
 impl From<Error> for TrapReason {
-    fn from(err: Error) -> Self {
-        TrapReason::User(err)
+    fn from(error: Error) -> Self {
+        #[cfg(feature = "gc")]
+        if error.is::<ThrownException>() {
+            return TrapReason::Exception;
+        }
+
+        TrapReason::User(error)
     }
 }
 
@@ -382,7 +440,7 @@ pub unsafe fn catch_traps<T, F>(
     store: &mut StoreContextMut<'_, T>,
     old_state: &mut EntryStoreContext,
     mut closure: F,
-) -> Result<(), Box<Trap>>
+) -> Result<()>
 where
     F: FnMut(NonNull<VMContext>, Option<InterpreterRef<'_>>) -> bool,
 {
@@ -428,16 +486,39 @@ where
         },
     });
 
-    return match result {
+    match result {
         Ok(x) => Ok(x),
-        Err((UnwindReason::Trap(reason), backtrace, coredumpstack)) => Err(Box::new(Trap {
-            reason,
+        #[cfg(feature = "gc")]
+        Err(UnwindState::UnwindToHost {
+            reason: UnwindReason::Trap(TrapReason::Exception),
+            backtrace: _,
+            coredump_stack: _,
+        }) => Err(ThrownException.into()),
+        Err(UnwindState::UnwindToHost {
+            reason: UnwindReason::Trap(reason),
             backtrace,
-            coredumpstack,
-        })),
+            coredump_stack,
+        }) => Err(crate::trap::from_runtime_box(
+            store.0,
+            Box::new(Trap {
+                reason,
+                backtrace,
+                coredumpstack: coredump_stack,
+            }),
+        )),
         #[cfg(all(feature = "std", panic = "unwind"))]
-        Err((UnwindReason::Panic(panic), _, _)) => std::panic::resume_unwind(panic),
-    };
+        Err(UnwindState::UnwindToHost {
+            reason: UnwindReason::Panic(panic),
+            ..
+        }) => std::panic::resume_unwind(panic),
+        #[cfg(feature = "gc")]
+        Err(UnwindState::UnwindToWasm { .. }) => {
+            unreachable!("We should not have returned to the host with an UnwindToWasm state");
+        }
+        Err(UnwindState::None) => {
+            unreachable!("We should not have gotten an error with no unwind state");
+        }
+    }
 }
 
 // Module to hide visibility of the `CallThreadState::prev` field and force
@@ -447,6 +528,42 @@ mod call_thread_state {
     use crate::EntryStoreContext;
     use crate::runtime::vm::{Unwind, VMStackChain};
 
+    /// Queued-up unwinding on the CallThreadState, ready to be
+    /// enacted by `unwind()`.
+    ///
+    /// This represents either a request to unwind to the entry point
+    /// from host (via longjmp), with associated data; or a request to
+    /// unwind into the middle of the Wasm action, e.g. when an
+    /// exception is caught.
+    pub enum UnwindState {
+        /// Unwind all the way to the entry from host to Wasm, using
+        /// `longjmp` to the `jmp_buf` on the `CallThreadState`.
+        UnwindToHost {
+            reason: UnwindReason,
+            backtrace: Option<Backtrace>,
+            coredump_stack: Option<CoreDumpStack>,
+        },
+        /// Unwind into Wasm. The exception destination has been
+        /// resolved. Note that the payload value is still not
+        /// specified, because it must remain rooted on the Store
+        /// until `unwind()` actually takes the value. The first
+        /// payload word in the underlying exception ABI is used to
+        /// send the raw `VMExnRef`.
+        #[cfg(feature = "gc")]
+        UnwindToWasm(Handler),
+        /// Do not unwind.
+        None,
+    }
+
+    impl UnwindState {
+        pub(super) fn is_none(&self) -> bool {
+            match self {
+                Self::None => true,
+                _ => false,
+            }
+        }
+    }
+
     /// Temporary state stored on the stack which is registered in the `tls`
     /// module below for calls into wasm.
     ///
@@ -469,7 +586,14 @@ mod call_thread_state {
     /// interior mutability here since that only gives access to
     /// `&CallThreadState`.
     pub struct CallThreadState {
-        pub(super) unwind: Cell<Option<(UnwindReason, Option<Backtrace>, Option<CoreDumpStack>)>>,
+        /// Unwind state set when initiating an unwind and read when
+        /// the control transfer occurs (after the `longjmp` point is
+        /// reached for host-code destinations and right when
+        /// performing the jump for Wasm-code destinations).
+        pub(super) unwind: Cell<UnwindState>,
+        /// Resume point established by `setjmp`, used when unwinding
+        /// all the way across the Wasm activation back to the entry
+        /// from host code. Traps and uncaught exceptions use this.
         pub(super) jmp_buf: Cell<*const u8>,
         #[cfg(all(has_native_signals))]
         pub(super) signal_handler: Option<*const SignalHandler>,
@@ -497,7 +621,7 @@ mod call_thread_state {
         fn drop(&mut self) {
             // Unwind information should not be present as it should have
             // already been processed.
-            debug_assert!(self.unwind.replace(None).is_none());
+            debug_assert!(self.unwind.replace(UnwindState::None).is_none());
         }
     }
 
@@ -510,7 +634,7 @@ mod call_thread_state {
             old_state: *mut EntryStoreContext,
         ) -> CallThreadState {
             CallThreadState {
-                unwind: Cell::new(None),
+                unwind: Cell::new(UnwindState::None),
                 unwinder: store.unwinder(),
                 jmp_buf: Cell::new(ptr::null()),
                 #[cfg(all(has_native_signals))]
@@ -525,8 +649,16 @@ mod call_thread_state {
         }
 
         /// Get the saved FP upon exit from Wasm for the previous `CallThreadState`.
+        ///
+        /// # Safety
+        ///
+        /// Requires that the saved last Wasm trampoline FP points to
+        /// a valid trampoline frame, or is null.
         pub unsafe fn old_last_wasm_exit_fp(&self) -> usize {
-            unsafe { (&*self.old_state).last_wasm_exit_fp }
+            let trampoline_fp = unsafe { (&*self.old_state).last_wasm_exit_trampoline_fp };
+            // SAFETY: `trampoline_fp` is either a valid FP from an
+            // active trampoline frame or is null.
+            unsafe { VMStoreContext::wasm_exit_fp_from_trampoline_fp(trampoline_fp) }
         }
 
         /// Get the saved PC upon exit from Wasm for the previous `CallThreadState`.
@@ -600,8 +732,8 @@ mod call_thread_state {
             unsafe {
                 let cx = self.vm_store_context.as_ref();
                 swap(
-                    &cx.last_wasm_exit_fp,
-                    &mut (*self.old_state).last_wasm_exit_fp,
+                    &cx.last_wasm_exit_trampoline_fp,
+                    &mut (*self.old_state).last_wasm_exit_trampoline_fp,
                 );
                 swap(
                     &cx.last_wasm_exit_pc,
@@ -618,18 +750,27 @@ mod call_thread_state {
 }
 pub use call_thread_state::*;
 
+#[cfg(feature = "gc")]
+use super::compute_handler;
+
 pub enum UnwindReason {
     #[cfg(all(feature = "std", panic = "unwind"))]
     Panic(Box<dyn std::any::Any + Send>),
     Trap(TrapReason),
 }
 
+impl<E> From<E> for UnwindReason
+where
+    E: Into<TrapReason>,
+{
+    fn from(value: E) -> UnwindReason {
+        UnwindReason::Trap(value.into())
+    }
+}
+
 impl CallThreadState {
     #[inline]
-    fn with(
-        mut self,
-        closure: impl FnOnce(&CallThreadState) -> bool,
-    ) -> Result<(), (UnwindReason, Option<Backtrace>, Option<CoreDumpStack>)> {
+    fn with(mut self, closure: impl FnOnce(&CallThreadState) -> bool) -> Result<(), UnwindState> {
         let succeeded = tls::set(&mut self, |me| closure(me));
         if succeeded {
             Ok(())
@@ -639,8 +780,8 @@ impl CallThreadState {
     }
 
     #[cold]
-    fn read_unwind(&self) -> (UnwindReason, Option<Backtrace>, Option<CoreDumpStack>) {
-        self.unwind.replace(None).unwrap()
+    fn read_unwind(&self) -> UnwindState {
+        self.unwind.replace(UnwindState::None)
     }
 
     /// Records the unwind information provided within this `CallThreadState`,
@@ -657,35 +798,71 @@ impl CallThreadState {
     ///
     /// Panics if unwind information has already been recorded as that should
     /// have been processed first.
-    fn record_unwind(&self, reason: UnwindReason) {
+    fn record_unwind(&self, store: &mut dyn VMStore, reason: UnwindReason) {
         if cfg!(debug_assertions) {
-            let prev = self.unwind.replace(None);
+            let prev = self.unwind.replace(UnwindState::None);
             assert!(prev.is_none());
         }
-        let (backtrace, coredump) = match &reason {
-            // Panics don't need backtraces. There is nowhere to attach the
-            // hypothetical backtrace to and it doesn't really make sense to try
-            // in the first place since this is a Rust problem rather than a
-            // Wasm problem.
+        let state = match reason {
             #[cfg(all(feature = "std", panic = "unwind"))]
-            UnwindReason::Panic(_) => (None, None),
+            UnwindReason::Panic(err) => {
+                // Panics don't need backtraces. There is nowhere to attach the
+                // hypothetical backtrace to and it doesn't really make sense to try
+                // in the first place since this is a Rust problem rather than a
+                // Wasm problem.
+                UnwindState::UnwindToHost {
+                    reason: UnwindReason::Panic(err),
+                    backtrace: None,
+                    coredump_stack: None,
+                }
+            }
+            // An unwind due to an already-set pending exception
+            // triggers the handler-search stack-walk. We store the
+            // resolved handler if one exists. In either case, the
+            // exception remains rooted in the Store until we actually
+            // perform the unwind, and then gets taken and becomes the
+            // payload at that point.
+            #[cfg(feature = "gc")]
+            UnwindReason::Trap(TrapReason::Exception) => {
+                // SAFETY: we are invoking `compute_handler()` while
+                // Wasm is on the stack and we have re-entered via a
+                // trampoline, as required by its stack-walking logic.
+                let handler = unsafe { compute_handler(store) };
+                match handler {
+                    Some(handler) => UnwindState::UnwindToWasm(handler),
+                    None => UnwindState::UnwindToHost {
+                        reason: UnwindReason::Trap(TrapReason::Exception),
+                        backtrace: None,
+                        coredump_stack: None,
+                    },
+                }
+            }
             // And if we are just propagating an existing trap that already has
             // a backtrace attached to it, then there is no need to capture a
             // new backtrace either.
             UnwindReason::Trap(TrapReason::User(err))
                 if err.downcast_ref::<WasmBacktrace>().is_some() =>
             {
-                (None, None)
+                UnwindState::UnwindToHost {
+                    reason: UnwindReason::Trap(TrapReason::User(err)),
+                    backtrace: None,
+                    coredump_stack: None,
+                }
             }
             UnwindReason::Trap(trap) => {
                 log::trace!("Capturing backtrace and coredump for {trap:?}");
-                (
-                    self.capture_backtrace(self.vm_store_context.as_ptr(), None),
-                    self.capture_coredump(self.vm_store_context.as_ptr(), None),
-                )
+                UnwindState::UnwindToHost {
+                    reason: UnwindReason::Trap(trap),
+                    backtrace: self.capture_backtrace(store.vm_store_context_mut(), None),
+                    coredump_stack: self.capture_coredump(store.vm_store_context_mut(), None),
+                }
             }
         };
-        self.unwind.set(Some((reason, backtrace, coredump)));
+
+        // Avoid unused-variable warning in non-exceptions/GC build.
+        let _ = store;
+
+        self.unwind.set(state);
     }
 
     /// Helper function to perform an actual unwinding operation.
@@ -696,14 +873,79 @@ impl CallThreadState {
     /// # Unsafety
     ///
     /// This function is not safe if the corresponding setjmp wasn't already
-    /// called. Additionally this isn't safe as it will skip all Rust
-    /// destructors on the stack, if there are any.
-    #[cfg(has_host_compiler_backend)]
-    unsafe fn unwind(&self) -> ! {
-        debug_assert!(!self.jmp_buf.get().is_null());
-        debug_assert!(self.jmp_buf.get() != CallThreadState::JMP_BUF_INTERPRETER_SENTINEL);
+    /// called. Additionally this isn't safe as it may skip all Rust
+    /// destructors on the stack, if there are any, for native executors as a
+    /// longjmp or equivalent will be used.
+    unsafe fn unwind(&self, store: &mut dyn VMStore) {
+        let unwind = self.unwind.replace(UnwindState::None);
+        match unwind {
+            UnwindState::UnwindToHost { .. } => {
+                // Keep the state around -- we will read it out again
+                // when we reach the entry-from-host side after the
+                // `longjmp`.
+                self.unwind.set(unwind);
+                unsafe {
+                    self.longjmp(store.executor());
+                }
+            }
+            #[cfg(feature = "gc")]
+            UnwindState::UnwindToWasm(handler) => {
+                // Take the pending exception at this time and use it as payload.
+                let payload1 = usize::try_from(
+                    store
+                        .take_pending_exception()
+                        .unwrap()
+                        .as_gc_ref()
+                        .as_raw_u32(),
+                )
+                .expect("GC ref does not fit in usize");
+                // We only use one of the payload words.
+                let payload2 = 0;
+                unsafe {
+                    self.resume_to_exception_handler(
+                        store.executor(),
+                        &handler,
+                        payload1,
+                        payload2,
+                    );
+                }
+            }
+            UnwindState::None => {
+                panic!("Attempting to unwind with no unwind state set.");
+            }
+        }
+    }
+
+    unsafe fn longjmp(&self, executor: ExecutorRef<'_>) {
+        let jmp_buf = self.jmp_buf.get();
+        debug_assert!(!jmp_buf.is_null());
+        debug_assert!(jmp_buf != CallThreadState::JMP_BUF_INTERPRETER_SENTINEL);
+
         unsafe {
-            traphandlers::wasmtime_longjmp(self.jmp_buf.get());
+            match executor {
+                ExecutorRef::Interpreter(r) => r.longjmp(jmp_buf),
+                #[cfg(has_host_compiler_backend)]
+                ExecutorRef::Native => traphandlers::wasmtime_longjmp(jmp_buf),
+            }
+        }
+    }
+
+    #[cfg(feature = "gc")]
+    unsafe fn resume_to_exception_handler(
+        &self,
+        executor: ExecutorRef<'_>,
+        handler: &Handler,
+        payload1: usize,
+        payload2: usize,
+    ) {
+        unsafe {
+            match executor {
+                ExecutorRef::Interpreter(r) => {
+                    r.resume_to_exception_handler(handler, payload1, payload2)
+                }
+                #[cfg(has_host_compiler_backend)]
+                ExecutorRef::Native => handler.resume_tailcc(payload1, payload2),
+            }
         }
     }
 
@@ -803,16 +1045,16 @@ impl CallThreadState {
         trap: wasmtime_environ::Trap,
     ) {
         let backtrace = self.capture_backtrace(self.vm_store_context.as_ptr(), Some((pc, fp)));
-        let coredump = self.capture_coredump(self.vm_store_context.as_ptr(), Some((pc, fp)));
-        self.unwind.set(Some((
-            UnwindReason::Trap(TrapReason::Jit {
+        let coredump_stack = self.capture_coredump(self.vm_store_context.as_ptr(), Some((pc, fp)));
+        self.unwind.set(UnwindState::UnwindToHost {
+            reason: UnwindReason::Trap(TrapReason::Jit {
                 pc,
                 faulting_addr,
                 trap,
             }),
             backtrace,
-            coredump,
-        )))
+            coredump_stack,
+        });
     }
 }
 
@@ -990,6 +1232,15 @@ pub(crate) mod tls {
         state: raw::Ptr,
     }
 
+    // SAFETY: This is a relatively unsafe unsafe block and not really all that
+    // well audited. The general idea is that the linked list of activations
+    // owned by `self.state` are safe to send to other threads, but that relies
+    // on everything internally being safe as well as stack variables and such.
+    // This is more-or-less tied to the very large comment in `fiber.rs` about
+    // `unsafe impl Send` there.
+    #[cfg(feature = "async")]
+    unsafe impl Send for AsyncWasmCallState {}
+
     #[cfg(feature = "async")]
     impl AsyncWasmCallState {
         /// Creates new state that initially starts as null.
