@@ -233,12 +233,20 @@ macro_rules! foreach_builtin_function {
             // denote the continuation being `None`, `init_contref`
             // may be 0.
             #[cfg(feature = "stack-switching")]
-            table_grow_cont_obj(vmctx: vmctx, table: u32, delta: u64, init_contref: pointer, init_revision: u64) -> pointer;
+            table_grow_cont_obj(vmctx: vmctx, table: u32, delta: u64, init_contref: pointer, init_revision: size) -> pointer;
 
             // `value_contref` and `value_revision` together encode
             // the Option<VMContObj>, as in previous libcall.
             #[cfg(feature = "stack-switching")]
-            table_fill_cont_obj(vmctx: vmctx, table: u32, dst: u64, value_contref: pointer, value_revision: u64, len: u64) -> bool;
+            table_fill_cont_obj(vmctx: vmctx, table: u32, dst: u64, value_contref: pointer, value_revision: size, len: u64) -> bool;
+
+            // Return the instance ID for a given vmctx.
+            #[cfg(feature = "gc")]
+            get_instance_id(vmctx: vmctx) -> u32;
+
+            // Throw an exception.
+            #[cfg(feature = "gc")]
+            throw_ref(vmctx: vmctx, exnref: u32) -> bool;
         }
     };
 }
@@ -247,8 +255,11 @@ macro_rules! foreach_builtin_function {
 /// `ComponentBuiltinFunctionIndex` using the iterator macro, e.g.
 /// `foreach_builtin_function`, as the way to generate accessor methods.
 macro_rules! declare_builtin_index {
-    ($index_name:ident, $iter:ident) => {
-        /// An index type for builtin functions.
+    (
+        $(#[$attr:meta])*
+        pub struct $index_name:ident : $for_each_builtin:ident ;
+    ) => {
+        $(#[$attr])*
         #[derive(Copy, Clone, Debug, PartialEq, Eq, PartialOrd, Ord, Hash)]
         pub struct $index_name(u32);
 
@@ -264,7 +275,7 @@ macro_rules! declare_builtin_index {
                 self.0
             }
 
-            $iter!(declare_builtin_index_constructors);
+            $for_each_builtin!(declare_builtin_index_constructors);
         }
     };
 }
@@ -332,7 +343,10 @@ macro_rules! declare_builtin_index_constructors {
 }
 
 // Define `struct BuiltinFunctionIndex`
-declare_builtin_index!(BuiltinFunctionIndex, foreach_builtin_function);
+declare_builtin_index! {
+    /// An index type for builtin functions.
+    pub struct BuiltinFunctionIndex : foreach_builtin_function;
+}
 
 /// Return value of [`BuiltinFunctionIndex::trap_sentinel`].
 pub enum TrapSentinel {
@@ -422,6 +436,8 @@ impl BuiltinFunctionIndex {
 
             (@get cont_new pointer) => (TrapSentinel::Negative);
 
+            (@get get_instance_id u32) => (return None);
+
             // Bool-returning functions use `false` as an indicator of a trap.
             (@get $name:ident bool) => (TrapSentinel::Falsy);
 
