@@ -3,8 +3,8 @@
 
 use crate::prelude::*;
 use crate::{
-    BuiltinFunctionIndex, DefinedFuncIndex, FlagValue, FuncIndex, FunctionLoc, ObjectKind,
-    PrimaryMap, StaticModuleIndex, TripleExt, WasmError, WasmFuncType,
+    DefinedFuncIndex, FlagValue, FunctionLoc, ObjectKind, PrimaryMap, StaticModuleIndex, TripleExt,
+    WasmError, WasmFuncType,
 };
 use crate::{Tunables, obj};
 use anyhow::Result;
@@ -17,6 +17,7 @@ use std::path;
 use std::sync::Arc;
 
 mod address_map;
+mod key;
 mod module_artifacts;
 mod module_environ;
 mod module_types;
@@ -24,6 +25,7 @@ mod stack_maps;
 mod trap_encoding;
 
 pub use self::address_map::*;
+pub use self::key::*;
 pub use self::module_artifacts::*;
 pub use self::module_environ::*;
 pub use self::module_types::*;
@@ -70,19 +72,6 @@ impl core::error::Error for CompileError {
     }
 }
 
-/// What relocations can be applied against.
-///
-/// Each wasm function may refer to various other `RelocationTarget` entries.
-#[derive(Copy, Clone, Debug, PartialEq, Eq)]
-pub enum RelocationTarget {
-    /// This is a reference to another defined wasm function in the same module.
-    Wasm(FuncIndex),
-    /// This is a reference to a trampoline for a builtin function.
-    Builtin(BuiltinFunctionIndex),
-    /// A pulley->host call from the interpreter.
-    PulleyHostcall(u32),
-}
-
 /// Implementation of an incremental compilation's key/value cache store.
 ///
 /// In theory, this could just be Cranelift's `CacheKvStore` trait, but it is not as we want to
@@ -191,7 +180,7 @@ pub struct CompiledFunctionBody {
 ///
 /// The diagram below depicts typical usage of this trait:
 ///
-/// ```ignore
+/// ```text
 ///                     +------+
 ///                     | Wasm |
 ///                     +------+
@@ -259,7 +248,7 @@ pub trait Compiler: Send + Sync {
     fn compile_function(
         &self,
         translation: &ModuleTranslation<'_>,
-        index: DefinedFuncIndex,
+        key: FuncKey,
         data: FunctionBodyData<'_>,
         types: &ModuleTypesBuilder,
         symbol: &str,
@@ -274,7 +263,7 @@ pub trait Compiler: Send + Sync {
         &self,
         translation: &ModuleTranslation<'_>,
         types: &ModuleTypesBuilder,
-        index: DefinedFuncIndex,
+        key: FuncKey,
         symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
@@ -286,6 +275,7 @@ pub trait Compiler: Send + Sync {
     fn compile_wasm_to_array_trampoline(
         &self,
         wasm_func_ty: &WasmFuncType,
+        key: FuncKey,
         symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
@@ -300,7 +290,7 @@ pub trait Compiler: Send + Sync {
     /// call.
     fn compile_wasm_to_builtin(
         &self,
-        index: BuiltinFunctionIndex,
+        key: FuncKey,
         symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError>;
 
@@ -309,7 +299,7 @@ pub trait Compiler: Send + Sync {
     fn compiled_function_relocation_targets<'a>(
         &'a self,
         func: &'a dyn Any,
-    ) -> Box<dyn Iterator<Item = RelocationTarget> + 'a>;
+    ) -> Box<dyn Iterator<Item = FuncKey> + 'a>;
 
     /// Appends a list of compiled functions to an in-memory object.
     ///
@@ -342,7 +332,7 @@ pub trait Compiler: Send + Sync {
         &self,
         obj: &mut Object<'static>,
         funcs: &[(String, Box<dyn Any + Send + Sync>)],
-        resolve_reloc: &dyn Fn(usize, RelocationTarget) -> usize,
+        resolve_reloc: &dyn Fn(usize, FuncKey) -> usize,
     ) -> Result<Vec<(SymbolId, FunctionLoc)>>;
 
     /// Creates a new `Object` file which is used to build the results of a
@@ -469,7 +459,7 @@ pub trait Compiler: Send + Sync {
 /// An inlining compiler.
 pub trait InliningCompiler: Sync + Send {
     /// Enumerate the function calls that the given `func` makes.
-    fn calls(&self, func: &CompiledFunctionBody, calls: &mut IndexSet<FuncIndex>) -> Result<()>;
+    fn calls(&self, func: &CompiledFunctionBody, calls: &mut IndexSet<FuncKey>) -> Result<()>;
 
     /// Get the abstract size of the given function, for the purposes of
     /// inlining heuristics.
@@ -483,7 +473,7 @@ pub trait InliningCompiler: Sync + Send {
     fn inline<'a>(
         &self,
         func: &mut CompiledFunctionBody,
-        get_callee: &'a mut dyn FnMut(FuncIndex) -> Option<&'a CompiledFunctionBody>,
+        get_callee: &'a mut dyn FnMut(FuncKey) -> Option<&'a CompiledFunctionBody>,
     ) -> Result<()>;
 
     /// Finish compiling the given function.
