@@ -194,6 +194,10 @@ pub struct Component {
     /// This is used to determine which set of instance flags are inspected when
     /// testing reentrance.
     pub defined_resource_instances: PrimaryMap<DefinedResourceIndex, RuntimeComponentInstanceIndex>,
+
+    /// All canonical options used by this component. Stored as a table here
+    /// from index-to-options so the options can be consulted at runtime.
+    pub options: PrimaryMap<OptionsIndex, CanonicalOptions>,
 }
 
 impl Component {
@@ -451,7 +455,7 @@ pub enum Export {
         /// Which core WebAssembly export is being lifted.
         func: CoreDef,
         /// Any options, if present, associated with this lifting.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
     /// A module defined within this component is exported.
     ModuleStatic {
@@ -480,6 +484,25 @@ pub enum Export {
     Type(TypeDef),
 }
 
+#[derive(Debug, Clone, Copy, Serialize, Deserialize)]
+/// Data is stored in a linear memory.
+pub struct LinearMemoryOptions {
+    /// The memory used by these options, if specified.
+    pub memory: Option<RuntimeMemoryIndex>,
+    /// The realloc function used by these options, if specified.
+    pub realloc: Option<RuntimeReallocIndex>,
+}
+
+/// The data model for objects that are not unboxed in locals.
+#[derive(Debug, Clone, Copy, Serialize, Deserialize)]
+pub enum CanonicalOptionsDataModel {
+    /// Data is stored in GC objects.
+    Gc {},
+
+    /// Data is stored in a linear memory.
+    LinearMemory(LinearMemoryOptions),
+}
+
 /// Canonical ABI options associated with a lifted or lowered function.
 #[derive(Debug, Clone, Serialize, Deserialize)]
 pub struct CanonicalOptions {
@@ -489,12 +512,6 @@ pub struct CanonicalOptions {
     /// The encoding used for strings.
     pub string_encoding: StringEncoding,
 
-    /// The memory used by these options, if specified.
-    pub memory: Option<RuntimeMemoryIndex>,
-
-    /// The realloc function used by these options, if specified.
-    pub realloc: Option<RuntimeReallocIndex>,
-
     /// The async callback function used by these options, if specified.
     pub callback: Option<RuntimeCallbackIndex>,
 
@@ -503,6 +520,17 @@ pub struct CanonicalOptions {
 
     /// Whether to use the async ABI for lifting or lowering.
     pub async_: bool,
+
+    /// Whether or not this function can consume a task cancellation
+    /// notification.
+    pub cancellable: bool,
+
+    /// The core function type that is being lifted from / lowered to.
+    pub core_type: ModuleInternedTypeIndex,
+
+    /// The data model (GC objects or linear memory) used with these canonical
+    /// options.
+    pub data_model: CanonicalOptionsDataModel,
 }
 
 /// Possible encodings of strings within the component model.
@@ -639,6 +667,7 @@ pub struct Resource {
 ///
 /// Note that this type does not implement `Serialize` or `Deserialize` and
 /// that's intentional as this isn't stored in the final compilation artifact.
+#[derive(Debug)]
 pub enum Trampoline {
     /// Description of a lowered import used in conjunction with
     /// `GlobalInitializer::LowerImport`.
@@ -652,7 +681,7 @@ pub enum Trampoline {
 
         /// The canonical ABI options used when lowering this function specified
         /// in the original component.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// Information about a string transcoding function required by an adapter
@@ -705,6 +734,18 @@ pub enum Trampoline {
         instance: RuntimeComponentInstanceIndex,
     },
 
+    /// A `backpressure.inc` intrinsic.
+    BackpressureInc {
+        /// The specific component instance which is calling the intrinsic.
+        instance: RuntimeComponentInstanceIndex,
+    },
+
+    /// A `backpressure.dec` intrinsic.
+    BackpressureDec {
+        /// The specific component instance which is calling the intrinsic.
+        instance: RuntimeComponentInstanceIndex,
+    },
+
     /// A `task.return` intrinsic, which returns a result to the caller of a
     /// lifted export function.  This allows the callee to continue executing
     /// after returning a result.
@@ -713,7 +754,15 @@ pub enum Trampoline {
         results: TypeTupleIndex,
 
         /// The canonical ABI options specified for this intrinsic.
-        options: CanonicalOptions,
+        options: OptionsIndex,
+    },
+
+    /// A `task.cancel` intrinsic, which acknowledges a `CANCELLED` event
+    /// delivered to a guest task previously created by a call to an async
+    /// export.
+    TaskCancel {
+        /// The specific component instance which is calling the intrinsic.
+        instance: RuntimeComponentInstanceIndex,
     },
 
     /// A `waitable-set.new` intrinsic.
@@ -726,24 +775,16 @@ pub enum Trampoline {
     /// outstanding async task/stream/future to make progress, returning the
     /// first such event.
     WaitableSetWait {
-        /// The specific component instance which is calling the intrinsic.
-        instance: RuntimeComponentInstanceIndex,
-        /// If `true`, indicates the caller instance maybe reentered.
-        async_: bool,
-        /// Memory to use when storing the event.
-        memory: RuntimeMemoryIndex,
+        /// Configuration options for this intrinsic call.
+        options: OptionsIndex,
     },
 
     /// A `waitable-set.poll` intrinsic, which checks whether any outstanding
     /// async task/stream/future has made progress.  Unlike `task.wait`, this
     /// does not block and may return nothing if no such event has occurred.
     WaitableSetPoll {
-        /// The specific component instance which is calling the intrinsic.
-        instance: RuntimeComponentInstanceIndex,
-        /// If `true`, indicates the caller instance maybe reentered.
-        async_: bool,
-        /// Memory to use when storing the event.
-        memory: RuntimeMemoryIndex,
+        /// Configuration options for this intrinsic call.
+        options: OptionsIndex,
     },
 
     /// A `waitable-set.drop` intrinsic.
@@ -758,11 +799,11 @@ pub enum Trampoline {
         instance: RuntimeComponentInstanceIndex,
     },
 
-    /// A `yield` intrinsic, which yields control to the host so that other
+    /// A `thread.yield` intrinsic, which yields control to the host so that other
     /// tasks are able to make progress, if any.
-    Yield {
+    ThreadYield {
         /// If `true`, indicates the caller instance maybe reentered.
-        async_: bool,
+        cancellable: bool,
     },
 
     /// A `subtask.drop` intrinsic to drop a specified task which has completed.
@@ -771,6 +812,15 @@ pub enum Trampoline {
         instance: RuntimeComponentInstanceIndex,
     },
 
+    /// A `subtask.cancel` intrinsic to drop an in-progress task.
+    SubtaskCancel {
+        /// The specific component instance which is calling the intrinsic.
+        instance: RuntimeComponentInstanceIndex,
+        /// If `false`, block until cancellation completes rather than return
+        /// `BLOCKED`.
+        async_: bool,
+    },
+
     /// A `stream.new` intrinsic to create a new `stream` handle of the
     /// specified type.
     StreamNew {
@@ -785,16 +835,17 @@ pub enum Trampoline {
 
         /// Any options (e.g. string encoding) to use when storing values to
         /// memory.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// A `stream.write` intrinsic to write to a `stream` of the specified type.
     StreamWrite {
         /// The table index for the specific `stream` type and caller instance.
         ty: TypeStreamTableIndex,
+
         /// Any options (e.g. string encoding) to use when storing values to
         /// memory.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// A `stream.cancel-read` intrinsic to cancel an in-progress read from a
@@ -817,16 +868,16 @@ pub enum Trampoline {
         async_: bool,
     },
 
-    /// A `stream.close-readable` intrinsic to close the readable end of a
+    /// A `stream.drop-readable` intrinsic to drop the readable end of a
     /// `stream` of the specified type.
-    StreamCloseReadable {
+    StreamDropReadable {
         /// The table index for the specific `stream` type and caller instance.
         ty: TypeStreamTableIndex,
     },
 
-    /// A `stream.close-writable` intrinsic to close the writable end of a
+    /// A `stream.drop-writable` intrinsic to drop the writable end of a
     /// `stream` of the specified type.
-    StreamCloseWritable {
+    StreamDropWritable {
         /// The table index for the specific `stream` type and caller instance.
         ty: TypeStreamTableIndex,
     },
@@ -845,16 +896,17 @@ pub enum Trampoline {
 
         /// Any options (e.g. string encoding) to use when storing values to
         /// memory.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// A `future.write` intrinsic to write to a `future` of the specified type.
     FutureWrite {
         /// The table index for the specific `future` type and caller instance.
         ty: TypeFutureTableIndex,
+
         /// Any options (e.g. string encoding) to use when storing values to
         /// memory.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// A `future.cancel-read` intrinsic to cancel an in-progress read from a
@@ -877,16 +929,16 @@ pub enum Trampoline {
         async_: bool,
     },
 
-    /// A `future.close-readable` intrinsic to close the readable end of a
+    /// A `future.drop-readable` intrinsic to drop the readable end of a
     /// `future` of the specified type.
-    FutureCloseReadable {
+    FutureDropReadable {
         /// The table index for the specific `future` type and caller instance.
         ty: TypeFutureTableIndex,
     },
 
-    /// A `future.close-writable` intrinsic to close the writable end of a
+    /// A `future.drop-writable` intrinsic to drop the writable end of a
     /// `future` of the specified type.
-    FutureCloseWritable {
+    FutureDropWritable {
         /// The table index for the specific `future` type and caller instance.
         ty: TypeFutureTableIndex,
     },
@@ -897,7 +949,7 @@ pub enum Trampoline {
         /// The table index for the `error-context` type in the caller instance.
         ty: TypeComponentLocalErrorContextTableIndex,
         /// String encoding, memory, etc. to use when loading debug message.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// A `error-context.debug-message` intrinsic to get the debug message for a
@@ -909,7 +961,7 @@ pub enum Trampoline {
         /// The table index for the `error-context` type in the caller instance.
         ty: TypeComponentLocalErrorContextTableIndex,
         /// String encoding, memory, etc. to use when storing debug message.
-        options: CanonicalOptions,
+        options: OptionsIndex,
     },
 
     /// A `error-context.drop` intrinsic to drop a specified `error-context`.
@@ -936,29 +988,30 @@ pub enum Trampoline {
     /// Same as `ResourceEnterCall` except for when exiting a call.
     ResourceExitCall,
 
-    /// An intrinsic used by FACT-generated modules to begin a call involving a
-    /// sync-lowered import and async-lifted export.
-    SyncEnterCall,
+    /// An intrinsic used by FACT-generated modules to prepare a call involving
+    /// an async-lowered import and/or an async-lifted export.
+    PrepareCall {
+        /// The memory used to verify that the memory specified for the
+        /// `task.return` that is called at runtime matches the one specified in
+        /// the lifted export.
+        memory: Option<RuntimeMemoryIndex>,
+    },
 
-    /// An intrinsic used by FACT-generated modules to complete a call involving
-    /// a sync-lowered import and async-lifted export.
-    SyncExitCall {
+    /// An intrinsic used by FACT-generated modules to start a call involving a
+    /// sync-lowered import and async-lifted export.
+    SyncStartCall {
         /// The callee's callback function, if any.
         callback: Option<RuntimeCallbackIndex>,
     },
 
-    /// An intrinsic used by FACT-generated modules to begin a call involving an
-    /// async-lowered import function.
-    AsyncEnterCall,
-
-    /// An intrinsic used by FACT-generated modules to complete a call involving
+    /// An intrinsic used by FACT-generated modules to start a call involving
     /// an async-lowered import function.
     ///
-    /// Note that `AsyncEnterCall` and `AsyncExitCall` could theoretically be
+    /// Note that `AsyncPrepareCall` and `AsyncStartCall` could theoretically be
     /// combined into a single `AsyncCall` intrinsic, but we separate them to
     /// allow the FACT-generated module to optionally call the callee directly
     /// without an intermediate host stack frame.
-    AsyncExitCall {
+    AsyncStartCall {
         /// The callee's callback, if any.
         callback: Option<RuntimeCallbackIndex>,
 
@@ -989,6 +1042,18 @@ pub enum Trampoline {
     /// are reference counted, meaning that sharing the handle with another
     /// component does not invalidate the handle in the original component.
     ErrorContextTransfer,
+
+    /// Intrinsic used to implement the `context.get` component model builtin.
+    ///
+    /// The payload here represents that this is accessing the Nth slot of local
+    /// storage.
+    ContextGet(u32),
+
+    /// Intrinsic used to implement the `context.set` component model builtin.
+    ///
+    /// The payload here represents that this is accessing the Nth slot of local
+    /// storage.
+    ContextSet(u32),
 }
 
 impl Trampoline {
@@ -1013,28 +1078,32 @@ impl Trampoline {
             ResourceRep(i) => format!("component-resource-rep[{}]", i.as_u32()),
             ResourceDrop(i) => format!("component-resource-drop[{}]", i.as_u32()),
             BackpressureSet { .. } => format!("backpressure-set"),
+            BackpressureInc { .. } => format!("backpressure-inc"),
+            BackpressureDec { .. } => format!("backpressure-dec"),
             TaskReturn { .. } => format!("task-return"),
+            TaskCancel { .. } => format!("task-cancel"),
             WaitableSetNew { .. } => format!("waitable-set-new"),
             WaitableSetWait { .. } => format!("waitable-set-wait"),
             WaitableSetPoll { .. } => format!("waitable-set-poll"),
             WaitableSetDrop { .. } => format!("waitable-set-drop"),
             WaitableJoin { .. } => format!("waitable-join"),
-            Yield { .. } => format!("yield"),
+            ThreadYield { .. } => format!("thread-yield"),
             SubtaskDrop { .. } => format!("subtask-drop"),
+            SubtaskCancel { .. } => format!("subtask-cancel"),
             StreamNew { .. } => format!("stream-new"),
             StreamRead { .. } => format!("stream-read"),
             StreamWrite { .. } => format!("stream-write"),
             StreamCancelRead { .. } => format!("stream-cancel-read"),
             StreamCancelWrite { .. } => format!("stream-cancel-write"),
-            StreamCloseReadable { .. } => format!("stream-close-readable"),
-            StreamCloseWritable { .. } => format!("stream-close-writable"),
+            StreamDropReadable { .. } => format!("stream-drop-readable"),
+            StreamDropWritable { .. } => format!("stream-drop-writable"),
             FutureNew { .. } => format!("future-new"),
             FutureRead { .. } => format!("future-read"),
             FutureWrite { .. } => format!("future-write"),
             FutureCancelRead { .. } => format!("future-cancel-read"),
             FutureCancelWrite { .. } => format!("future-cancel-write"),
-            FutureCloseReadable { .. } => format!("future-close-readable"),
-            FutureCloseWritable { .. } => format!("future-close-writable"),
+            FutureDropReadable { .. } => format!("future-drop-readable"),
+            FutureDropWritable { .. } => format!("future-drop-writable"),
             ErrorContextNew { .. } => format!("error-context-new"),
             ErrorContextDebugMessage { .. } => format!("error-context-debug-message"),
             ErrorContextDrop { .. } => format!("error-context-drop"),
@@ -1042,13 +1111,14 @@ impl Trampoline {
             ResourceTransferBorrow => format!("component-resource-transfer-borrow"),
             ResourceEnterCall => format!("component-resource-enter-call"),
             ResourceExitCall => format!("component-resource-exit-call"),
-            SyncEnterCall => format!("component-sync-enter-call"),
-            SyncExitCall { .. } => format!("component-sync-exit-call"),
-            AsyncEnterCall => format!("component-async-enter-call"),
-            AsyncExitCall { .. } => format!("component-async-exit-call"),
+            PrepareCall { .. } => format!("component-prepare-call"),
+            SyncStartCall { .. } => format!("component-sync-start-call"),
+            AsyncStartCall { .. } => format!("component-async-start-call"),
             FutureTransfer => format!("future-transfer"),
             StreamTransfer => format!("stream-transfer"),
             ErrorContextTransfer => format!("error-context-transfer"),
+            ContextGet(_) => format!("context-get"),
+            ContextSet(_) => format!("context-set"),
         }
     }
 }
