@@ -193,6 +193,12 @@ enum LocalInitializer<'data> {
     BackpressureSet {
         func: ModuleInternedTypeIndex,
     },
+    BackpressureInc {
+        func: ModuleInternedTypeIndex,
+    },
+    BackpressureDec {
+        func: ModuleInternedTypeIndex,
+    },
     TaskReturn {
         result: Option<ComponentValType>,
         options: LocalCanonicalOptions,
@@ -215,9 +221,9 @@ enum LocalInitializer<'data> {
     WaitableJoin {
         func: ModuleInternedTypeIndex,
     },
-    Yield {
+    ThreadYield {
         func: ModuleInternedTypeIndex,
-        async_: bool,
+        cancellable: bool,
     },
     SubtaskDrop {
         func: ModuleInternedTypeIndex,
@@ -386,6 +392,7 @@ struct LocalCanonicalOptions {
     string_encoding: StringEncoding,
     post_return: Option<FuncIndex>,
     async_: bool,
+    cancellable: bool,
     callback: Option<FuncIndex>,
     /// The type index of the core GC types signature.
     core_type: ModuleInternedTypeIndex,
@@ -813,6 +820,17 @@ impl<'a, 'data> Translator<'a, 'data> {
                             core_func_index += 1;
                             LocalInitializer::BackpressureSet { func: core_type }
                         }
+                        wasmparser::CanonicalFunction::BackpressureInc => {
+                            let core_type = self.core_func_signature(core_func_index)?;
+                            core_func_index += 1;
+                            LocalInitializer::BackpressureInc { func: core_type }
+                        }
+                        wasmparser::CanonicalFunction::BackpressureDec => {
+                            let core_type = self.core_func_signature(core_func_index)?;
+                            core_func_index += 1;
+                            LocalInitializer::BackpressureDec { func: core_type }
+                        }
+
                         wasmparser::CanonicalFunction::TaskReturn { result, options } => {
                             let result = result.map(|ty| match ty {
                                 wasmparser::ComponentValType::Primitive(ty) => {
@@ -839,13 +857,17 @@ impl<'a, 'data> Translator<'a, 'data> {
                             core_func_index += 1;
                             LocalInitializer::WaitableSetNew { func }
                         }
-                        wasmparser::CanonicalFunction::WaitableSetWait { async_, memory } => {
+                        wasmparser::CanonicalFunction::WaitableSetWait {
+                            cancellable,
+                            memory,
+                        } => {
                             let core_type = self.core_func_signature(core_func_index)?;
                             core_func_index += 1;
                             LocalInitializer::WaitableSetWait {
                                 options: LocalCanonicalOptions {
                                     core_type,
-                                    async_,
+                                    cancellable,
+                                    async_: false,
                                     data_model: LocalDataModel::LinearMemory {
                                         memory: Some(MemoryIndex::from_u32(memory)),
                                         realloc: None,
@@ -856,13 +878,17 @@ impl<'a, 'data> Translator<'a, 'data> {
                                 },
                             }
                         }
-                        wasmparser::CanonicalFunction::WaitableSetPoll { async_, memory } => {
+                        wasmparser::CanonicalFunction::WaitableSetPoll {
+                            cancellable,
+                            memory,
+                        } => {
                             let core_type = self.core_func_signature(core_func_index)?;
                             core_func_index += 1;
                             LocalInitializer::WaitableSetPoll {
                                 options: LocalCanonicalOptions {
                                     core_type,
-                                    async_,
+                                    async_: false,
+                                    cancellable,
                                     data_model: LocalDataModel::LinearMemory {
                                         memory: Some(MemoryIndex::from_u32(memory)),
                                         realloc: None,
@@ -883,10 +909,10 @@ impl<'a, 'data> Translator<'a, 'data> {
                             core_func_index += 1;
                             LocalInitializer::WaitableJoin { func }
                         }
-                        wasmparser::CanonicalFunction::Yield { async_ } => {
+                        wasmparser::CanonicalFunction::ThreadYield { cancellable } => {
                             let func = self.core_func_signature(core_func_index)?;
                             core_func_index += 1;
-                            LocalInitializer::Yield { func, async_ }
+                            LocalInitializer::ThreadYield { func, cancellable }
                         }
                         wasmparser::CanonicalFunction::SubtaskDrop => {
                             let func = self.core_func_signature(core_func_index)?;
@@ -1063,6 +1089,25 @@ impl<'a, 'data> Translator<'a, 'data> {
                             core_func_index += 1;
                             LocalInitializer::ContextSet { i, func }
                         }
+
+                        wasmparser::CanonicalFunction::ThreadIndex => {
+                            bail!("unimplemented `thread.index`");
+                        }
+                        wasmparser::CanonicalFunction::ThreadNewIndirect { .. } => {
+                            bail!("unimplemented `thread.new-indirect`");
+                        }
+                        wasmparser::CanonicalFunction::ThreadSwitchTo { .. } => {
+                            bail!("unimplemented `thread.switch-to`");
+                        }
+                        wasmparser::CanonicalFunction::ThreadSuspend { .. } => {
+                            bail!("unimplemented `thread.suspend`");
+                        }
+                        wasmparser::CanonicalFunction::ThreadResumeLater => {
+                            bail!("unimplemented `thread.resume-later`");
+                        }
+                        wasmparser::CanonicalFunction::ThreadYieldTo { .. } => {
+                            bail!("unimplemented `thread.yield-to`");
+                        }
                     };
                     self.result.initializers.push(init);
                 }
@@ -1082,10 +1127,12 @@ impl<'a, 'data> Translator<'a, 'data> {
             } => {
                 let index = self.validator.types(0).unwrap().module_count();
                 self.validator.module_section(&unchecked_range)?;
+                let static_module_index = self.static_modules.next_key();
                 let translation = ModuleEnvironment::new(
                     self.tunables,
                     self.validator,
                     self.types.module_types_builder(),
+                    static_module_index,
                 )
                 .translate(
                     parser,
@@ -1101,12 +1148,13 @@ impl<'a, 'data> Translator<'a, 'data> {
                             .context("wasm component contains an invalid module section")
                         })?,
                 )?;
-                let static_idx = self.static_modules.push(translation);
+                let static_module_index2 = self.static_modules.push(translation);
+                assert_eq!(static_module_index, static_module_index2);
                 let types = self.validator.types(0).unwrap();
                 let ty = types.module_at(index);
                 self.result
                     .initializers
-                    .push(LocalInitializer::ModuleStatic(static_idx, ty));
+                    .push(LocalInitializer::ModuleStatic(static_module_index, ty));
                 return Ok(Action::Skip(unchecked_range.end - unchecked_range.start));
             }
 
@@ -1489,6 +1537,7 @@ impl<'a, 'data> Translator<'a, 'data> {
         Ok(LocalCanonicalOptions {
             string_encoding,
             post_return,
+            cancellable: false,
             async_,
             callback,
             core_type,
