@@ -372,6 +372,7 @@ impl EngineOrModuleTypeIndex {
     }
 
     /// Get the underlying engine-level type index, if any.
+    #[inline]
     pub fn as_engine_type_index(self) -> Option<VMSharedTypeIndex> {
         match self {
             Self::Engine(e) => Some(e),
@@ -381,6 +382,7 @@ impl EngineOrModuleTypeIndex {
 
     /// Get the underlying engine-level type index, or panic.
     #[track_caller]
+    #[inline]
     pub fn unwrap_engine_type_index(self) -> VMSharedTypeIndex {
         match self.as_engine_type_index() {
             Some(x) => x,
@@ -645,7 +647,7 @@ impl WasmHeapType {
 }
 
 /// A top heap type.
-#[derive(Debug, Clone, Copy, Eq, PartialEq)]
+#[derive(Debug, Clone, Copy, Eq, PartialEq, Hash, Serialize, Deserialize)]
 pub enum WasmHeapTopType {
     /// The common supertype of all external references.
     Extern,
@@ -1800,6 +1802,7 @@ impl ConstExpr {
     /// Returns the new const expression as well as the escaping function
     /// indices that appeared in `ref.func` instructions, if any.
     pub fn from_wasmparser(
+        env: &dyn TypeConvert,
         expr: wasmparser::ConstExpr<'_>,
     ) -> WasmResult<(Self, SmallVec<[FuncIndex; 1]>)> {
         let mut iter = expr
@@ -1825,12 +1828,13 @@ impl ConstExpr {
                 escaped.push(FuncIndex::from_u32(*function_index));
             }
 
-            ops.push(ConstOp::from_wasmparser(op, offset)?);
+            ops.push(ConstOp::from_wasmparser(env, op, offset)?);
         }
         Ok((Self { ops }, escaped))
     }
 
     /// Get the opcodes that make up this const expression.
+    #[inline]
     pub fn ops(&self) -> &[ConstOp] {
         &self.ops
     }
@@ -1875,7 +1879,7 @@ pub enum ConstOp {
     V128Const(u128),
     GlobalGet(GlobalIndex),
     RefI31,
-    RefNull,
+    RefNull(WasmHeapTopType),
     RefFunc(FuncIndex),
     I32Add,
     I32Sub,
@@ -1905,7 +1909,11 @@ pub enum ConstOp {
 
 impl ConstOp {
     /// Convert a `wasmparser::Operator` to a `ConstOp`.
-    pub fn from_wasmparser(op: wasmparser::Operator<'_>, offset: usize) -> WasmResult<Self> {
+    pub fn from_wasmparser(
+        env: &dyn TypeConvert,
+        op: wasmparser::Operator<'_>,
+        offset: usize,
+    ) -> WasmResult<Self> {
         use wasmparser::Operator as O;
         Ok(match op {
             O::I32Const { value } => Self::I32Const(value),
@@ -1913,7 +1921,7 @@ impl ConstOp {
             O::F32Const { value } => Self::F32Const(value.bits()),
             O::F64Const { value } => Self::F64Const(value.bits()),
             O::V128Const { value } => Self::V128Const(u128::from_le_bytes(*value.bytes())),
-            O::RefNull { hty: _ } => Self::RefNull,
+            O::RefNull { hty } => Self::RefNull(env.convert_heap_type(hty)?.top()),
             O::RefFunc { function_index } => Self::RefFunc(FuncIndex::from_u32(function_index)),
             O::GlobalGet { global_index } => Self::GlobalGet(GlobalIndex::from_u32(global_index)),
             O::RefI31 => Self::RefI31,
@@ -2230,6 +2238,8 @@ impl From<wasmparser::MemoryType> for Memory {
 pub struct Tag {
     /// The tag signature type.
     pub signature: EngineOrModuleTypeIndex,
+    /// The corresponding exception type.
+    pub exception: EngineOrModuleTypeIndex,
 }
 
 impl TypeTrace for Tag {
@@ -2237,14 +2247,18 @@ impl TypeTrace for Tag {
     where
         F: FnMut(EngineOrModuleTypeIndex) -> Result<(), E>,
     {
-        func(self.signature)
+        func(self.signature)?;
+        func(self.exception)?;
+        Ok(())
     }
 
     fn trace_mut<F, E>(&mut self, func: &mut F) -> Result<(), E>
     where
         F: FnMut(&mut EngineOrModuleTypeIndex) -> Result<(), E>,
     {
-        func(&mut self.signature)
+        func(&mut self.signature)?;
+        func(&mut self.exception)?;
+        Ok(())
     }
 }
 
