@@ -14,7 +14,7 @@ use cranelift_codegen::isa::{
     unwind::{UnwindInfo, UnwindInfoKind},
 };
 use cranelift_codegen::print_errors::pretty_error;
-use cranelift_codegen::{CompiledCode, Context};
+use cranelift_codegen::{CompiledCode, Context, FinalizedMachCallSite};
 use cranelift_entity::PrimaryMap;
 use cranelift_frontend::FunctionBuilder;
 use object::write::{Object, StandardSegment, SymbolId};
@@ -28,13 +28,15 @@ use std::ops::Range;
 use std::path;
 use std::sync::{Arc, Mutex};
 use wasmparser::{FuncValidatorAllocations, FunctionBody};
+use wasmtime_environ::obj::ELF_WASMTIME_EXCEPTIONS;
 use wasmtime_environ::{
     AddressMapSection, BuiltinFunctionIndex, CacheStore, CompileError, CompiledFunctionBody,
-    DefinedFuncIndex, FlagValue, FuncIndex, FunctionBodyData, FunctionLoc, HostCall,
-    InliningCompiler, ModuleTranslation, ModuleTypesBuilder, PtrSize, RelocationTarget,
-    StackMapSection, StaticModuleIndex, TrapEncodingBuilder, TrapSentinel, TripleExt, Tunables,
-    VMOffsets, WasmFuncType, WasmValType,
+    DefinedFuncIndex, FlagValue, FuncKey, FunctionBodyData, FunctionLoc, HostCall,
+    InliningCompiler, ModuleTranslation, ModuleTypesBuilder, PtrSize, StackMapSection,
+    StaticModuleIndex, TrapEncodingBuilder, TrapSentinel, TripleExt, Tunables, VMOffsets,
+    WasmFuncType, WasmValType,
 };
+use wasmtime_unwinder::ExceptionTableBuilder;
 
 #[cfg(feature = "component-model")]
 mod component;
@@ -85,6 +87,7 @@ pub struct Compiler {
     tunables: Tunables,
     contexts: Mutex<Vec<CompilerContext>>,
     isa: OwnedTargetIsa,
+    emit_debug_checks: bool,
     linkopts: LinkOptions,
     cache_store: Option<Arc<dyn CacheStore>>,
     clif_dir: Option<path::PathBuf>,
@@ -125,6 +128,7 @@ impl Compiler {
         tunables: Tunables,
         isa: OwnedTargetIsa,
         cache_store: Option<Arc<dyn CacheStore>>,
+        emit_debug_checks: bool,
         linkopts: LinkOptions,
         clif_dir: Option<path::PathBuf>,
         wmemcheck: bool,
@@ -134,6 +138,7 @@ impl Compiler {
             contexts: Default::default(),
             tunables,
             isa,
+            emit_debug_checks,
             linkopts,
             cache_store,
             clif_dir,
@@ -176,12 +181,13 @@ impl Compiler {
                 .params
                 .insert(0, ir::AbiParam::new(self.isa.pointer_type()));
             let new_sig = builder.func.import_signature(new_signature);
-            let name = ir::ExternalName::User(builder.func.declare_imported_user_function(
-                ir::UserExternalName {
-                    namespace: crate::NS_PULLEY_HOSTCALL,
-                    index: hostcall.into().index(),
-                },
-            ));
+            let key = FuncKey::PulleyHostCall(hostcall.into());
+            let (namespace, index) = key.into_raw_parts();
+            let name = ir::ExternalName::User(
+                builder
+                    .func
+                    .declare_imported_user_function(ir::UserExternalName { namespace, index }),
+            );
             let func = builder.func.import_function(ir::ExtFuncData {
                 name,
                 signature: new_sig,
@@ -198,6 +204,29 @@ impl Compiler {
     }
 }
 
+fn box_dyn_any_compiled_function(f: CompiledFunction) -> Box<dyn Any + Send + Sync> {
+    let b = box_dyn_any(f);
+    debug_assert!(b.is::<CompiledFunction>());
+    b
+}
+
+fn box_dyn_any_compiler_context(ctx: Option<CompilerContext>) -> Box<dyn Any + Send + Sync> {
+    let b = box_dyn_any(ctx);
+    debug_assert!(b.is::<Option<CompilerContext>>());
+    b
+}
+
+fn box_dyn_any(x: impl Any + Send + Sync) -> Box<dyn Any + Send + Sync> {
+    log::trace!(
+        "making Box<dyn Any + Send + Sync> of {}",
+        std::any::type_name_of_val(&x)
+    );
+    let b = Box::new(x);
+    let r: &(dyn Any + Sync + Send) = &*b;
+    log::trace!("  --> {r:#p}");
+    b
+}
+
 impl wasmtime_environ::Compiler for Compiler {
     fn inlining_compiler(&self) -> Option<&dyn wasmtime_environ::InliningCompiler> {
         Some(self)
@@ -206,14 +235,20 @@ impl wasmtime_environ::Compiler for Compiler {
     fn compile_function(
         &self,
         translation: &ModuleTranslation<'_>,
-        func_index: DefinedFuncIndex,
+        key: FuncKey,
         input: FunctionBodyData<'_>,
         types: &ModuleTypesBuilder,
         symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError> {
+        log::trace!("compiling Wasm function: {key:?} = {symbol:?}");
+
         let isa = &*self.isa;
         let module = &translation.module;
-        let func_index = module.func_index(func_index);
+
+        let (module_index, def_func_index) = key.unwrap_defined_wasm_function();
+        debug_assert_eq!(translation.module_index, module_index);
+
+        let func_index = module.func_index(def_func_index);
         let sig = translation.module.functions[func_index]
             .signature
             .unwrap_module_type_index();
@@ -223,10 +258,8 @@ impl wasmtime_environ::Compiler for Compiler {
 
         let context = &mut compiler.cx.codegen_context;
         context.func.signature = wasm_call_signature(isa, wasm_func_ty, &self.tunables);
-        context.func.name = UserFuncName::User(UserExternalName {
-            namespace: crate::NS_WASM_FUNC,
-            index: func_index.as_u32(),
-        });
+        let (namespace, index) = key.into_raw_parts();
+        context.func.name = UserFuncName::User(UserExternalName { namespace, index });
 
         if self.tunables.generate_native_debuginfo {
             context.func.collect_debug_info();
@@ -311,7 +344,7 @@ impl wasmtime_environ::Compiler for Compiler {
         log::trace!("`{symbol}` timing info\n{timing}");
 
         Ok(CompiledFunctionBody {
-            code: Box::new(Some(compiler.cx)),
+            code: box_dyn_any_compiler_context(Some(compiler.cx)),
             needs_gc_heap: func_env.needs_gc_heap(),
         })
     }
@@ -320,9 +353,14 @@ impl wasmtime_environ::Compiler for Compiler {
         &self,
         translation: &ModuleTranslation<'_>,
         types: &ModuleTypesBuilder,
-        def_func_index: DefinedFuncIndex,
-        _symbol: &str,
+        key: FuncKey,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError> {
+        log::trace!("compiling array-to-wasm trampoline: {key:?} = {symbol:?}");
+
+        let (module_index, def_func_index) = key.unwrap_array_to_wasm_trampoline();
+        debug_assert_eq!(translation.module_index, module_index);
+
         let func_index = translation.module.func_index(def_func_index);
         let sig = translation.module.functions[func_index]
             .signature
@@ -357,7 +395,7 @@ impl wasmtime_environ::Compiler for Compiler {
         //
         // Assert that we were really given a core Wasm vmctx, since that's
         // what we are assuming with our offsets below.
-        debug_assert_vmctx_kind(isa, &mut builder, vmctx, wasmtime_environ::VMCONTEXT_MAGIC);
+        self.debug_assert_vmctx_kind(&mut builder, vmctx, wasmtime_environ::VMCONTEXT_MAGIC);
         let offsets = VMOffsets::new(isa.pointer_bytes(), &translation.module);
         let vm_store_context_offset = offsets.ptr.vmctx_store_context();
         save_last_wasm_entry_fp(
@@ -369,7 +407,8 @@ impl wasmtime_environ::Compiler for Compiler {
         );
 
         // Then call the Wasm function with those arguments.
-        let call = declare_and_call(&mut builder, wasm_call_sig, func_index.as_u32(), &args);
+        let callee_key = FuncKey::DefinedWasmFunction(module_index, def_func_index);
+        let call = declare_and_call(&mut builder, wasm_call_sig, callee_key, &args);
         let results = builder.func.dfg.inst_results(call).to_vec();
 
         // Then store the results back into the array.
@@ -390,7 +429,7 @@ impl wasmtime_environ::Compiler for Compiler {
         builder.finalize();
 
         Ok(CompiledFunctionBody {
-            code: Box::new(Some(compiler.cx)),
+            code: box_dyn_any_compiler_context(Some(compiler.cx)),
             needs_gc_heap: false,
         })
     }
@@ -398,8 +437,11 @@ impl wasmtime_environ::Compiler for Compiler {
     fn compile_wasm_to_array_trampoline(
         &self,
         wasm_func_ty: &WasmFuncType,
-        _symbol: &str,
+        key: FuncKey,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError> {
+        log::trace!("compiling wasm-to-array trampoline: {key:?} = {symbol:?}");
+
         let isa = &*self.isa;
         let pointer_type = isa.pointer_type();
         let wasm_call_sig = wasm_call_signature(isa, wasm_func_ty, &self.tunables);
@@ -417,8 +459,7 @@ impl wasmtime_environ::Compiler for Compiler {
         //
         // Assert that the caller vmctx really is a core Wasm vmctx, since
         // that's what we are assuming with our offsets below.
-        debug_assert_vmctx_kind(
-            isa,
+        self.debug_assert_vmctx_kind(
             &mut builder,
             caller_vmctx,
             wasmtime_environ::VMCONTEXT_MAGIC,
@@ -464,7 +505,7 @@ impl wasmtime_environ::Compiler for Compiler {
         builder.finalize();
 
         Ok(CompiledFunctionBody {
-            code: Box::new(Some(compiler.cx)),
+            code: box_dyn_any_compiler_context(Some(compiler.cx)),
             needs_gc_heap: false,
         })
     }
@@ -473,8 +514,13 @@ impl wasmtime_environ::Compiler for Compiler {
         &self,
         obj: &mut Object<'static>,
         funcs: &[(String, Box<dyn Any + Send + Sync>)],
-        resolve_reloc: &dyn Fn(usize, RelocationTarget) -> usize,
+        resolve_reloc: &dyn Fn(usize, FuncKey) -> usize,
     ) -> Result<Vec<(SymbolId, FunctionLoc)>> {
+        log::trace!(
+            "appending functions to object file: {:#?}",
+            funcs.iter().map(|(sym, _)| sym).collect::<Vec<_>>()
+        );
+
         let mut builder =
             ModuleTextBuilder::new(obj, self, self.isa.text_section_builder(funcs.len()));
         if self.linkopts.force_jump_veneers {
@@ -483,27 +529,41 @@ impl wasmtime_environ::Compiler for Compiler {
         let mut addrs = AddressMapSection::default();
         let mut traps = TrapEncodingBuilder::default();
         let mut stack_maps = StackMapSection::default();
+        let mut exception_tables = ExceptionTableBuilder::default();
 
         let mut ret = Vec::with_capacity(funcs.len());
         for (i, (sym, func)) in funcs.iter().enumerate() {
+            debug_assert!(!func.is::<Option<CompilerContext>>());
+            debug_assert!(func.is::<CompiledFunction>());
             let func = func.downcast_ref::<CompiledFunction>().unwrap();
-            let (sym, range) = builder.append_func(&sym, func, |idx| resolve_reloc(i, idx));
+
+            let (sym_id, range) = builder.append_func(&sym, func, |idx| resolve_reloc(i, idx));
+            log::trace!("symbol id {sym_id:?} = {sym:?}");
+
             if self.tunables.generate_address_map {
                 let addr = func.address_map();
                 addrs.push(range.clone(), &addr.instructions);
             }
+
             clif_to_env_stack_maps(
                 &mut stack_maps,
                 range.clone(),
                 func.buffer.user_stack_maps(),
             );
+
             traps.push(range.clone(), &func.traps().collect::<Vec<_>>());
+            clif_to_env_exception_tables(
+                &mut exception_tables,
+                range.clone(),
+                func.buffer.call_sites(),
+            )?;
             builder.append_padding(self.linkopts.padding_between_functions);
+
             let info = FunctionLoc {
                 start: u32::try_from(range.start).unwrap(),
                 length: u32::try_from(range.end - range.start).unwrap(),
             };
-            ret.push((sym, info));
+            ret.push((sym_id, info));
         }
 
         builder.finish();
@@ -514,6 +574,15 @@ impl wasmtime_environ::Compiler for Compiler {
         stack_maps.append_to(obj);
         traps.append_to(obj);
 
+        let exception_section = obj.add_section(
+            obj.segment_name(StandardSegment::Data).to_vec(),
+            ELF_WASMTIME_EXCEPTIONS.as_bytes().to_vec(),
+            SectionKind::ReadOnlyData,
+        );
+        exception_tables.serialize(|bytes| {
+            obj.append_section_data(exception_section, bytes, 1);
+        });
+
         Ok(ret)
     }
 
@@ -549,13 +618,19 @@ impl wasmtime_environ::Compiler for Compiler {
         dwarf_package_bytes: Option<&'a [u8]>,
         tunables: &'a Tunables,
     ) -> Result<()> {
+        log::trace!("appending DWARF debug info");
+
         let get_func = move |m, f| {
             let (sym, any) = get_func(m, f);
+            log::trace!("get_func({m:?}, {f:?}) -> ({sym:?}, {any:#p})");
+            debug_assert!(!any.is::<Option<CompilerContext>>());
+            debug_assert!(any.is::<CompiledFunction>());
             (
                 sym,
                 any.downcast_ref::<CompiledFunction>().unwrap().metadata(),
             )
         };
+
         let mut compilation = crate::debug::Compilation::new(
             &*self.isa,
             translations,
@@ -613,15 +688,18 @@ impl wasmtime_environ::Compiler for Compiler {
 
     fn compile_wasm_to_builtin(
         &self,
-        index: BuiltinFunctionIndex,
-        _symbol: &str,
+        key: FuncKey,
+        symbol: &str,
     ) -> Result<CompiledFunctionBody, CompileError> {
+        log::trace!("compiling wasm-to-builtin trampoline: {key:?} = {symbol:?}");
+
         let isa = &*self.isa;
         let ptr_size = isa.pointer_bytes();
         let pointer_type = isa.pointer_type();
         let sigs = BuiltinFunctionSignatures::new(self);
-        let wasm_sig = sigs.wasm_signature(index);
-        let host_sig = sigs.host_signature(index);
+        let builtin_func_index = key.unwrap_wasm_to_builtin_trampoline();
+        let wasm_sig = sigs.wasm_signature(builtin_func_index);
+        let host_sig = sigs.host_signature(builtin_func_index);
 
         let mut compiler = self.function_compiler();
         let func = ir::Function::with_name_signature(Default::default(), wasm_sig.clone());
@@ -631,7 +709,7 @@ impl wasmtime_environ::Compiler for Compiler {
         // Debug-assert that this is the right kind of vmctx, and then
         // additionally perform the "routine of the exit trampoline" of saving
         // fp/pc/etc.
-        debug_assert_vmctx_kind(isa, &mut builder, vmctx, wasmtime_environ::VMCONTEXT_MAGIC);
+        self.debug_assert_vmctx_kind(&mut builder, vmctx, wasmtime_environ::VMCONTEXT_MAGIC);
         let vm_store_context = builder.ins().load(
             pointer_type,
             MemFlags::trusted(),
@@ -643,7 +721,7 @@ impl wasmtime_environ::Compiler for Compiler {
         // Now it's time to delegate to the actual builtin. Forward all our own
         // arguments to the libcall itself.
         let args = builder.block_params(block0).to_vec();
-        let call = self.call_builtin(&mut builder, vmctx, &args, index, host_sig);
+        let call = self.call_builtin(&mut builder, vmctx, &args, builtin_func_index, host_sig);
         let results = builder.func.dfg.inst_results(call).to_vec();
 
         // Libcalls do not explicitly `longjmp` for example but instead return a
@@ -652,7 +730,7 @@ impl wasmtime_environ::Compiler for Compiler {
         // value and raise a trap as appropriate. With the `results` above check
         // what `index` is and for each libcall that has a trapping return value
         // process it here.
-        match index.trap_sentinel() {
+        match builtin_func_index.trap_sentinel() {
             Some(TrapSentinel::Falsy) => {
                 self.raise_if_host_trapped(&mut builder, vmctx, results[0]);
             }
@@ -685,7 +763,7 @@ impl wasmtime_environ::Compiler for Compiler {
         builder.finalize();
 
         Ok(CompiledFunctionBody {
-            code: Box::new(Some(compiler.cx)),
+            code: box_dyn_any_compiler_context(Some(compiler.cx)),
             needs_gc_heap: false,
         })
     }
@@ -693,7 +771,9 @@ impl wasmtime_environ::Compiler for Compiler {
     fn compiled_function_relocation_targets<'a>(
         &'a self,
         func: &'a dyn Any,
-    ) -> Box<dyn Iterator<Item = RelocationTarget> + 'a> {
+    ) -> Box<dyn Iterator<Item = FuncKey> + 'a> {
+        debug_assert!(!func.is::<Option<CompilerContext>>());
+        debug_assert!(func.is::<CompiledFunction>());
         let func = func.downcast_ref::<CompiledFunction>().unwrap();
         Box::new(func.relocations().map(|r| r.reloc_target))
     }
@@ -703,8 +783,10 @@ impl InliningCompiler for Compiler {
     fn calls(
         &self,
         func_body: &CompiledFunctionBody,
-        calls: &mut wasmtime_environ::prelude::IndexSet<FuncIndex>,
+        calls: &mut wasmtime_environ::prelude::IndexSet<FuncKey>,
     ) -> Result<()> {
+        debug_assert!(!func_body.code.is::<CompiledFunction>());
+        debug_assert!(func_body.code.is::<Option<CompilerContext>>());
         let cx = func_body
             .code
             .downcast_ref::<Option<CompilerContext>>()
@@ -716,13 +798,15 @@ impl InliningCompiler for Compiler {
             func.params
                 .user_named_funcs()
                 .values()
-                .filter(|name| name.namespace == crate::NS_WASM_FUNC)
-                .map(|name| FuncIndex::from_u32(name.index)),
+                .map(|name| FuncKey::from_raw_parts(name.namespace, name.index))
+                .filter(|key| matches!(key, FuncKey::DefinedWasmFunction(_, _))),
         );
         Ok(())
     }
 
     fn size(&self, func_body: &CompiledFunctionBody) -> u32 {
+        debug_assert!(!func_body.code.is::<CompiledFunction>());
+        debug_assert!(func_body.code.is::<Option<CompilerContext>>());
         let cx = func_body
             .code
             .downcast_ref::<Option<CompilerContext>>()
@@ -737,8 +821,10 @@ impl InliningCompiler for Compiler {
     fn inline<'a>(
         &self,
         func_body: &mut CompiledFunctionBody,
-        get_callee: &'a mut dyn FnMut(FuncIndex) -> Option<&'a CompiledFunctionBody>,
+        get_callee: &'a mut dyn FnMut(FuncKey) -> Option<&'a CompiledFunctionBody>,
     ) -> Result<()> {
+        debug_assert!(!func_body.code.is::<CompiledFunction>());
+        debug_assert!(func_body.code.is::<Option<CompilerContext>>());
         let code = func_body
             .code
             .downcast_mut::<Option<CompilerContext>>()
@@ -748,7 +834,7 @@ impl InliningCompiler for Compiler {
         cx.codegen_context.inline(Inliner(get_callee))?;
         return Ok(());
 
-        struct Inliner<'a>(&'a mut dyn FnMut(FuncIndex) -> Option<&'a CompiledFunctionBody>);
+        struct Inliner<'a>(&'a mut dyn FnMut(FuncKey) -> Option<&'a CompiledFunctionBody>);
 
         impl cranelift_codegen::inline::Inline for Inliner<'_> {
             fn inline(
@@ -767,22 +853,27 @@ impl InliningCompiler for Compiler {
                     | ir::ExternalName::KnownSymbol(_) => return InlineCommand::KeepCall,
                 };
                 let callee = &caller.params.user_named_funcs()[callee];
-                let callee = if callee.namespace == crate::NS_WASM_FUNC {
-                    FuncIndex::from_u32(callee.index)
-                } else {
-                    return InlineCommand::KeepCall;
-                };
-                match (self.0)(callee) {
-                    None => InlineCommand::KeepCall,
-                    Some(func_body) => {
-                        let cx = func_body
-                            .code
-                            .downcast_ref::<Option<CompilerContext>>()
-                            .unwrap();
-                        InlineCommand::Inline(Cow::Borrowed(
-                            &cx.as_ref().unwrap().codegen_context.func,
-                        ))
-                    }
+                let callee = FuncKey::from_raw_parts(callee.namespace, callee.index);
+                match callee {
+                    FuncKey::DefinedWasmFunction(_, _) => match (self.0)(callee) {
+                        None => InlineCommand::KeepCall,
+                        Some(func_body) => {
+                            debug_assert!(!func_body.code.is::<CompiledFunction>());
+                            debug_assert!(func_body.code.is::<Option<CompilerContext>>());
+                            let cx = func_body
+                                .code
+                                .downcast_ref::<Option<CompilerContext>>()
+                                .unwrap();
+                            InlineCommand::Inline {
+                                callee: Cow::Borrowed(&cx.as_ref().unwrap().codegen_context.func),
+                                // We've already visited the callee for inlining
+                                // due to our bottom-up approach, no need to
+                                // visit it again.
+                                visit_callee: false,
+                            }
+                        }
+                    },
+                    _ => InlineCommand::KeepCall,
                 }
             }
         }
@@ -794,6 +885,9 @@ impl InliningCompiler for Compiler {
         input: Option<wasmparser::FunctionBody<'_>>,
         symbol: &str,
     ) -> Result<()> {
+        log::trace!("finish compiling {symbol:?}");
+        debug_assert!(!func_body.code.is::<CompiledFunction>());
+        debug_assert!(func_body.code.is::<Option<CompilerContext>>());
         let cx = func_body
             .code
             .downcast_mut::<Option<CompilerContext>>()
@@ -818,7 +912,7 @@ impl InliningCompiler for Compiler {
         log::debug!("`{symbol}` compiled in {:?}", timing.total());
         log::trace!("`{symbol}` timing info\n{timing}");
 
-        func_body.code = Box::new(compiled_func);
+        func_body.code = box_dyn_any_compiled_function(compiled_func);
         Ok(())
     }
 }
@@ -942,7 +1036,7 @@ impl Compiler {
         values_vec_capacity: Value,
     ) {
         debug_assert_eq!(types.len(), values.len());
-        debug_assert_enough_capacity_for_length(builder, types.len(), values_vec_capacity);
+        self.debug_assert_enough_capacity_for_length(builder, types.len(), values_vec_capacity);
 
         // Note that loads and stores are unconditionally done in the
         // little-endian format rather than the host's native-endianness,
@@ -982,7 +1076,7 @@ impl Compiler {
         let isa = &*self.isa;
         let value_size = mem::size_of::<u128>();
 
-        debug_assert_enough_capacity_for_length(builder, types.len(), values_vec_capacity);
+        self.debug_assert_enough_capacity_for_length(builder, types.len(), values_vec_capacity);
 
         // Note that this is little-endian like `store_values_to_array` above,
         // see notes there for more information.
@@ -1105,6 +1199,46 @@ impl Compiler {
     pub fn tunables(&self) -> &Tunables {
         &self.tunables
     }
+
+    fn debug_assert_enough_capacity_for_length(
+        &self,
+        builder: &mut FunctionBuilder,
+        length: usize,
+        capacity: ir::Value,
+    ) {
+        if !self.emit_debug_checks {
+            return;
+        }
+        let enough_capacity = builder.ins().icmp_imm(
+            ir::condcodes::IntCC::UnsignedGreaterThanOrEqual,
+            capacity,
+            ir::immediates::Imm64::new(length.try_into().unwrap()),
+        );
+        builder.ins().trapz(enough_capacity, TRAP_INTERNAL_ASSERT);
+    }
+
+    fn debug_assert_vmctx_kind(
+        &self,
+        builder: &mut FunctionBuilder,
+        vmctx: ir::Value,
+        expected_vmctx_magic: u32,
+    ) {
+        if !self.emit_debug_checks {
+            return;
+        }
+        let magic = builder.ins().load(
+            ir::types::I32,
+            MemFlags::trusted().with_endianness(self.isa.endianness()),
+            vmctx,
+            0,
+        );
+        let is_expected_vmctx = builder.ins().icmp_imm(
+            ir::condcodes::IntCC::Equal,
+            magic,
+            i64::from(expected_vmctx_magic),
+        );
+        builder.ins().trapz(is_expected_vmctx, TRAP_INTERNAL_ASSERT);
+    }
 }
 
 struct FunctionCompiler<'a> {
@@ -1253,18 +1387,33 @@ fn clif_to_env_stack_maps(
     }
 }
 
+/// Convert from Cranelift's representation of exception handler
+/// metadata to Wasmtime's compiler-agnostic representation.
+///
+/// Here `builder` is the wasmtime-unwinder exception section being
+/// created and `range` is the range of the function being added. The
+/// `call_sites` iterator is the raw iterator over callsite metadata
+/// (including exception handlers) from Cranelift.
+fn clif_to_env_exception_tables<'a>(
+    builder: &mut ExceptionTableBuilder,
+    range: Range<u64>,
+    call_sites: impl Iterator<Item = FinalizedMachCallSite<'a>>,
+) -> anyhow::Result<()> {
+    builder.add_func(CodeOffset::try_from(range.start).unwrap(), call_sites)
+}
+
 fn declare_and_call(
     builder: &mut FunctionBuilder,
     signature: ir::Signature,
-    func_index: u32,
+    callee_key: FuncKey,
     args: &[ir::Value],
 ) -> ir::Inst {
-    let name = ir::ExternalName::User(builder.func.declare_imported_user_function(
-        ir::UserExternalName {
-            namespace: crate::NS_WASM_FUNC,
-            index: func_index,
-        },
-    ));
+    let (namespace, index) = callee_key.into_raw_parts();
+    let name = ir::ExternalName::User(
+        builder
+            .func
+            .declare_imported_user_function(ir::UserExternalName { namespace, index }),
+    );
     let signature = builder.func.import_signature(signature);
     let callee = builder.func.dfg.ext_funcs.push(ir::ExtFuncData {
         name,
@@ -1274,43 +1423,6 @@ fn declare_and_call(
     builder.ins().call(callee, &args)
 }
 
-fn debug_assert_enough_capacity_for_length(
-    builder: &mut FunctionBuilder,
-    length: usize,
-    capacity: ir::Value,
-) {
-    if cfg!(debug_assertions) {
-        let enough_capacity = builder.ins().icmp_imm(
-            ir::condcodes::IntCC::UnsignedGreaterThanOrEqual,
-            capacity,
-            ir::immediates::Imm64::new(length.try_into().unwrap()),
-        );
-        builder.ins().trapz(enough_capacity, TRAP_INTERNAL_ASSERT);
-    }
-}
-
-fn debug_assert_vmctx_kind(
-    isa: &dyn TargetIsa,
-    builder: &mut FunctionBuilder,
-    vmctx: ir::Value,
-    expected_vmctx_magic: u32,
-) {
-    if cfg!(debug_assertions) {
-        let magic = builder.ins().load(
-            ir::types::I32,
-            MemFlags::trusted().with_endianness(isa.endianness()),
-            vmctx,
-            0,
-        );
-        let is_expected_vmctx = builder.ins().icmp_imm(
-            ir::condcodes::IntCC::Equal,
-            magic,
-            i64::from(expected_vmctx_magic),
-        );
-        builder.ins().trapz(is_expected_vmctx, TRAP_INTERNAL_ASSERT);
-    }
-}
-
 fn save_last_wasm_entry_fp(
     builder: &mut FunctionBuilder,
     pointer_type: ir::Type,
@@ -1342,25 +1454,17 @@ fn save_last_wasm_exit_fp_and_pc(
     ptr: &impl PtrSize,
     limits: Value,
 ) {
-    // Save the exit Wasm FP to the limits. We dereference the current FP to get
-    // the previous FP because the current FP is the trampoline's FP, and we
-    // want the Wasm function's FP, which is the caller of this trampoline.
+    // Save the trampoline FP to the limits. Exception unwind needs
+    // this so that it can know the SP (bottom of frame) for the very
+    // last Wasm frame.
     let trampoline_fp = builder.ins().get_frame_pointer(pointer_type);
-    let wasm_fp = builder.ins().load(
-        pointer_type,
-        MemFlags::trusted(),
-        trampoline_fp,
-        // The FP always points to the next older FP for all supported
-        // targets. See assertion in
-        // `crates/wasmtime/src/runtime/vm/traphandlers/backtrace.rs`.
-        0,
-    );
     builder.ins().store(
         MemFlags::trusted(),
-        wasm_fp,
+        trampoline_fp,
         limits,
-        ptr.vmstore_context_last_wasm_exit_fp(),
+        ptr.vmstore_context_last_wasm_exit_trampoline_fp(),
     );
+
     // Finally save the Wasm return address to the limits.
     let wasm_pc = builder.ins().get_return_address(pointer_type);
     builder.ins().store(
