@@ -9,12 +9,12 @@ use cranelift_codegen::ir::condcodes::IntCC;
 use cranelift_codegen::ir::{self, InstBuilder, MemFlags, Value};
 use cranelift_codegen::isa::{CallConv, TargetIsa};
 use cranelift_frontend::FunctionBuilder;
-use wasmtime_environ::fact::PREPARE_CALL_FIXED_PARAMS;
 use wasmtime_environ::{CompiledFunctionBody, component::*};
 use wasmtime_environ::{
     EntityRef, HostCall, ModuleInternedTypeIndex, PtrSize, TrapSentinel, Tunables, WasmFuncType,
     WasmValType,
 };
+use wasmtime_environ::{FuncKey, fact::PREPARE_CALL_FIXED_PARAMS};
 
 struct TrampolineCompiler<'a> {
     compiler: &'a Compiler,
@@ -240,6 +240,28 @@ impl<'a> TrampolineCompiler<'a> {
                     },
                 );
             }
+            Trampoline::BackpressureInc { instance } => {
+                self.translate_libcall(
+                    host::backpressure_modify,
+                    TrapSentinel::Falsy,
+                    WasmArgs::InRegisters,
+                    |me, params| {
+                        params.push(me.index_value(*instance));
+                        params.push(me.builder.ins().iconst(ir::types::I8, 1));
+                    },
+                );
+            }
+            Trampoline::BackpressureDec { instance } => {
+                self.translate_libcall(
+                    host::backpressure_modify,
+                    TrapSentinel::Falsy,
+                    WasmArgs::InRegisters,
+                    |me, params| {
+                        params.push(me.index_value(*instance));
+                        params.push(me.builder.ins().iconst(ir::types::I8, 0));
+                    },
+                );
+            }
             Trampoline::TaskReturn { results, options } => {
                 self.translate_libcall(
                     host::task_return,
@@ -311,13 +333,17 @@ impl<'a> TrampolineCompiler<'a> {
                     },
                 );
             }
-            Trampoline::Yield { async_ } => {
+            Trampoline::ThreadYield { cancellable } => {
                 self.translate_libcall(
-                    host::yield_,
+                    host::thread_yield,
                     TrapSentinel::NegativeOne,
                     WasmArgs::InRegisters,
                     |me, params| {
-                        params.push(me.builder.ins().iconst(ir::types::I8, i64::from(*async_)));
+                        params.push(
+                            me.builder
+                                .ins()
+                                .iconst(ir::types::I8, i64::from(*cancellable)),
+                        );
                     },
                 );
             }
@@ -772,7 +798,7 @@ impl<'a> TrampolineCompiler<'a> {
         )
     }
 
-    /// Translates an invokation of a host function and interpret the result.
+    /// Translates an invocation of a host function and interpret the result.
     ///
     /// This is intended to be a relatively narrow waist which most intrinsics
     /// go through. The configuration supported here is:
@@ -1060,7 +1086,7 @@ impl<'a> TrampolineCompiler<'a> {
                 vmctx,
                 i32::try_from(self.offsets.resource_destructor(index)).unwrap(),
             );
-            if cfg!(debug_assertions) {
+            if self.compiler.emit_debug_checks {
                 self.builder
                     .ins()
                     .trapz(dtor_func_ref, TRAP_INTERNAL_ASSERT);
@@ -1281,7 +1307,7 @@ impl ComponentCompiler for Compiler {
         &self,
         component: &ComponentTranslation,
         types: &ComponentTypesBuilder,
-        index: TrampolineIndex,
+        key: FuncKey,
         tunables: &Tunables,
         _symbol: &str,
     ) -> Result<AllCallFunc<CompiledFunctionBody>> {
@@ -1292,7 +1318,7 @@ impl ComponentCompiler for Compiler {
                 &mut compiler,
                 &component.component,
                 types,
-                index,
+                key.unwrap_component_trampoline(),
                 abi,
                 tunables,
             );
@@ -1304,8 +1330,7 @@ impl ComponentCompiler for Compiler {
             // always hold.
             let vmctx = c.builder.block_params(c.block0)[0];
             let pointer_type = self.isa.pointer_type();
-            super::debug_assert_vmctx_kind(
-                &*self.isa,
+            self.debug_assert_vmctx_kind(
                 &mut c.builder,
                 vmctx,
                 wasmtime_environ::component::VMCOMPONENT_MAGIC,
@@ -1325,12 +1350,12 @@ impl ComponentCompiler for Compiler {
                 );
             }
 
-            c.translate(&component.trampolines[index]);
+            c.translate(&component.trampolines[key.unwrap_component_trampoline()]);
             c.builder.finalize();
             compiler.cx.abi = Some(abi);
 
             Ok(CompiledFunctionBody {
-                code: Box::new(Some(compiler.cx)),
+                code: super::box_dyn_any_compiler_context(Some(compiler.cx)),
                 needs_gc_heap: false,
             })
         };
